// Eye.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "resource.h"
#include "..\_3D_SRC\DeRender.h"
#include <commdlg.h>
#include <windowsx.h>
#include <winuser.h>

#ifndef WM_MOUSEWHEEL
	#define WM_MOUSEWHEEL                   0x020A
#endif

#ifndef SM_MOUSEWHEELPRESENT
	#define SM_MOUSEWHEELPRESENT    75
#endif


#define MAX_LOADSTRING 100

// Minimalni rozmery okna
#define MIN_WIDTH		200
#define MIN_HEIGHT		200

// Global Variables:
HINSTANCE ghInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];								// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];								// The title bar text
bool gfShowToolbox; // Jestli je toolbox zobrazen
HWND ghWndMain; // Handle na hlavni okno
HWND ghWndToolbox; // Handle na okno toolboxu
char gszLastOpenedFile[MAX_PATH]; // Posledni soubor otevreny z menu File | Open...

ViewParams ViewParameters;
RenderedPicture Picture; // Posledni vyrenderovany obrazek
RenderScene RScene;
DeRenderScene DRScene;




// Foward declarations of functions included in this code module:
ATOM				MyRegisterClass();
BOOL				InitInstance(int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK ToolboxWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);
bool OpenTheFile();
bool SaveTheFile();
bool ApplyChangesAndRenderScene(unsigned short Command, signed short Param1, signed short Param2);
bool ProcessCommandLineParams();
void PrintLogo();

// Makra jakozto parametry funkce ApplyChangesAndRenderScene()
#define CMD_STARTUP		(0)
#define CMD_APPLY		(1)
#define CMD_FORWARDS	(2)
#define CMD_BACKWARDS	(3)
#define CMD_LEFT		(4)
#define CMD_RIGHT		(5)
#define CMD_UP			(6)
#define CMD_DOWN		(7)
#define CMD_LOOKUP		(8)
#define CMD_LOOKDOWN	(9)
#define CMD_LOOKLEFT	(10)
#define CMD_LOOKRIGHT	(11)
#define CMD_ZOOMINC		(12)
#define CMD_ZOOMDEC		(13)
#define CMD_ROTATE		(14) // Pokud je uzit tento parametr (jakozto Command), Params1 a Params2 obsahuji
						     // relativni zmenu x- a y-ove souradnice kursoru mysi, podle niz se vypocita
						     // velikost a smer rotace.
#define CMD_MASTER		(15)
#define CMD_SLAVE		(16)



void ToolboxShow(bool fShow);
bool DrawRenderedFrame(HWND hWnd);






#define MyCheckMenuItem(hMenu, wIDCheckItem, fCheck) \
    CheckMenuItem((hMenu),(wIDCheckItem),(fCheck)?MF_CHECKED:MF_UNCHECKED)


// Posledni chyby
unsigned int gLastError(0);
#define R_SUCCESS				0
#define R_TOOLBOXFORMAT_ERROR	1
#define R_LOWMEMORY_ERROR		2
#define R_RENDERING_ERROR		3
#define R_OPENDRS_ERROR			4
#define R_SOLVEDRS_ERROR		5
#define R_OPENRSF_ERROR			6
#define R_OPENFILE_ERROR		7
#define R_SAVERSF_ERROR			8
#define R_COMMANDLINE_ERROR     9


// Procedura ErrorMsgBox zobrazi modalni message-box s posledni nastalou chybou
char gErrorString[256], gErrorTitle[30]; // Pozdeji jiz nemusi byt na text s chybou pamet
#define gErrorStringLen		(sizeof(gErrorString)/sizeof(gErrorString[0]))
void ErrorMsgBox()
{		
	switch(gLastError)
	{
		case R_SUCCESS: 
			strcpy(gErrorString, "Command completed successfully."); break;			
		case R_TOOLBOXFORMAT_ERROR:
			strcpy(gErrorString, "All edit boxes in toolbox must contain real numbers (for example '14', '58.3', '-15.34e-5', ...).\n'VP distance' must be greater than zero. Decimal separator is dot ('.')"); break;
		case R_LOWMEMORY_ERROR:
			strcpy(gErrorString, "Insufficient memory."); break;
		case R_RENDERING_ERROR:
			strcpy(gErrorString, "Cannot render scene."); break;
		case R_OPENDRS_ERROR:
			strcpy(gErrorString, "Cannot open/read the DRS file, possible incorrect format."); break;
		case R_SOLVEDRS_ERROR:
			strcpy(gErrorString, "Cannot derender the scene, possible incorrect values in DRS file."); break;
		case R_OPENRSF_ERROR:
			strcpy(gErrorString, "Cannot open/read the RSF file. possible incorrect format."); break;
		case R_OPENFILE_ERROR:
			strcpy(gErrorString, "Cannot open specified file."); break;
		case R_SAVERSF_ERROR:
			strcpy(gErrorString, "Cannot save RSF file."); break;
		case R_COMMANDLINE_ERROR:
			strcpy(gErrorString, "Unknown command-line options."); break;
		default:
			strcpy(gErrorString, "Unknown error."); break;
	}

	strcpy(gErrorTitle, "Render error...");	
	gLastError = R_SUCCESS;

	MessageBox(NULL, gErrorString, gErrorTitle, MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL);
}


#define COMMAND_LINE_ARGS_STRING	"Eye.exe -o FILE.RSF\n\tOpens and shows Render Scene File FILE.RSF  \n\nEye.exe -d FILE.DRS\n\tSolves and shows DeRender Scene file FILE.DRS  "

bool ProcessCommandLineParams()
{
	CloseDeRenderScene( &DRScene );	
	CloseRenderScene( &RScene );
	DestroyRenderedPicture(&Picture);


	switch( __argc )
	{
		case 1:
			// "eye.exe"
			return( true );
	
		case 3:
			// "eye.exe -o filename"
			if( 0 == strcmp("-o", __argv[1]) )
			{
				if( !OpenRenderScene( __argv[2], &RScene ) )
				{
					gLastError = R_OPENRSF_ERROR;
					return( false );
				}
				ApplyChangesAndRenderScene(CMD_STARTUP, 0, 0);
				return( true );
			}
			
			// "eye.exe -d filename"
			if( 0 == strcmp("-d", __argv[1]) )
			{
				if( !OpenDeRenderScene(__argv[2], &DRScene) )
				{
					gLastError = R_OPENDRS_ERROR;
					return( false );
				}
				
				if( !SolveDeRenderScene(DRScene, &RScene) )
				{	
					CloseDeRenderScene(&DRScene);
					gLastError = R_SOLVEDRS_ERROR;
					return( false );
				}

				CloseDeRenderScene(&DRScene);
				ApplyChangesAndRenderScene(CMD_STARTUP, 0, 0);
				return( true );
			}

		default:
			// chybicka se vloudila
			gLastError = R_COMMANDLINE_ERROR;
			return( false );
	}

}







int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_RENDER, szWindowClass, MAX_LOADSTRING);
	
	ghInst = hInstance;
	if( !MyRegisterClass() )
		return FALSE;

	// Perform application initialization:
	if( !InitInstance (nCmdShow) ) 
		return FALSE;

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_RENDER);

	if( !ProcessCommandLineParams() )
	{
		ErrorMsgBox();
		return 0;
	}

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return msg.wParam;
}



//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage is only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass()
{
	WNDCLASSEX wcex;
	
	wcex.cbSize = sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= ghInst;
	wcex.hIcon			= LoadIcon(ghInst, (LPCTSTR)IDI_RENDER);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOWTEXT+1); //(HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCSTR)IDC_RENDER;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);
	
	return( RegisterClassEx(&wcex) );
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(int nCmdShow)
{
	gfShowToolbox = false;
	
	ghWndMain = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
	                         CW_USEDEFAULT, 0, 640, 480, NULL, NULL, ghInst, NULL);

	if (!ghWndMain)
	{
		return( FALSE );
	}

	ghWndToolbox = CreateDialog(ghInst, MAKEINTRESOURCE(IDD_TOOLBOX), ghWndMain, 
	                            (DLGPROC)ToolboxWndProc);    	

	if (!ghWndToolbox)
	{
		DestroyWindow(ghWndMain);
		return FALSE;
	}

	strcpy(gszLastOpenedFile, "\0");


	ShowWindow(ghWndMain, nCmdShow);
	UpdateWindow(ghWndMain);
	
	ToolboxShow(gfShowToolbox);	


	return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	//TCHAR szHello[MAX_LOADSTRING];
	//LoadString(ghInst, IDS_HELLO, szHello, MAX_LOADSTRING);

	static int LClickPosX(0), LClickPosY(0);
	static int MouseMovesCounter(0); // Pocitadlo, kolikrat bylo pohnuto mysi
	static int fHasWheel(FALSE);

	const int MaxMouseMovesCount = 2; // Po kolika posunech mysi se bude renderovat novy frame

	
	switch (message) 
	{
		case WM_CREATE:		  
            fHasWheel = GetSystemMetrics( SM_MOUSEWHEELPRESENT );

		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 

			// Aby se pri stisknuti tlacitka mysi v nektere z nabidek menu hned nezmenil pohled
			MouseMovesCounter = 0;

			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_ABOUT:
				   MessageBox(NULL, "Eye version 1.00\nCopyright(c) 2003 Jan Curn\nJan.Curn@seznam.cz", "About...", MB_OK | MB_TASKMODAL);				   
				   break;

				case IDM_COMMAND_LINE_ARGS:
				   MessageBox(NULL, COMMAND_LINE_ARGS_STRING, "Command-line options", MB_OK | MB_TASKMODAL);				   
				   break;

				case IDM_EXIT:
				   DestroyWindow(hWnd);
				   break;

				case IDM_FILE_OPEN:
					// Vypneme toolbox - openfile dialog neni vuci nemu modalni
					ToolboxShow(false);
					// Otevreme DRS/RSF soubor
					if( !OpenTheFile() )
						ErrorMsgBox();					
					// Toolbox vratime do puvodniho stavu					
					ToolboxShow(gfShowToolbox);						
					break;

				case IDM_FILE_SAVE:
					ToolboxShow(false);
					if( !SaveTheFile() )
						ErrorMsgBox();
					ToolboxShow(gfShowToolbox);	
					break;

				case IDM_VIEWTOOLBOX:
					gfShowToolbox = gfShowToolbox ? false : true;
					ToolboxShow(gfShowToolbox);		   
					break;

				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;

		case WM_INITMENU:
			if (GetMenu(ghWndMain) == (HMENU)wParam)
				MyCheckMenuItem((HMENU)wParam, IDM_VIEWTOOLBOX, gfShowToolbox);				
			break;
		
		case WM_PAINT:
			// Vykreslime posledni vyrenderovany frame			
			DrawRenderedFrame(ghWndMain);
			break;

		
		case WM_LBUTTONDOWN:
			// Bylo stisknuto leve tlacitko mysi
			// Ulozime souradnice kurzoru
			LClickPosX = (short)LOWORD(lParam);
			LClickPosY = (short)HIWORD(lParam); 

			// A provedeme nulovou rotaci, to znamena ze rotace se neprovede, pouze se pohled (presneji
			// vertikalni a horizontalni uhel pohledu) namiri na bod, kolem ktereho se rotuje.
			if( !ApplyChangesAndRenderScene( CMD_ROTATE, 0, 0 ) )
				ErrorMsgBox();
			                            
			break;

		case WM_MOUSEWHEEL:
			// Zmena zoomu pri otoceni kolecka mysi
			if( !fHasWheel )
				break;

			if( (short)HIWORD(wParam) > 0 )
				if( !ApplyChangesAndRenderScene(CMD_ZOOMINC, 0, 0) )
					ErrorMsgBox();

			if( (short)HIWORD(wParam) < 0 )			
				if( !ApplyChangesAndRenderScene(CMD_ZOOMDEC, 0, 0) )
					ErrorMsgBox();
			break;

		case WM_MOUSEMOVE:
			// Dal pokracujeme jen pokud je stiskunto leve tlacitko mysi
			if( !(wParam & MK_LBUTTON) )
				break;

			MouseMovesCounter++;

			// Pokracujeme dal jen po MaxMouseMovesCount zmenach polohy mysi, aby se scena neustale 
			// nerenderovala, to by bylo prilis narocne na vykon
			if( MouseMovesCounter <= MaxMouseMovesCount )
				break;

			MouseMovesCounter = 0;

			// A provedeme rotaci zavisejici na relativni zmene polohy kursoru mysi od posledni rotace
			if( !ApplyChangesAndRenderScene( CMD_ROTATE, 
			                                 (short)LOWORD(lParam) - LClickPosX, 
			                                 (short)HIWORD(lParam) - LClickPosY ) )
				ErrorMsgBox();

			// A ulozime aktualni souradnici kurzoru
			LClickPosX = (short)LOWORD(lParam);
			LClickPosY = (short)HIWORD(lParam); 

			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}



// Funkce zobrazi/skryje toolbox
void ToolboxShow(bool fShow)
{
	if( ghWndToolbox )
		if( fShow )
			ShowWindow(ghWndToolbox, SW_SHOWNA);
		else
			ShowWindow(ghWndToolbox, SW_HIDE);  
}


// Funkce provede zmeny pohledu, ty ulozi do globalni promenne ViewParameters, vyrenderuje novy obraz sceny
// a ten vykresli do hlavniho okna. Command urcuje typ zmeny, ktera se ma provest, Params1 a Params2 jsou
// pripadne parametry pro dany Command - viz. nahore definici maker CMD_...
bool ApplyChangesAndRenderScene(unsigned short Command, signed short Param1, signed short Param2)
{
	RECT Rect;
	char TextString[200]; 
	float FloatVal;
	double VerAngleInc, HorAngleInc;

	// Tyto konstanty lze menit, ovlivnuji jak moc se meni hodnoty v toolboxu pri stisknuti daneho tlacitka
	const double MoveLength = 0.3, 
	             TurnAngle = 0.05,
	             RotationRatio = 6,
	             ZoomIncCoeff = 1.2,
	             ZoomDecCoeff = 0.83;

	// Toto cele ma smysl jen pokud je otevrena nejaka RSF scena
	if( !RScene.Points )
		return( true ); // Aby se nespustil message box s chybou


	// Do ViewParams ulozime aktualni stav z toolboxu a z hlavniho okna
	GetClientRect(ghWndMain, &Rect);
	ViewParameters.ResX = Rect.right;
	ViewParameters.ResY = Rect.bottom;
		
	if( Command==CMD_STARTUP )
	{
		// Inicializace pri prvni pouziti
		ViewParameters.DrawPoints = true;
		ViewParameters.DrawTrianTextures = true;
		ViewParameters.DrawTrianBorders = false;
		ViewParameters.DontDrawTriangles = false;
		ViewParameters.HorViewAngle = 0;
		ViewParameters.VerViewAngle = 0;
		ViewParameters.ViewPlaneDistance = 0.001 + fabs(ViewParameters.ResX) * 1.732; // Zorny uhel (zoom) nastavime cca na 60 stupnu, podle tohoto predpisu vzdy kladny
		ViewParameters.ViewPoint.X = ViewParameters.ViewPoint.Y = ViewParameters.ViewPoint.Z = 0;
		ViewParameters.BgColor = 0; // Cerna barva pozadi

		// Nastavime checkboxy a radiobuttony podle predchoziho nastaveni ViewParameters
		Button_SetCheck(GetDlgItem(ghWndToolbox, IDC_DRAWPOINTS_CHECKBOX), ViewParameters.DrawPoints?BST_CHECKED:BST_UNCHECKED);
		Button_SetCheck(GetDlgItem(ghWndToolbox, IDC_TRIANTEXTURES_RADIO), ViewParameters.DrawTrianTextures?BST_CHECKED:BST_UNCHECKED);
		Button_SetCheck(GetDlgItem(ghWndToolbox, IDC_TRIANBORDER_RADIO), ViewParameters.DrawTrianBorders?BST_CHECKED:BST_UNCHECKED);
		Button_SetCheck(GetDlgItem(ghWndToolbox, IDC_NOTRIAN_RADIO), ViewParameters.DontDrawTriangles?BST_CHECKED:BST_UNCHECKED);
	
	}
	else
		if( Command==CMD_ROTATE )
		{
			// Rotace pretazenim mysi pres obrazovku
			VerAngleInc = (double)Param2 * PI / ViewParameters.ResY;
			HorAngleInc = -(double)Param1 * 2 * PI / ViewParameters.ResX;

			RotateViewAroundPoint(&ViewParameters, RScene.Points[0].Position, HorAngleInc, VerAngleInc);
		}
		else
		{	
			// Prisel nejaky prikaz z toolboxu
			// Nacteme z edit boxu ciselne hodnoty
			GetDlgItemText(ghWndToolbox, IDC_CAMX_EDIT, TextString, sizeof(TextString));
			if(TextToFloat(&FloatVal, TextString))
				ViewParameters.ViewPoint.X = FloatVal;
			else 
				{ gLastError=R_TOOLBOXFORMAT_ERROR; return(false); }	
	
			GetDlgItemText(ghWndToolbox, IDC_CAMY_EDIT, TextString, sizeof(TextString));
			if(TextToFloat(&FloatVal, TextString))
				ViewParameters.ViewPoint.Y = FloatVal;
			else
				{ gLastError=R_TOOLBOXFORMAT_ERROR; return(false); }

			GetDlgItemText(ghWndToolbox, IDC_CAMZ_EDIT, TextString, sizeof(TextString));
			if(TextToFloat(&FloatVal, TextString))
				ViewParameters.ViewPoint.Z = FloatVal;
			else
				{ gLastError=R_TOOLBOXFORMAT_ERROR; return(false); }

			GetDlgItemText(ghWndToolbox, IDC_VERANGLE_EDIT, TextString, sizeof(TextString));
			if(TextToFloat(&FloatVal, TextString))
				ViewParameters.VerViewAngle = FloatVal;
			else
				{ gLastError=R_TOOLBOXFORMAT_ERROR; return(false); }

			GetDlgItemText(ghWndToolbox, IDC_HORANGLE_EDIT, TextString, sizeof(TextString));
			if(TextToFloat(&FloatVal, TextString))
				ViewParameters.HorViewAngle = FloatVal;
			else
				{ gLastError=R_TOOLBOXFORMAT_ERROR; return(false); }

			GetDlgItemText(ghWndToolbox, IDC_VPDISTANCE_EDIT, TextString, sizeof(TextString));
			if(TextToFloat(&FloatVal, TextString))
			{
				// Tato polozka musi byt vzdy kladna
				if(FloatVal <= 0)
					{ gLastError=R_TOOLBOXFORMAT_ERROR; return(false); }
				ViewParameters.ViewPlaneDistance = FloatVal;
			}
			else
				{ gLastError=R_TOOLBOXFORMAT_ERROR; return(false); }
		
			// Zjistime stav checkboxu a radiobuttonu
			ViewParameters.DrawPoints = Button_GetCheck(GetDlgItem(ghWndToolbox, IDC_DRAWPOINTS_CHECKBOX));
			
			ViewParameters.DrawTrianBorders = Button_GetCheck(GetDlgItem(ghWndToolbox, IDC_TRIANBORDER_RADIO));
			ViewParameters.DrawTrianTextures = Button_GetCheck(GetDlgItem(ghWndToolbox, IDC_TRIANTEXTURES_RADIO));
			ViewParameters.DontDrawTriangles = Button_GetCheck(GetDlgItem(ghWndToolbox, IDC_NOTRIAN_RADIO));

			// Kdyby nahodou byly nastaveny dva radio buttony najednou, toto se s tim vyporada
			if( ViewParameters.DrawTrianBorders )
				ViewParameters.DrawTrianTextures = ViewParameters.DontDrawTriangles = false;
			else
			{
				if( ViewParameters.DrawTrianTextures )
					ViewParameters.DrawTrianBorders = ViewParameters.DontDrawTriangles = false;
				else
					ViewParameters.DontDrawTriangles = true;
			}
					

		}


	// Provedeme zmeny ViewParameters podle toho, ktere tlacitko bylo stisknuto
	switch(Command)
	{
	    case CMD_FORWARDS:
			ViewParameters.ViewPoint.Z += MoveLength; break;
		case CMD_BACKWARDS:
			ViewParameters.ViewPoint.Z -= MoveLength; break;
		case CMD_LEFT:
			ViewParameters.ViewPoint.X -= MoveLength; break;
		case CMD_RIGHT:
			ViewParameters.ViewPoint.X += MoveLength; break;
		case CMD_UP:
			ViewParameters.ViewPoint.Y += MoveLength; break;
		case CMD_DOWN:
			ViewParameters.ViewPoint.Y -= MoveLength; break;
		case CMD_LOOKUP:
			ViewParameters.VerViewAngle -= TurnAngle; break;
		case CMD_LOOKDOWN:
			ViewParameters.VerViewAngle += TurnAngle; break;
		case CMD_LOOKLEFT:
			ViewParameters.HorViewAngle += TurnAngle; break;
		case CMD_LOOKRIGHT:
			ViewParameters.HorViewAngle -= TurnAngle; break;
		case CMD_ZOOMINC:
			ViewParameters.ViewPlaneDistance *= ZoomIncCoeff; break;
		case CMD_ZOOMDEC:
			ViewParameters.ViewPlaneDistance *= ZoomDecCoeff; break;
		case CMD_MASTER:
			ViewParameters.ViewPoint = RScene.MasterCameraPosition;
			RotateViewAroundPoint(&ViewParameters, RScene.Points[0].Position, 0, 0);
			break;
		case CMD_SLAVE:
			ViewParameters.ViewPoint = RScene.SlaveCameraPosition; 
			RotateViewAroundPoint(&ViewParameters, RScene.Points[0].Position, 0, 0);
			break;
		default: // Zadna zmena
			break;
	}


	// Upravime editboxy podle ViewParameters
	sprintf(TextString, "%f", ViewParameters.ViewPoint.X);
	SetDlgItemText(ghWndToolbox, IDC_CAMX_EDIT, TextString); 
	sprintf(TextString, "%f", ViewParameters.ViewPoint.Y);
	SetDlgItemText(ghWndToolbox, IDC_CAMY_EDIT, TextString); 
	sprintf(TextString, "%f", ViewParameters.ViewPoint.Z);
	SetDlgItemText(ghWndToolbox, IDC_CAMZ_EDIT, TextString); 

	sprintf(TextString, "%f", ViewParameters.HorViewAngle);
	SetDlgItemText(ghWndToolbox, IDC_HORANGLE_EDIT, TextString); 
	sprintf(TextString, "%f", ViewParameters.VerViewAngle);
	SetDlgItemText(ghWndToolbox, IDC_VERANGLE_EDIT, TextString); 

	sprintf(TextString, "%f", ViewParameters.ViewPlaneDistance);
	SetDlgItemText(ghWndToolbox, IDC_VPDISTANCE_EDIT, TextString); 

	// a jeste checkbox a radiobuttony
	Button_SetCheck(GetDlgItem(ghWndToolbox, IDC_DRAWPOINTS_CHECKBOX), ViewParameters.DrawPoints?BST_CHECKED:BST_UNCHECKED);
	if(ViewParameters.DrawTrianTextures)
		Button_SetCheck(GetDlgItem(ghWndToolbox, IDC_TRIANTEXTURES_RADIO), BST_CHECKED);
	else
		if(ViewParameters.DrawTrianBorders)
			Button_SetCheck(GetDlgItem(ghWndToolbox, IDC_TRIANBORDER_RADIO), BST_CHECKED);
		else
			Button_SetCheck(GetDlgItem(ghWndToolbox, IDC_NOTRIAN_RADIO), BST_CHECKED);


	// Hlavni okno se brzy prekresli
	InvalidateRect(ghWndMain, NULL, FALSE);

	// Pokud scena neexistuje nebo pokud je hlavni okno tak male, ze se na nej neda nic
	// vykreslovat, znicime puvodni obraz a vyskocime z funkce.
	if( !(RScene.Points || RScene.Triangles) || (ViewParameters.ResX==0) || (ViewParameters.ResY==0) )
	{
		DestroyRenderedPicture(&Picture);
		return( true );
	}
	
	// Vytvorime novy obraz pro vysledky renderingu, puvodni obraz bude znicen
	if( !CreateRenderedPicture(&Picture, ViewParameters) )		
	{ 
		gLastError=R_LOWMEMORY_ERROR; 
		return( false ); 
	}	

	if( !RenderWholeScene(RScene, ViewParameters, &Picture) )
	{
		DestroyRenderedPicture(&Picture);
		gLastError = R_RENDERING_ERROR;
		return( false );
	}


	return( true );
}




//
//   FUNCTION: OpenTheFile(HWND hwnd, HWND hwndEdit)
//
//   PURPOSE: Invokes common dialog function to open a file and opens it.
//
//   COMMENTS:
//
//	This function initializes the OPENFILENAME structure and calls
//            the GetOpenFileName() common dialog function.  
//	
//    RETURN VALUES:
//        TRUE - The file was opened successfully and read into the buffer.
//        FALSE - No files were opened.
//
//
bool OpenTheFile()
{
   	OPENFILENAME OpenFileName;
	char szFileName[MAX_PATH];

	strcpy(szFileName, gszLastOpenedFile);
	
	OpenFileName.lStructSize       = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner         = ghWndMain;
    OpenFileName.hInstance         = NULL;
    OpenFileName.lpstrFilter       = "Render/DeRender Scene Files\0*.rsf;*.drs\0\0";
    OpenFileName.lpstrCustomFilter = NULL;
    OpenFileName.nMaxCustFilter    = 0;
    OpenFileName.nFilterIndex      = 0;
    OpenFileName.lpstrFile         = szFileName;
    OpenFileName.nMaxFile          = sizeof(szFileName) - 1;
    OpenFileName.lpstrFileTitle    = NULL;
    OpenFileName.nMaxFileTitle     = 0;
    OpenFileName.lpstrInitialDir   = NULL;
    OpenFileName.lpstrTitle        = "Open a File";
    OpenFileName.nFileOffset       = 0;
    OpenFileName.nFileExtension    = 0;
    OpenFileName.lpstrDefExt       = NULL;
    OpenFileName.lCustData         = NULL;
	OpenFileName.lpfnHook 		   = NULL;
	OpenFileName.lpTemplateName    = "";
    OpenFileName.Flags             = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_PATHMUSTEXIST;

	// Call the common dialog function.
    if (!GetOpenFileName(&OpenFileName))
		return( true );	// Aby se nezobrazil message box s chybou

	CloseDeRenderScene( &DRScene );	
	CloseRenderScene( &RScene );

	// Deaktivujeme polozku menu File | Save...
	EnableMenuItem(GetMenu(ghWndMain), IDM_FILE_SAVE, MF_GRAYED);

	DestroyRenderedPicture(&Picture);

	char *FileExtension = (char *)OpenFileName.lpstrFile + OpenFileName.nFileExtension;
	strupr(FileExtension);	

	strcpy(gszLastOpenedFile, OpenFileName.lpstrFile);
	
	// Otevreme DRS soubor a prevedem ho na render scenu (derenderujeme ho)
	if( !strcmp(FileExtension, "DRS") )
	{
		if( !OpenDeRenderScene(OpenFileName.lpstrFile, &DRScene) )
		{
			gLastError = R_OPENDRS_ERROR;
			return( false );
		}
		
		if( !SolveDeRenderScene(DRScene, &RScene) )
		{	
			CloseDeRenderScene(&DRScene);
			gLastError = R_SOLVEDRS_ERROR;
			return( false );
		}

		CloseDeRenderScene(&DRScene);
	}
	else
	{
		if( !strcmp(FileExtension, "RSF") )
		{
			if( !OpenRenderScene(OpenFileName.lpstrFile, &RScene) )
			{
				gLastError = R_OPENRSF_ERROR;
				return( false );
			}
		}
		else
		{
			gLastError = R_OPENFILE_ERROR;
			return( false );
		}
	}

	// Aktivujeme polozku menu File | Save...
	EnableMenuItem(GetMenu(ghWndMain), IDM_FILE_SAVE, MF_ENABLED);

	ApplyChangesAndRenderScene(CMD_STARTUP, 0, 0);
	
	return( true );
	
}


// Funkce spusti save as dialog a ulozi otevrenou render-scenu do souboru
bool SaveTheFile()
{
	OPENFILENAME OpenFileName;
	char szFileName[MAX_PATH];
	char *ExtensionPtr;

	// Scena k ulozeni tu neni ?
	if( !RScene.Points )
	{
		// Deaktivujeme polozku menu File | Save..., pokud uz takova neni
		EnableMenuItem(GetMenu(ghWndMain), IDM_FILE_SAVE, MF_GRAYED);
		return( true );
	}

	// Zaridime, aby uzivateli bylo nabidnuto jmeno souboru stejne, jako u .DRS, jen s priponou .RSF
	strcpy(szFileName, gszLastOpenedFile);

	// Zmenime priponu souboru na .rsf
	ExtensionPtr = strrchr(szFileName, '.');
	if( !ExtensionPtr )
		szFileName[0] = NULL;
	else
		strcpy(ExtensionPtr, ".rsf");

	
	OpenFileName.lStructSize       = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner         = ghWndMain;
    OpenFileName.hInstance         = NULL;
    OpenFileName.lpstrFilter       = "Render Scene Files\0*.rsf\0\0";
    OpenFileName.lpstrCustomFilter = NULL;
    OpenFileName.nMaxCustFilter    = 0;
    OpenFileName.nFilterIndex      = 0;
    OpenFileName.lpstrFile         = szFileName;
    OpenFileName.nMaxFile          = sizeof(szFileName)/sizeof(szFileName[0]) - 1;
    OpenFileName.lpstrFileTitle    = NULL;
    OpenFileName.nMaxFileTitle     = 0;
    OpenFileName.lpstrInitialDir   = NULL;
    OpenFileName.lpstrTitle        = "Save the File As";
    OpenFileName.nFileOffset       = 0;
    OpenFileName.nFileExtension    = 0;
    OpenFileName.lpstrDefExt       = "rsf";
    OpenFileName.lCustData         = NULL;
	OpenFileName.lpfnHook 		   = NULL;
	OpenFileName.lpTemplateName    = "";
    OpenFileName.Flags             = OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_OVERWRITEPROMPT;
	
	if( !GetSaveFileName( &OpenFileName ) )
		return( true ); // Aby se nezobrazil message box s chybou

	if( !SaveRenderScene(OpenFileName.lpstrFile, RScene) )
	{
		gLastError = R_OPENRSF_ERROR;
		return( false );	
	}

	return( true );
}

 
LRESULT CALLBACK ToolboxWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	bool ViewParamsChanged = false;
	int Command = -1;
	
	switch (message) 
	{
		case WM_INITDIALOG:
			return TRUE;

		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 			
			switch(wmId)
			{
				case IDC_APPLY_BUTTON:
					Command = CMD_APPLY; break;
				case IDC_FORWARDS_BUTTON:
					Command = CMD_FORWARDS; break;
				case IDC_BACKWARDS_BUTTON:
					Command = CMD_BACKWARDS; break;
				case IDC_LEFT_BUTTON:
					Command = CMD_LEFT; break;
				case IDC_RIGHT_BUTTON:
					Command = CMD_RIGHT; break;
				case IDC_UP_BUTTON:
					Command = CMD_UP; break;
				case IDC_DOWN_BUTTON:
					Command = CMD_DOWN; break;
				case IDC_LOOKUP_BUTTON:
					Command = CMD_LOOKUP; break;
				case IDC_LOOKDOWN_BUTTON:
					Command = CMD_LOOKDOWN; break;
				case IDC_LOOKLEFT_BUTTON:
					Command = CMD_LOOKLEFT; break;
				case IDC_LOOKRIGHT_BUTTON:
					Command = CMD_LOOKRIGHT; break;
				case IDC_ZOOMINC_BUTTON:
					Command = CMD_ZOOMINC; break;
				case IDC_ZOOMDEC_BUTTON:
					Command = CMD_ZOOMDEC; break;				
				case IDC_LOOKTOOBJ_BUTTON:
					Command = CMD_ROTATE; break; // Rotace o 0,0 znamena primy pohled na scenu (na 1. bod sceny)
				case IDC_MASTER_BUTTON:
					Command = CMD_MASTER; break;
				case IDC_SLAVE_BUTTON:
					Command = CMD_SLAVE; break;
			}
			break;


		case WM_CLOSE:
			gfShowToolbox = FALSE;
			ToolboxShow(gfShowToolbox);
			break;
			
		case WM_DESTROY:
			ghWndToolbox = NULL;
			gfShowToolbox = FALSE;
			break;
	}
    
	if(Command != (-1))
		if( !ApplyChangesAndRenderScene(Command, 0, 0) )
			ErrorMsgBox();	

	return 0;
}




bool DrawRenderedFrame(HWND hWnd)
{
	BITMAPINFO Info;
	HDC hdc;
	PAINTSTRUCT ps;	

	Info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	Info.bmiHeader.biWidth = Picture.Resolution.X;
	Info.bmiHeader.biHeight = Picture.Resolution.Y;
	Info.bmiHeader.biPlanes = 1;
	Info.bmiHeader.biBitCount = 32;
	Info.bmiHeader.biCompression = BI_RGB;
	Info.bmiHeader.biSizeImage = 0;
	Info.bmiHeader.biXPelsPerMeter = 4000; // Zhruba 100 dpi
	Info.bmiHeader.biYPelsPerMeter = 4000; 
	Info.bmiHeader.biClrUsed = 0;
	Info.bmiHeader.biClrImportant = 0;
	//Info.bmiColors = ???

	hdc = BeginPaint(hWnd, &ps);
	if(Picture.Pixels)
		SetDIBitsToDevice(hdc, 0, 0, Picture.Resolution.X, Picture.Resolution.Y, 0, 0,
						  0, Picture.Resolution.Y, Picture.Pixels, &Info, DIB_RGB_COLORS);	 
	EndPaint(hWnd, &ps);				
	
	return( true );

}