

#include <malloc.h>
#include <memory.h>
#include <math.h>
#include <stdlib.h>
#include "Render.h"
#include "MyProcs.h"
#include "DeRendAI.h"




bool CutTwoPhotos(Point2D MasStartPos, const Camera &MasCam, const Camera &SlaCam, GeneralPhotosCut *pCut)
{
	return( CutTwoPhotosInline(MasStartPos, MasCam, SlaCam, pCut) );
}



unsigned int __stdcall GetSimilarityRGB_1x1( int x, int y, const Color32 *pMasterLine, const Color32 *pSlaveLine )
{
	return( GetSimilarityRGB(pMasterLine+x, pSlaveLine+y) );
}

unsigned int __stdcall GetSimilarityRGB_3x3( int x, int y, const Color32 *pMasterLine, const Color32 *pSlaveLine )
{
	int RetVal =
		  GetSimilarityRGB( &pMasterLine[x-1], &pSlaveLine[y-1] )
 		+ GetSimilarityRGB( &pMasterLine[x-1], &pSlaveLine[y+1] )
		+ GetSimilarityRGB( &pMasterLine[x+1], &pSlaveLine[y-1] )
		+ GetSimilarityRGB( &pMasterLine[x+1], &pSlaveLine[y+1] )

		+ 2*(   GetSimilarityRGB( &pMasterLine[x], &pSlaveLine[y-1] )
		      + GetSimilarityRGB( &pMasterLine[x], &pSlaveLine[y+1] )
		      + GetSimilarityRGB( &pMasterLine[x-1], &pSlaveLine[y] )
		      + GetSimilarityRGB( &pMasterLine[x+1], &pSlaveLine[y] ) )

		+ 4*GetSimilarityRGB( &pMasterLine[x], &pSlaveLine[y] );
	return( RetVal / 16 );
}


unsigned int __stdcall GetSimilarityRGB_5x5( int x, int y, const Color32 *pMasterLine, const Color32 *pSlaveLine )
{
	int RetVal =
		  GetSimilarityRGB( &pMasterLine[x-2], &pSlaveLine[y-2] )
 		+ GetSimilarityRGB( &pMasterLine[x-2], &pSlaveLine[y+2] )
		+ GetSimilarityRGB( &pMasterLine[x+2], &pSlaveLine[y-2] )
		+ GetSimilarityRGB( &pMasterLine[x+2], &pSlaveLine[y+2] )

		+ 2*(   GetSimilarityRGB( &pMasterLine[x-2], &pSlaveLine[y-1] )
		      + GetSimilarityRGB( &pMasterLine[x-2], &pSlaveLine[y+1] )
		      + GetSimilarityRGB( &pMasterLine[x-1], &pSlaveLine[y-2] )
		      + GetSimilarityRGB( &pMasterLine[x-1], &pSlaveLine[y+2] )
			  + GetSimilarityRGB( &pMasterLine[x+1], &pSlaveLine[y-2] )
		      + GetSimilarityRGB( &pMasterLine[x+1], &pSlaveLine[y+2] )
		      + GetSimilarityRGB( &pMasterLine[x+2], &pSlaveLine[y-1] )
		      + GetSimilarityRGB( &pMasterLine[x+2], &pSlaveLine[y+1] ) )

		+ 3*(   GetSimilarityRGB( &pMasterLine[x], &pSlaveLine[y-2] )
		      + GetSimilarityRGB( &pMasterLine[x], &pSlaveLine[y+2] )
		      + GetSimilarityRGB( &pMasterLine[x-2], &pSlaveLine[y] )
		      + GetSimilarityRGB( &pMasterLine[x+2], &pSlaveLine[y] ) )

		+ 4*(   GetSimilarityRGB( &pMasterLine[x-1], &pSlaveLine[y-1] )
		      + GetSimilarityRGB( &pMasterLine[x-1], &pSlaveLine[y+1] )
		      + GetSimilarityRGB( &pMasterLine[x+1], &pSlaveLine[y-1] )
		      + GetSimilarityRGB( &pMasterLine[x+1], &pSlaveLine[y+1] ) )

		+ 6*(   GetSimilarityRGB( &pMasterLine[x], &pSlaveLine[y-1] )
		      + GetSimilarityRGB( &pMasterLine[x], &pSlaveLine[y+1] )
		      + GetSimilarityRGB( &pMasterLine[x-1], &pSlaveLine[y] )
		      + GetSimilarityRGB( &pMasterLine[x+1], &pSlaveLine[y] ) )

		+ 9*GetSimilarityRGB( &pMasterLine[x], &pSlaveLine[y] );
	return( RetVal / 81 );
}






bool ShowSimilarityMap(
	RenderedPicture             * pPicture,
	int                         Sensitivity,
	const Color32               * pMasterLine,
	const Color32               * pSlaveLine,
	int                         MasterLength,	
	int                         SlaveLength,
	int                         StripSize,
	int                         BarlettEyeSize
	)
{

	Point2D Res;
	int i,j;
	pColor32 Ptr;
	Line2D Line;
	Point2D Pixel;
	LineCalculationProcedure LProc;
	LineCalculationState LState;
	int help;
	GetSimilarityProcedure *GetSimilarity;

	if( Sensitivity <= 0 )
		return( false );

	switch(BarlettEyeSize)
	{
			case 3:
				GetSimilarity = GetSimilarityRGB_3x3;
				break;

			case 5:
				GetSimilarity = GetSimilarityRGB_5x5;
				break;

			case 7: // zatim neimplementovano
			default:
				GetSimilarity = GetSimilarityRGB_1x1;
				break;
	}

	const Color32 BorderColor = 0x00ff0000;


	Res.X = StripSize + 2 + MasterLength + 2 + StripSize;
	Res.Y = StripSize + 2 + SlaveLength + 2 + StripSize;

	if( !CreateRenderedPicture(pPicture, Res) )
		return( false );
	
	Ptr = pPicture->Pixels;
	for(i=0; i<StripSize; i++)
	{
		// cerne okraje
		memset(Ptr, 0, StripSize*sizeof(Color32));
		
		Ptr[ StripSize ] = BorderColor;
		Ptr[ StripSize+1 ] = BorderColor;

		memcpy( Ptr + StripSize + 2, pMasterLine, MasterLength * sizeof(Color32) );
		
		Ptr[ StripSize + 2 + MasterLength ] = BorderColor;
		Ptr[ StripSize + 2 + MasterLength + 1 ] = BorderColor;

		memset(Ptr+StripSize+2+MasterLength+2, 0, StripSize*sizeof(Color32));

		Ptr += pPicture->Resolution.X;
	}

	for(i=0; i<pPicture->Resolution.X; i++)
		Ptr[i] = BorderColor;
	Ptr += pPicture->Resolution.X;
	for(i=0; i<pPicture->Resolution.X; i++)
		Ptr[i] = BorderColor;
	Ptr += pPicture->Resolution.X;


	const int Border = 3;

	for(i=Border; i<SlaveLength-Border; i++)
	{	
		for(j=0; j<StripSize; j++)
			Ptr[ j ] = pSlaveLine[ i ];
		
		Ptr[ StripSize ] = BorderColor;
		Ptr[ StripSize + 1 ] = BorderColor;

		for(j=Border; j<MasterLength-Border; j++)
		{
			//help = 255 - (int)MIN( sqr((int)GetSimilarityRGB(pMasterLine[j], pSlaveLine[i])) / Sensitivity, 255 );
			help = 255 - (int)MIN( Sensitivity*GetSimilarityRGB( pMasterLine+j, pSlaveLine+i ), 255 );
			Ptr[ StripSize + 2 + j ] = COLOR32_CREATE( help, help, help );;
		}

		Ptr[ StripSize + 2 + MasterLength ] = BorderColor;
		Ptr[ StripSize + 2 + MasterLength + 1 ] = BorderColor;

		for(j=0; j<StripSize; j++)
			Ptr[ StripSize + 2 + MasterLength + 2 + j ] = pSlaveLine[ i ];

		Ptr += pPicture->Resolution.X;
	}

	// hranice
	for(i=0; i<pPicture->Resolution.X; i++)
		Ptr[i] = BorderColor;
	Ptr += pPicture->Resolution.X;
	for(i=0; i<pPicture->Resolution.X; i++)
		Ptr[i] = BorderColor;
	Ptr += pPicture->Resolution.X;

	// a vrsek
	for(i=0; i<StripSize; i++)
	{
		memset(Ptr, 0, StripSize*sizeof(Color32));

		Ptr[ StripSize ] = BorderColor;
		Ptr[ StripSize+1 ] = BorderColor;

		memcpy( Ptr + StripSize + 2, pMasterLine, MasterLength * sizeof(Color32) );
		
		Ptr[ StripSize + 2 + MasterLength ] = BorderColor;
		Ptr[ StripSize + 2 + MasterLength + 1 ] = BorderColor;

		memset(Ptr+StripSize+2+MasterLength+2, 0, StripSize*sizeof(Color32));

		Ptr += pPicture->Resolution.X;
	}

	return( true );

}


/*
bool __cdecl SegmentStartXGreater( const void *pSegA, const void *pSegB )
	{ return( ((SimilarSegment *)pSegA)->Start.X > ((SimilarSegment *)pSegB)->Start.X ); }

bool __cdecl SegmentStartYGreater( const void *pSegA, const void *pSegB )
	{ return( ((SimilarSegment *)pSegA)->Start.Y > ((SimilarSegment *)pSegB)->Start.Y ); }

bool __cdecl SegmentEndXGreater( const void *pSegA, const void *pSegB )
	{ return( ((SimilarSegment *)pSegA)->End.X > ((SimilarSegment *)pSegB)->End.X ); }

bool __cdecl SegmentEndYGreater( const void *pSegA, const void *pSegB )
	{ return( ((SimilarSegment *)pSegA)->End.Y > ((SimilarSegment *)pSegB)->End.Y ); }



bool Segmentation_Prepare(		
		int *pActSimilaritySum,
		LineCalculationState *pActState,
		LineCalculationState *pPrevState,
		LineCalculationProcedure *pLineProc,
		const SimilarSegmentParameters &Params,
		const Color32 *pMasterLine,
		const Color32 *pSlaveLine,
		const Line2D &ScanLine)
{	
	Point2D Point, Resolution;
	int PointsScaned = 0;

	*pActSimilaritySum = 0;

	Resolution.X = Resolution.Y = -1;

	*pLineProc = PrepareLineCalculation( pActState, ScanLine, Resolution );
	*pPrevState = *pActState;

	while( LineCalculationContinues(*pActState) )
	{
		Point = (*pLineProc)( pActState );
		*pActSimilaritySum += MIN( Params.WorstSimilarity, GetSimilarityRGB_Ex(Point.X, Point.Y, pMasterLine, pSlaveLine, Params.Scan_Distance) );	
		if( ++PointsScaned == Params.Segmentation_SectionLength )
			return( true );
	}

	return( false );
}



bool Segmentation_GetNextSegment(
		SimilarSegment *pSegment,						
		int *pActSimilaritySum,
		LineCalculationState *pActState,
		LineCalculationState *pPrevState,
		LineCalculationProcedure LineProc,
		const SimilarSegmentParameters &Params,
		const Color32 *pMasterLine,
		const Color32 *pSlaveLine   )
{
	Point2D ActPoint, PrevPoint;
	int InSegment = 0;

	while( LineCalculationContinues(*pActState) )
	{
		ActPoint = LineProc( pActState );
		*pActSimilaritySum += MIN( Params.WorstSimilarity, GetSimilarityRGB_Ex(ActPoint.X, ActPoint.Y, pMasterLine, pSlaveLine, Params.Scan_Distance) );
		
		PrevPoint = LineProc( pPrevState );
		*pActSimilaritySum -= MIN( Params.WorstSimilarity, GetSimilarityRGB_Ex(PrevPoint.X, PrevPoint.Y, pMasterLine, pSlaveLine, Params.Scan_Distance) );

		if( InSegment )
		{			
			if( *pActSimilaritySum > Params.Segmentation_WorstSegmentSimilarity )
			{
				// Segment konci
				if( InSegment - Params.Segmentation_SectionLength > Params.MinSegmentLength )
				{
					pSegment->End.X = PrevPoint.X;
					pSegment->End.Y = PrevPoint.Y;

					// Pokud se scan-primkou prochazi zprava (od vyssich x k mensim),
					// prohodime zacatek a konec segmentu
					if( pSegment->Start.X > pSegment->End.X )
						MySwap( pSegment->Start, pSegment->End );
					return( true );
				}
				
				// To, co jsme nasli, neni segment => hledame dal
				InSegment = 0;
			}
			else
				InSegment ++;
		}
		else
		{
			if( *pActSimilaritySum <= Params.Segmentation_WorstSegmentSimilarity )
			{
				// Segment zacina
				pSegment->Start.X = ActPoint.X;
				pSegment->Start.Y = ActPoint.Y;
				InSegment = 1;
			}
		}
	}


	if( InSegment - Params.Segmentation_SectionLength > Params.MinSegmentLength )
	{
		pSegment->End.X = PrevPoint.X;
		pSegment->End.Y = PrevPoint.Y;

		// Pokud se scan-primkou prochazi zprava (od vyssich x k mensim),
		// prohodime zacatek a konec segmentu
		if( pSegment->Start.X > pSegment->End.X )
			MySwap( pSegment->Start, pSegment->End );
		return( true );
	}
	else
		return( false );
	
} // inline bool Segmentation_GetNextSegment(...

*/