/* DeRendAI.h : Hlavickovy soubor s procedurami pro automaticky derendering
   Copyright(c) 2002,2003 Jan Curn (Jan.Curn@seznam.cz) 
*/


#ifndef _DERENDAI_H_INCLUDED_
#define _DERENDAI_H_INCLUDED_

#include "DeRender.h"
#include <math.h>



#include <crtdbg.h>


/*
typedef struct {
	int HUnimportance, LUnimportance, SUnimportance;
	int MaxStep;
	unsigned int MinSegmentSimilarity;
	int MinSegmentEquals;
	int SegmentBadCoefficient;
} MappingParameters;


class FEPOL_Configuration {
public:	
	unsigned long NumOfIterations;	
	MappingParameters MappingParams;
};



#define NO_SIMILARITY	(0xFFFFFFFF)
inline unsigned int GetSimilarityHLS( ColorHLS Col1, ColorHLS Col2, const MappingParameters &Params )
{
	return( GetHDifference(Col1, Col2) * Params.HUnimportance
	        + GetLDifference(Col1, Col2) * Params.LUnimportance
	        + GetSDifference(Col1, Col2) * Params.SUnimportance );
}
*/


inline unsigned int GetSimilarityRGB( const Color32 *pColorA, const Color32 *pColorB )
{
	unsigned char *ColorA_BytePtr = (unsigned char *)pColorA,
	              *ColorB_BytePtr = (unsigned char *)pColorB;
	return(   abs( (short)ColorA_BytePtr[0] - (short)ColorB_BytePtr[0] )
	        + abs( (short)ColorA_BytePtr[1] - (short)ColorB_BytePtr[1] )
	        + abs( (short)ColorA_BytePtr[2] - (short)ColorB_BytePtr[2] ) );
}


inline unsigned int GetSimilarityRGB_Ex( int m, int s, const Color32 *pMasterLine, const Color32 *pSlaveLine, int Distance )
{
	int sim, best_sim = GetSimilarityRGB( &pMasterLine[m], &pSlaveLine[s] );

	for(int i=1; i<=Distance; i++)
	{
		sim = GetSimilarityRGB( &pMasterLine[ m+i ] , &pSlaveLine[ s+i ] );
		if( sim < best_sim )
			best_sim = sim;
		sim = GetSimilarityRGB( &pMasterLine[ m+i ] , &pSlaveLine[ s-i ] );
		if( sim < best_sim )
			best_sim = sim;
		sim = GetSimilarityRGB( &pMasterLine[ m-i ] , &pSlaveLine[ s+i ] );
		if( sim < best_sim )
			best_sim = sim;
		sim = GetSimilarityRGB( &pMasterLine[ m-i ] , &pSlaveLine[ s-i ] );
		if( sim < best_sim )
			best_sim = sim;
	}
	
	return( best_sim );
}



unsigned int __stdcall GetSimilarityRGB_1x1( int x, int y, const Color32 *pMasterLine, const Color32 *pSlaveLine );
unsigned int __stdcall GetSimilarityRGB_3x3( int x, int y, const Color32 *pMasterLine, const Color32 *pSlaveLine );
//unsigned int __stdcall GetSimilarityRGB_5x5( int x, int y, const Color32 *pMasterLine, const Color32 *pSlaveLine );
//unsigned int __stdcall GetSimilarityRGB_7x7( int x, int y, Color32 *pMasterLine, Color32 *pSlaveLine );

typedef unsigned int (__stdcall GetSimilarityProcedure)( int x, int y, const Color32 *pMasterLine, const Color32 *pSlaveLine );
typedef GetSimilarityProcedure *pGetSimilarityProcedure;



struct GeneralPhotosCut {
	Point2D MasterLeft, MasterRight;
	Point2D SlaveLeft, SlaveRight;	
};



// Je inline protoze ve funkci CubicKnife() se CutTwoPhotos() vola uvnitr for-cyklu
// a mnoho promennych se v jeho tele nemeni => vyssi rychlost vypoctu
inline bool CutTwoPhotosInline(
	Point2D MasStartPos,
	const Camera &MasCam,
	const Camera &SlaCam,
	GeneralPhotosCut *pCut )
{
	double Value;
	Triangle3D Trian;
	StraightLine Line;
	Point3D Point1, Point2;	
	Point2D MasMax, SlaMax;
	PlaneCoeff SlaLeftPlane, SlaRightPlane, MasLeftPlane, MasRightPlane;

	MasMax.X = 2*MasCam.ResXDiv2 - 1;
	MasMax.Y = 2*MasCam.ResYDiv2 - 1;
	SlaMax.X = 2*SlaCam.ResXDiv2 - 1;
	SlaMax.Y = 2*SlaCam.ResYDiv2 - 1;	

	// Pripravime referencni primku z master-kamery
	Line.Point = MasCam.ViewPoint;
	Point1 = AddVector( MasCam.Pixel00_Point, MasCam.XAxis_Vector, MasStartPos.X );
	Point1 = AddVector( Point1, MasCam.YAxis_Vector, MasStartPos.Y );
	Line.Vector = GetVector(Line.Point, Point1);

	// Nalezneme obe plochy na leve a prave strane slave fotky, obe prochazeji bodem pohledu slave-kamery 
	Trian.A = SlaCam.ViewPoint;
	Trian.B = SlaCam.Pixel00_Point;
	Trian.C = AddVector( SlaCam.Pixel00_Point, SlaCam.YAxis_Vector );
	SlaLeftPlane = GetPlaneOfTriangle( Trian );
	Trian.B = AddVector( Trian.B, SlaCam.XAxis_Vector, SlaMax.X );
	Trian.C = AddVector( Trian.C, SlaCam.XAxis_Vector, SlaMax.X );
	SlaRightPlane = GetPlaneOfTriangle( Trian );

	// Ted pruniky obou slave-ploch s master-referencni-primkou zleva a zprava	
	if( !GetPenetrationPoint( &Value, Line, SlaLeftPlane ) )
		return( false );
	Point1 = AddVector( Line.Point, Line.Vector, Value );	
	if( !GetPenetrationPoint( &Value, Line, SlaRightPlane ) )
		return( false );
	Point2 = AddVector( Line.Point, Line.Vector, Value );

	// Ted zjistime polohu odpovidajiciho pixelu vlevo na slave-fotce...
	Line.Point = SlaCam.Pixel00_Point;
	Line.Vector = SlaCam.YAxis_Vector;
	if( !GetPenetrationPoint( &Value, Line, SlaCam.ViewPoint, Point1 ) )
		return( false );
	pCut->SlaveLeft.X = 0;
	pCut->SlaveLeft.Y = (short)Value;
	// ...a ted vpravo
	Line.Point = AddVector( SlaCam.Pixel00_Point, SlaCam.XAxis_Vector, SlaMax.X );
	Line.Vector = SlaCam.YAxis_Vector;
	if( !GetPenetrationPoint( &Value, Line, SlaCam.ViewPoint, Point2 ) )
		return( false );
	pCut->SlaveRight.X = SlaMax.X;
	pCut->SlaveRight.Y = (short)Value;

	// Pripravime referencni primku ze slave-kamery
	// - pouziva se jeste Value z minuleho vypoctu !!!
	Line.Point = SlaCam.ViewPoint;
	Point1 = AddVector( SlaCam.Pixel00_Point, SlaCam.XAxis_Vector, SlaMax.X );
	Point1 = AddVector( Point1, SlaCam.YAxis_Vector, Value );
	Line.Vector = GetVector(Line.Point, Point1);

	// Nalezneme obe plochy na leve a prave strane master fotky, obe prochazeji bodem pohledu master-kamery 
	Trian.A = MasCam.ViewPoint;
	Trian.B = MasCam.Pixel00_Point;
	Trian.C = AddVector( MasCam.Pixel00_Point, MasCam.YAxis_Vector );
	MasLeftPlane = GetPlaneOfTriangle( Trian );
	Trian.B = AddVector( Trian.B, MasCam.XAxis_Vector, MasMax.X );
	Trian.C = AddVector( Trian.C, MasCam.XAxis_Vector, MasMax.X );
	MasRightPlane = GetPlaneOfTriangle( Trian );

	// Ted pruniky obou master-ploch se slave-referencni-primkou zleva a zprava	
	if( !GetPenetrationPoint( &Value, Line, MasLeftPlane ) )
		return( false );
	Point1 = AddVector( Line.Point, Line.Vector, Value );	
	if( !GetPenetrationPoint( &Value, Line, MasRightPlane ) )
		return( false );
	Point2 = AddVector( Line.Point, Line.Vector, Value );

	// Ted zjistime polohu odpovidajiciho pixelu vlevo na master-fotce...
	Line.Point = MasCam.Pixel00_Point;
	Line.Vector = MasCam.YAxis_Vector;
	if( !GetPenetrationPoint( &Value, Line, MasCam.ViewPoint, Point1 ) )
		return( false );
	pCut->MasterLeft.X = 0;
	pCut->MasterLeft.Y = (short)Value;
	// ...a ted vpravo
	Line.Point = AddVector( MasCam.Pixel00_Point, MasCam.XAxis_Vector, MasMax.X );
	Line.Vector = MasCam.YAxis_Vector;
	if( !GetPenetrationPoint( &Value, Line, MasCam.ViewPoint, Point2 ) )
		return( false );
	pCut->MasterRight.X = SlaMax.X;
	pCut->MasterRight.Y = (short)Value;

	return( true );
}





class PhotosCut: public GeneralPhotosCut {

public:

	Color32 *MasterLine;
	Color32 *SlaveLine;

	int MasterLength, SlaveLength;

	inline void Initialize()	{ MasterLine = SlaveLine = NULL; MasterLength = SlaveLength = 0; }
	inline void Release()		{ if(MasterLine) free(MasterLine); if(SlaveLine) free(SlaveLine); Initialize(); }

	PhotosCut()		{ Initialize(); }
	~PhotosCut()	{ Release(); }

	
	bool Create( GeneralPhotosCut &Cut,
	             const RenderedPicture &MasterPicture,
	             const RenderedPicture &SlavePicture )
	{
		Release();
		int LineLength;
		Line2D LineSource;

		LineSource.A = Cut.MasterLeft;
		LineSource.B = Cut.MasterRight;
		if( !GetLineFromPicture( MasterPicture.Pixels, MasterPicture.Resolution, LineSource, &MasterLine, &MasterLength, 1) )
		{
			MasterLine = NULL;
			return( false );
		}

		LineSource.A = Cut.SlaveLeft;
		LineSource.B = Cut.SlaveRight;
		if( !GetLineFromPicture( SlavePicture.Pixels, SlavePicture.Resolution, LineSource, &SlaveLine, &SlaveLength, 1) )
		{
			free( MasterLine );
			MasterLine = NULL;
			SlaveLine = NULL;
			return( false );
		}
		
		MasterLeft = Cut.MasterLeft;
		MasterRight = Cut.MasterRight;
		SlaveLeft = Cut.SlaveLeft;
		SlaveRight = Cut.SlaveRight;
			
		return( true );
	}

};


class PhotosCuts: public ReAllocField<PhotosCut> {

public:
	PhotosCuts()	{ Initialize(); }
	~PhotosCuts()	{ Release(); }

	bool CreateFromPictures(
		const RenderedPicture &MasterPicture,
		const RenderedPicture &SlavePicture,
		const Camera &MasterCamera,
		const Camera &SlaveCamera,
		int Step)
	{
		GeneralPhotosCut Cut;
		Point2D CutStartPoint;
		int MasterLeftStartX, MasterLeftEndX,
		    StartY, EndY,
		    y,
			NumOfCuts;

		PhotosCut
		
		Release();

		if( Step < 0 )
			return( false );

		// Najdeme dolni mez
		CutStartPoint.X = 0;
		StartY = MasterPicture.Resolution.Y;
		EndY = 0;
		for( y=0; y<MasterPicture.Resolution.Y; y++ )
		{
			CutStartPoint.Y = y;

			if( !CutTwoPhotosInline(CutStartPoint, MasterCamera, SlaveCamera, &Cut) )
				continue;

			// Je rez na master i slave kamere v poradku ?
			if( Cut.MasterLeft.Y>0 && Cut.MasterRight.Y>0 && Cut.SlaveLeft.Y>0 && Cut.SlaveRight.Y>0 )
			{
				StartY = y;
				break;
			}
		}

		// Najdeme horni mez
		for( y=MasterPicture.Resolution.Y-1; y>0; y-- )
		{
			CutStartPoint.Y = y;

			if( !CutTwoPhotosInline(CutStartPoint, MasterCamera, SlaveCamera, &Cut) )
				continue;

			// Je rez na master i slave kamere v poradku ?
			if(    Cut.MasterLeft.Y<=MasterPicture.Resolution.Y
			    && Cut.MasterRight.Y<=MasterPicture.Resolution.Y
			    && Cut.SlaveLeft.Y<=SlavePicture.Resolution.Y
			    && Cut.SlaveRight.Y<=SlavePicture.Resolution.Y )
			{
				EndY = y;
				break;
			}
		}

		//
		if( EndY <= StartY )
			return( false );

		Step++ ;
		NumOfCuts = (EndY - StartY + Step) / Step;

		if( !ReAlloc(NumOfCuts) )
			return( false ); 

		int counter=0;

		for( y=StartY; y<EndY; y += Step )
		{
			CutStartPoint.Y = y;
			if( !CutTwoPhotosInline(CutStartPoint, MasterCamera, SlaveCamera, &Cut) )
				return( false );

			if( ! (AddItem())->Create(Cut, MasterPicture, SlavePicture) )
				return( false );

			counter++;
			_ASSERT( counter <= NumOfCuts );
			_ASSERT( (GetLastItem())->MasterLength == (GetLastItem())->SlaveLength );			
		}

		return( true );
	}


	bool DrawToPicture( RenderedPicture *pNewPicture )
	{
		Point2D Resolution;
		int x, y, i;

		if( !ItemsUsed )
			return( false );

		Resolution.X = MAX( Field[0].MasterLength , Field[0].SlaveLength );
		Resolution.Y = 2 * ItemsUsed;		

		if( !CreateRenderedPicture( pNewPicture, Resolution ) )
			return( false );

		for(i=0; i<ItemsUsed; i++)		
			for(x=0; x<Field[0].SlaveLength; x++)
				pNewPicture->Pixels[ x + i*Resolution.X ] = Field[i].SlaveLine[x];

		for(i=0; i<ItemsUsed; i++)		
			for(x=0; x<Field[0].MasterLength; x++)
				pNewPicture->Pixels[ x + (i+ItemsUsed)*Resolution.X ] = Field[i].MasterLine[x];

		return( true );
	}

};



struct CorrespondingLines {
	Color32 *pMasterLine, *pSlaveLine;
	unsigned int MasterLength, SlaveLength;
};


struct SimilarSegmentParameters {

	double Declination; // (1,3>
	double InvDeclination; // = 1 / Declination

	int Scan_Distance; // Velikost nahodneho scanovaciho oka <1,20>	
	int Scan_Border; // Hranice scanovaci oblasti - musi byt vetsi nez Scan_Distance
	int Scan_BlindWidth; // Sirka slepe oblasti vlevo nahore a vpravo dole, typicky tak Lines.SlaveLegth/2

	int C_XFrom, C_XTo;
	int C_MaxVerticalDisplacement;

	int WorstSimilarity; // Nejhorsi podobnost, pres tuto hranici se podobnost nebere v uvahu
	int WorstSegmentSimilarity; // Nejhorsi podobnost v segmentu

	int MinSegmentLength;

	int Segmentation_SectionLength;
	int Segmentation_WorstSegmentSimilarity; // = WorstSegmentSimilarity * Segmentation_SectionLength

	int Pooling_A_Attempts;
	int Pooling_B_Step;
	int Pooling_MaxSegments;

	int Entity_MaxSegments;

	// New

	int SegColumns_Step; // Rozestup sloupcu pro segmentaci, typicky 8
	int SegColumns_TotalBorder; // Sirka segmentacniho pasu, obvykle tak 2/5 delky rezu
	int SegRows_ScanStep; // Krok mezi jednotlivymi scany na radcich, typicky tak SegColumns_Step/2

	int Seg_WorstSimilarity; // Nejhorsi podobnost v segmentu
	
};


#pragma pack(push, 8)

struct SimilarSegment {
	unsigned short Position;
	unsigned short Length;
	long Next; // -1 znamena za timto segmentem konci spojovy seznamu

	// Povinne - viz. ReAllocField<SimilarSegmentInPool>
	inline void Initialize() {}
	inline void Release()    {}
};

#pragma pack(pop)


#pragma pack(push, 4)

struct SegmentationRow {

	long FirstSegmentIndex;

	inline void Initialize()	{ FirstSegmentIndex = -1; }
	inline void Release()		{}	
};

#pragma pack(pop)


struct SegmentationColumn {

	unsigned short MasterStartIndex, SlaveStartIndex;
	ReAllocField<SegmentationRow> SegRows;	

	inline void Initialize()	{ SegRows.Initialize(); }
	inline void Release()		{ SegRows.Release(); }

	SegmentationColumn()		{ Initialize(); }
	~SegmentationColumn()		{ Release(); }
};


inline unsigned short int GetBestSquareSimilarity(
	pColor32 MasterLine,
	pColor32 SlaveLine,
	unsigned short int MasterIndex,
	unsigned short int SlaveIndex,
	unsigned short int SizeMinus1 )
{
	unsigned short int similarity, best_similarity, m, s, max_m, max_s;

	best_similarity = 1000;
	max_m = MasterIndex + SizeMinus1;
	max_s = SlaveIndex + SizeMinus1;

	for( m=MasterIndex; m<max_m; m+=2 )
		for( s=SlaveIndex; s<max_s; s+=2 )
		{
			similarity = GetSimilarityRGB( &MasterLine[m], &SlaveLine[s] );
			if( similarity < best_similarity )
				best_similarity = similarity;
		}
	return( best_similarity );
}


#define CUTS_ANALYZATION_SUCCESS			0
#define CUTS_ANALYZATION_INSUFF_MEMORY		1
#define CUTS_ANALYZATION_POOL_FULL			2
#define CUTS_ANALYZATION_INVALID_ARGS		3


class PhotosCutsAnalyzation {
public:

	PhotosCuts Cuts;

	// SimilarSegmenty jsou pro kazdou SegmentationRow organizovany jako jednosmerny spojovy
	// seznam. Zbyle, nevyuzite, segmenty v SegmentsPool si zorganizujeme opet jakozto
	// jednosmerny spojovy seznam, ten bude zacinat segmentem s indexem FirstUnusedSegmentIndex
	ReAllocField<SimilarSegment> SegmentsPool;
	
	unsigned long NumOfUsedSegments;

	long FirstUnusedSegmentIndex;

	ReAllocField<SegmentationColumn> SegColumns;

	void Initialize()	{ FirstUnusedSegmentIndex=-1; Cuts.Initialize(); SegmentsPool.Initialize(); SegColumns.Initialize(); }
	void Release()		{ FirstUnusedSegmentIndex=-1; Cuts.Release(); SegmentsPool.Release(); SegColumns.Release(); }

	PhotosCutsAnalyzation()		{ Initialize(); }
	~PhotosCutsAnalyzation()	{ Release(); }


	inline bool CreatePhotoCuts(
		const RenderedPicture &MasterPicture,
		const RenderedPicture &SlavePicture,
		const Camera &MasterCamera,
		const Camera &SlaveCamera,
		int Step)
	{
		return( Cuts.CreateFromPictures(MasterPicture,SlavePicture,MasterCamera,SlaveCamera,Step) );
	}

	
	bool SegmentateRow(
		unsigned long ColumnIndex,
		unsigned long RowIndex,
		const SimilarSegmentParameters &Params )
	{	
		unsigned short MasterIndex, SlaveIndex, MaxMasterIndex, Similarity;
		// Stav segmentace je urcen poctem nalezenych ctvercu. SquaresFound nabyva
		// hodnot 0..zadny nenalezen, 1...jeden nalezen, 2...dva nebo vice nalezeno
		unsigned char SquaresFound;	
		SimilarSegment *pNewSegment;

		unsigned short NewSegPosition;
		long *pNextSegmentIndex;
		unsigned long i;

		SegmentationColumn &ColumnRef = SegColumns.Field[ColumnIndex]; 		
		SegmentationRow &RowRef = ColumnRef.SegRows.Field[RowIndex];
		PhotosCut &CutRef = Cuts.Field[ RowIndex ];

		// Odkud kam se bude segmentovat
		MasterIndex = ColumnRef.MasterStartIndex;
		SlaveIndex = ColumnRef.SlaveStartIndex;		
		MaxMasterIndex = CutRef.MasterLength - ColumnRef.SlaveStartIndex - Params.SegRows_ScanStep - 1;

		// Pokud se nenalezne zadny segment, na konci se do RowRef.FirstSegmentIndex nastavi -1
		pNextSegmentIndex = & RowRef.FirstSegmentIndex;
		// Zadny segment jsme zatim nenalezli
		SquaresFound = 0;

		for( ; MasterIndex<MaxMasterIndex; MasterIndex += Params.SegRows_ScanStep )
		{			
			Similarity = GetBestSquareSimilarity( CutRef.MasterLine, CutRef.SlaveLine, MasterIndex, SlaveIndex, Params.SegRows_ScanStep );
			
			// Pokud je ctverec dost podobny
			if( Similarity<=Params.Seg_WorstSimilarity )
				switch( SquaresFound )
				{
				case 0:
					NewSegPosition = MasterIndex;
				case 1:
					SquaresFound++ ;
					break;
				case 2: // 2 nebo vice
					break;
				}
			else
			{
				// Pokud mame dost ctvercu, nalezli jsme segment
				if( SquaresFound >= 2 )
				{	
					if( SegmentsPool.IsFull() )
					{
						*pNextSegmentIndex = -1;
						return( false );
					}

					*pNextSegmentIndex = SegmentsPool.ItemsUsed;

					pNewSegment = SegmentsPool.AddItem();					
					pNewSegment->Position = NewSegPosition - ColumnRef.MasterStartIndex ;
					pNewSegment->Length = MasterIndex - NewSegPosition;					

					pNextSegmentIndex = &pNewSegment->Next;
				}

				// A hledame dalsi segment od zacatku
				SquaresFound = 0;
			}
				
			SlaveIndex += Params.SegRows_ScanStep;
		}

		// Ukoncime spojovy seznam
		*pNextSegmentIndex = -1;

		return( true );
	}


	unsigned long StartAnalyzation( const SimilarSegmentParameters &Params, unsigned long NumOfSegmentsToAllocate )
	{
		int SegColumns_ToAllocate, i, j, SegColumns_OnOneSide;
		SegmentationColumn *pNewColumn;
		long *pNextSegmentIndex;
		SimilarSegment *pNewSegment;
		unsigned long RetVal;

		if( NumOfSegmentsToAllocate <= 1 )
			return( CUTS_ANALYZATION_INVALID_ARGS );

		if( !SegmentsPool.ReAlloc(NumOfSegmentsToAllocate) )
			return( CUTS_ANALYZATION_INSUFF_MEMORY );

		// Alokujeme sloupce
		SegColumns_OnOneSide = (Params.SegColumns_TotalBorder + Params.SegColumns_Step) / Params.SegColumns_Step;
		SegColumns_ToAllocate = 2*SegColumns_OnOneSide - 1;
		if( !SegColumns.ReAlloc( SegColumns_ToAllocate ) )
		{
			Release();
			return( CUTS_ANALYZATION_INSUFF_MEMORY );
		}

		// Sloupce nastavime podle Params a alokujeme pro ne i segmentacni radky
		pNewColumn = SegColumns.AddItem();
		pNewColumn->MasterStartIndex = 0;
		pNewColumn->SlaveStartIndex = 0;
		if( !pNewColumn->SegRows.ReAlloc(Cuts.ItemsUsed) )
		{
			Release();
			return( CUTS_ANALYZATION_INSUFF_MEMORY );
		}
		pNewColumn->SegRows.ItemsUsed = Cuts.ItemsUsed;
		for( i=Params.SegColumns_Step; i<=Params.SegColumns_TotalBorder; i+=Params.SegColumns_Step )
		{
			pNewColumn = SegColumns.AddItem();
			pNewColumn->MasterStartIndex = i;
			pNewColumn->SlaveStartIndex = 0;			
			if( !pNewColumn->SegRows.ReAlloc(Cuts.ItemsUsed) )
			{
				Release();
				return( CUTS_ANALYZATION_INSUFF_MEMORY );
			}
			pNewColumn->SegRows.ItemsUsed = Cuts.ItemsUsed;

			pNewColumn = SegColumns.AddItem();			
			pNewColumn->MasterStartIndex = 0;	
			pNewColumn->SlaveStartIndex = i ;
			if( !pNewColumn->SegRows.ReAlloc(Cuts.ItemsUsed) )
			{
				Release();
				return( CUTS_ANALYZATION_INSUFF_MEMORY );
			}
			pNewColumn->SegRows.ItemsUsed = Cuts.ItemsUsed;
		}

		// Rozsegmetnovani radek
		for(i=0; i<SegColumns_ToAllocate; i++)
			for(j=0; j<Cuts.ItemsUsed; j++)
				if( ! SegmentateRow(i, j, Params) )
				{
					FirstUnusedSegmentIndex = -1;
					NumOfUsedSegments = SegmentsPool.ItemsUsed;
					return( CUTS_ANALYZATION_POOL_FULL );
				}

		NumOfUsedSegments = SegmentsPool.ItemsUsed;

		// Nevyuzite segmenty v SegmentsPool zorganizujeme jako spojovy seznam
		pNextSegmentIndex = &FirstUnusedSegmentIndex;
		while( !SegmentsPool.IsFull() )
		{
			*pNextSegmentIndex = SegmentsPool.ItemsUsed;
			pNewSegment = SegmentsPool.AddItem();			
			pNextSegmentIndex = &pNewSegment->Next;
		}
		*pNextSegmentIndex = -1;

		return( CUTS_ANALYZATION_SUCCESS );
	}




	bool ShowToRenderScene( RenderScene *pScene )
	{
		unsigned long i, j, LineIndex = 0;
		long SegIndex;	
		SimilarSegment *pSegment;
		RSFPoint *pRSF_Point;
		RSFLine *pRSF_Line;

		CloseRenderScene( pScene );

		pScene->Points = (RSFPoint *)malloc( sizeof(RSFPoint) * 2 * NumOfUsedSegments );
		if( !pScene->Points )
			return( false );

		pScene->Lines = (RSFLine *)malloc( sizeof(RSFLine) * NumOfUsedSegments );
		if( !pScene->Lines )
		{
			free(pScene->Points);
			pScene->Points = NULL;
			return( false );
		}

		pScene->NumOfPoints = 2 * NumOfUsedSegments;
		pScene->NumOfLines = NumOfUsedSegments;

		pRSF_Line = pScene->Lines;
		pRSF_Point = pScene->Points;

		for(i=0; i<SegColumns.ItemsUsed; i++)
			for(j=0; j<SegColumns.Field[i].SegRows.ItemsUsed; j++)
			{				
				SegIndex = SegColumns.Field[i].SegRows.Field[j].FirstSegmentIndex;

				while( SegIndex>=0 )
				{
					pSegment = &SegmentsPool.Field[ SegIndex ];					

					pRSF_Line->PointsIndexes[0] = LineIndex*2;
					pRSF_Line->PointsIndexes[1] = LineIndex*2 + 1;
					pRSF_Line->Color = 0x00ffffff;

					pRSF_Point->BillboardSize = 5;
					pRSF_Point->Color = 0x00ff0000;
					pRSF_Point->Position.X = SegColumns.Field[i].MasterStartIndex + pSegment->Position;
					pRSF_Point->Position.Z = SegColumns.Field[i].SlaveStartIndex + pSegment->Position;
					pRSF_Point->Position.Y = j;

					pRSF_Point++;

					pRSF_Point->BillboardSize = 5;
					pRSF_Point->Color = 0x00ff0000;
					pRSF_Point->Position.X = SegColumns.Field[i].MasterStartIndex + pSegment->Position + pSegment->Length;
					pRSF_Point->Position.Z = SegColumns.Field[i].SlaveStartIndex + pSegment->Position + pSegment->Length;
					pRSF_Point->Position.Y = j;

					SegIndex = pSegment->Next;
					LineIndex++;
					pRSF_Line++;
					pRSF_Point++;
				}

			}	



		return( true );
	}


		



};



/*



// Segment podobnosti
class SimilarSegment {

public:

	Point2D_Ex<int> Start, End;	

	// Segmenty jsou v SegmentsPool organizovany pomoci 4 jednosmernych
	// spojovych seznamu, -1 znamena konec/zacatek seznamu
	int NextSegIndex[2], PrevSegIndex[2]; 


	// Povinne - viz. ReAllocField<SimilarSegmentInPool>
	inline void Initialize() {}
	inline void Release()    {}
};


typedef SimilarSegment *pSimilarSegment;


bool Segmentation_Prepare(		
		int *pActSimilaritySum,
		LineCalculationState *pActState,
		LineCalculationState *pPrevState,
		LineCalculationProcedure *pLineProc,
		const SimilarSegmentParameters &Params,
		const Color32 *pMasterLine,
		const Color32 *pSlaveLine,
		const Line2D &ScanLine);


bool Segmentation_GetNextSegment(
		SimilarSegment *pSegment,						
		int *pActSimilaritySum,
		LineCalculationState *pActState,
		LineCalculationState *pPrevState,
		LineCalculationProcedure LineProc,
		const SimilarSegmentParameters &Params,
		const Color32 *pMasterLine,
		const Color32 *pSlaveLine   );




// Jedinec reseni
class SolutionEntity
{

public:

	SimilarSegment *Segments; // Pole se segmenty podobnosti
	unsigned int SegmentsUsed; // Pocet pouzitych polozek v poli Segments
	unsigned int SegmentsAllocated;


	inline void Initialize() { Segments = NULL; SegmentsUsed = SegmentsAllocated = 0; }
	inline void Release()    { if(Segments) free(Segments); Initialize(); }
	SolutionEntity()  { Initialize(); }
	~SolutionEntity() { Release(); }

	// Metoda vytvori jedince, ten bude moct mit maximalne SegmentsToAllocate segmentu
	bool Create(unsigned int SegmentsToAllocate)
	{
		if( SegmentsToAllocate == 0 )
			return( false );

		SegmentsUsed = 0;

		if( SegmentsToAllocate == SegmentsAllocated )
			return( true );			

		Release();		

		Segments = (SimilarSegment *)malloc( SegmentsToAllocate * sizeof(SimilarSegment) );
		if( !Segments )

			return( false );

		SegmentsAllocated = SegmentsToAllocate;		
		return( true );
	}

	// Metoda prida segment NewSeg k jedinci, neprovadi zadne kontroly, vraci index v poli Segments,
	// kam se segment ulozil.
	inline unsigned int AddSegment( const SimilarSegment &NewSeg )
	{
		Segments[ SegmentsUsed ] = NewSeg;
		return( SegmentsUsed++ );
	}


	unsigned int GetRate()
	{
		unsigned int i, Rate = 0;

		for(i=0; i<SegmentsUsed; i++)
			Rate += (unsigned int)( Segments[i].End.X - Segments[i].Start.X + 1
			                        + Segments[i].End.X - Segments[i].Start.X + 1 );
		return( Rate );
	}

	void DrawToPicture(RenderedPicture *pPicture, Point2D Where, float Zoom, Color32 Color)
	{
		unsigned int i;
		Line2D SegmentLine;
		LineCalculationState LState;
		LineCalculationProcedure LProc;
		Point2D Pixel;

		for(i=0; i<SegmentsUsed; i++)
		{
			SegmentLine.A.X = Where.X + Segments[i].Start.X*Zoom;
			SegmentLine.A.Y = Where.Y + Segments[i].Start.Y*Zoom;
			SegmentLine.B.X = Where.X + Segments[i].End.X*Zoom;
			SegmentLine.B.Y = Where.Y + Segments[i].End.Y*Zoom;

			LProc = PrepareLineCalculation( &LState, SegmentLine, pPicture->Resolution );

			while( LineCalculationContinues(LState) )
			{
				Pixel = LProc( &LState );
				pPicture->Pixels[ Pixel.X + Pixel.Y*pPicture->Resolution.X ] = Color;
				pPicture->Pixels[ Pixel.X + 1 + Pixel.Y*pPicture->Resolution.X ] = Color;
				pPicture->Pixels[ Pixel.X - 1 + Pixel.Y*pPicture->Resolution.X ] = Color;
			}			
		}
	}

	bool ChildOf( const SolutionEntity &Mother, const SolutionEntity &Father )
	{



		return( true );

	}


}; // class SolutionEntity




bool __cdecl SegmentStartXGreater( const void *pSegA, const void *pSegB );
bool __cdecl SegmentStartYGreater( const void *pSegA, const void *pSegB );
bool __cdecl SegmentEndXGreater( const void *pSegA, const void *pSegB );
bool __cdecl SegmentEndYGreater( const void *pSegA, const void *pSegB );


#define POOL_ALLOC_STEP		(32)

class SegmentsPool: public ReAllocField<SimilarSegment> {

public:
	
	inline void Initialize() { ReAllocField<SimilarSegment>::Initialize(); }
	inline void Release()    { ReAllocField<SimilarSegment>::Release(); }

	SegmentsPool() { Initialize(); }
	~SegmentsPool() { Release(); }

private:

	inline void GetBInterval( int *pBFrom, int *pBTo, double a, Point2D Resolution, const SimilarSegmentParameters &Params )
	{
		if( a < 1.0 )
		{
			// Params.InvDeclination <= a < 1
			*pBFrom = - a* ( Resolution.X - 1 - Params.Scan_BlindWidth ) + 1;
			*pBTo = Resolution.Y - 1 - a*Params.Scan_BlindWidth - 1;
		}
		else
		{
			// 1 <= a <= Params.Declination
			*pBFrom = Params.Scan_BlindWidth - a*( Resolution.X - 1 ) + 1;
			*pBTo = Resolution.Y - 1 - Params.Scan_BlindWidth - 1;
		}
	}

	inline bool ApplyBlindAreasToLine( Line2D *pLine, double a, double b, Point2D Resolution, const SimilarSegmentParameters &Params )
	{
		double t;
		Point2D_Ex<double> Start, End;

		// *** Najdeme pocatecni a koncovy bod scan-usecky na primce, ta je cela uvnitr nasi scan-oblasti
		// Prunik s primkou y=0
		Start.X = - b/a;
		Start.Y = 0.0;

		// Prunik s primkou y=Resolution.Y-1
		End.X = - (b - Resolution.Y + 1) / a;
		End.Y = Resolution.Y - 1;

		if( End.X<0 || Start.X>=Resolution.X )
			return( false ); // cela usecka je mimo nas vyrez

		// Aplikujeme prunik s primkou y=t, x=0, t je parametr
		if( Start.X < 0 )
		{
			Start.X = 0;
			Start.Y = b;
		}

		// a jeste s primkou y=t, x=Resolution.X-1, t je parametr
		if( End.X >= Resolution.X )
		{
			End.X = Resolution.X - 1;
			End.Y = a*End.X + b;
		}

		// Pokud by bylo 1-a==0, pak vypocet bude v telech obou nasledujicich testu odchycen (pokud se tam ale vubec dostane),
		// to diku tomu, ze slepe oblasti jsou tvoreny rovnoramenym pravouhlym trojuhelnikem (jeho prepona ma taky smernici 1).
			
		// Orizneme usecku podle leve horni "slepe" oblasti
		if( Start.Y >= Resolution.Y - Params.Scan_BlindWidth - 1 )
		{
			if( End.X <= Params.Scan_BlindWidth )
				return( false ); // cela usecka je ve "slepe" oblasti vyrezu
			// Vypocitame prunik s primkou ohranicujici slepou oblast
			t = (b - (Resolution.Y-Params.Scan_BlindWidth - 1)) / (1 - a); 
			Start.X = t;
			Start.Y = Resolution.Y-Params.Scan_BlindWidth-1 + t;
		}
		else
			if( End.X <= Params.Scan_BlindWidth )
			{
				t = (b - (Resolution.Y-Params.Scan_BlindWidth-1)) / (1 - a); 
				End.X = t;
				End.Y = Resolution.Y-Params.Scan_BlindWidth-1 + t;
			}

		// Orizneme usecku podle prave spodni "slepe" oblasti
		if( (End.X == Resolution.X-1) && (End.Y <= Params.Scan_BlindWidth) )
		{
			if( (Start.X >= Resolution.X-Params.Scan_BlindWidth-1) && (Start.Y == 0) )
				return( false ); // cela usecka je ve "slepe" oblasti vyrezu
			End.X = (b + (Resolution.X-Params.Scan_BlindWidth-1)) / (1 - a);
			End.Y = a*End.X + b;
		}			
		else
			if( (Start.X >= Resolution.X-Params.Scan_BlindWidth-1) && (Start.Y == 0) )
			{				
				Start.X = (b + (Resolution.X-Params.Scan_BlindWidth-1)) / (1 - a); 
				Start.Y = a*Start.X + b;
			}

		pLine->A.X = Start.X;
		pLine->A.Y = Start.Y;
		pLine->B.X = End.X;
		pLine->B.Y = End.Y;
		return( true );
	}


public:


	// Pri chybe vraci zapornou hodnotu, jinak pocet nactenych segmentu
	int Generate( const CorrespondingLines &Lines, const SimilarSegmentParameters &Params )	             
	{
		int A_Attempts;
		double a, a_bellow_1_step, a_over_1_step;
		int b, b_from, b_to, ActSimilaritySum, SegmentsFound = 0;
		Point2D_Ex<double> Start, End;
		Point2D Resolution;
		Line2D ScanLine;	
		LineCalculationState ActState, PrevState;
		LineCalculationProcedure LineProc;
		SimilarSegment NewSegment;

		// Spojove seznamy se zatim negeneruji
		NewSegment.NextSegIndex[0] = NewSegment.NextSegIndex[1] = -1;
		NewSegment.PrevSegIndex[0] = NewSegment.PrevSegIndex[1] = -1;

		Release();
	
		// Mezi scan-vyrezem a celou similarity map je Params.Scan_Border bodu siroka hranice,
		// ta je tam aby se pri scanovani s nahodnymi pokusy necetlo z bodu mimo simalirity map.
		// Pred samotnym scanovanim je nutno ziskanou usecku posunout podle Params.Scan_Border.
		Resolution.X = Lines.MasterLength - 2*Params.Scan_Border;
		Resolution.Y = Lines.SlaveLength - 2*Params.Scan_Border;


		A_Attempts = Params.Pooling_A_Attempts;

		// A_Attempts bude liche cislo vetsi nebo rovno 5
		if( A_Attempts < 4 )
			return( false );		
		if( A_Attempts % 2 == 0 )
			A_Attempts++;	

		// Velikost zmeny a v kazdem kroku
		a_bellow_1_step = (1.0-Params.InvDeclination) / (A_Attempts/2);
		a_over_1_step = (Params.Declination-1.0) / (A_Attempts/2);
		

		//for( a=Params.InvDeclination; a<1.0; a+=a_bellow_1_step )
		for( a=1.0; a<=1.0; a+=a_bellow_1_step )
		{
			GetBInterval( &b_from, &b_to, a, Resolution, Params );

			for( b=b_from; b<=b_to; b+=Params.Pooling_B_Step )
			{
				if( !ApplyBlindAreasToLine( &ScanLine, a, (double)b, Resolution, Params ) )
					continue;

				// Posuneme jeste cely scan-vyrez, doted byl relativne vztahovan k Params.Scan_Border
				ScanLine.A.X += Params.Scan_Border;
				ScanLine.A.Y += Params.Scan_Border;
				ScanLine.B.X += Params.Scan_Border;
				ScanLine.B.Y += Params.Scan_Border;
				
			//	NewSegment.Start.X = ScanLine.A.X;
			//	NewSegment.Start.Y = ScanLine.A.Y;
			//	NewSegment.End.X = ScanLine.B.X;
			/	NewSegment.End.Y = ScanLine.B.Y;
				if( !AddItem(NewSegment) )
					return( -1 ); // Nedostatek pameti	
				continue; 

				// Prochazime scan-useckou a vytvarime segmenty			
				if( !Segmentation_Prepare( &ActSimilaritySum, &ActState, &PrevState, &LineProc, Params, Lines.pMasterLine, Lines.pSlaveLine, ScanLine) )
					continue;

				while( Segmentation_GetNextSegment( &NewSegment, &ActSimilaritySum, &ActState, &PrevState, LineProc, Params, Lines.pMasterLine, Lines.pSlaveLine ) )
				{
					if( !AddItem(NewSegment) )
						return( -1 ); // Nedostatek pameti				
					if( ++SegmentsFound == Params.Pooling_MaxSegments )
						return( SegmentsFound );
				}
			} // for( b=...
		} // for( a=...

	/*	for( a=1.0; a<Params.Declination; a+=a_over_1_step )
		{
			GetBInterval( &b_from, &b_to, a, Resolution, Params );

			for( b=b_from; b<=b_to; b+=Params.Pooling_B_Step )
			{
				if( !ApplyBlindAreasToLine( &ScanLine, a, (double)b, Resolution, Params ) )
					continue;

				// Posuneme jeste cely scan-vyrez, doted byl relativne vztahovan k Params.Scan_Border
				ScanLine.A.X += Params.Scan_Border;
				ScanLine.A.Y += Params.Scan_Border;
				ScanLine.B.X += Params.Scan_Border;
				ScanLine.B.Y += Params.Scan_Border;			

				// Prochazime scan-useckou a vytvarime segmenty			
				if( !Segmentation_Prepare( &ActSimilaritySum, &ActState, &PrevState, &LineProc, Params, Lines.pMasterLine, Lines.pSlaveLine, ScanLine) )
					continue;

				while( Segmentation_GetNextSegment( &NewSegment, &ActSimilaritySum, &ActState, &PrevState, LineProc, Params, Lines.pMasterLine, Lines.pSlaveLine ) )
				{
					if( !AddItem(NewSegment) )
						return( -1 ); // Nedostatek pameti				
					if( ++SegmentsFound == Params.Pooling_MaxSegments )
						return( SegmentsFound );
				}
			} // for( b=...
		} // for( a=...		

		return( SegmentsFound );

	} // int Generate(...)



	bool SortIt()
	{
		int *AllocPtr, *SortedIndexes, i;

		if( ItemsUsed == 0 )
			return( false );

		AllocPtr = (int *)malloc( (ItemsUsed + 2)*sizeof(int) );
		if( !AllocPtr )
			return( false );

		SortedIndexes = AllocPtr + 1;
		SortedIndexes[-1] = -1;
		SortedIndexes[ItemsUsed] = -1;
		
		// Inicializace indexu
		for(i=0; i<(int)ItemsUsed; i++)
			SortedIndexes[i] = i;

		// Postupne kazdemu segmentu do Field priradime predchudce a nasledovnika
		// ve spojovem seznamu, poradi zapojeni je urceno velikosti
		// hodnot jejiSegment::Start.X. a -.Y a
		///urcime setridenimSetridime Field podle vlastnosti dane funkci SegmentStartXGreater, vysledek - indexy
		// do pole Field ulozime do SortedIndexes.
		MyQuickSort( Field, SortedIndexes, ItemsUsed, SegmentStartXGreater );
		for(i=0; i<ItemsUsed; i++)
			Field[ SortedIndexes[i] ].NextSegIndex[0] = SortedIndexes[i+1];		

		MyQuickSort( Field, SortedIndexes, ItemsUsed, SegmentStartYGreater );
		for(i=0; i<ItemsUsed; i++)				
			Field[ SortedIndexes[i] ].NextSegIndex[1] = SortedIndexes[i+1];
		
		MyQuickSort( Field, SortedIndexes, ItemsUsed, SegmentEndXGreater );
		for(i=0; i<ItemsUsed; i++)	
			Field[ SortedIndexes[i] ].PrevSegIndex[0] = SortedIndexes[i-1];			

		MyQuickSort( Field, SortedIndexes, ItemsUsed, SegmentEndYGreater );
		for(i=0; i<ItemsUsed; i++)
			Field[ SortedIndexes[i] ].PrevSegIndex[1] = SortedIndexes[i-1];		

		return( true );
	}


	unsigned int GetRandomEntity( SolutionEntity *pEntity, unsigned int MaxSegments )
	{	
		SimilarSegment *pPrevSeg, *pNextSeg, *pStartingSeg; // Ukazatele do Field
		SimilarSegment *pSeg; // Ukazatel do pEntity
		int RandMod2, Index;

		if( ItemsUsed == 0 )
			return( 0 );

		if( MaxSegments <= 1 )
			return( 0 );
		
		if( !pEntity->Create( MaxSegments ) )
			return( 0 );
		
		// Nahodny vyber pocatecniho segmentu
		pStartingSeg = &Field[ rand() % ItemsUsed ];				

		// Pridame pocatecni segment do pEntity a pointer na nej si ulozime do pSeg
		pEntity->AddSegment( *pStartingSeg );
		
		// Nahodna volba smeru napojovani
		RandMod2 = rand() % 2;

		// cyklus pres hledani segmentu k pridani - dopredu		
		pSeg = & pEntity->Segments[0];
		pNextSeg = pStartingSeg;
		while( pNextSeg->NextSegIndex[0]>=0 && pNextSeg->NextSegIndex[1]>=0 )
		{		
			// vyber dalsiho segmentu podle puvodni nahodne volby smeru
			pNextSeg = &Field[ pNextSeg->NextSegIndex[RandMod2] ];
		
			// Pokud je segment pNextSeg korektni, pridame ho do pEntity...
			if(  pSeg->End.X < pNextSeg->Start.X  &&  pSeg->End.Y < pNextSeg->Start.Y  )
			{
				Index = (int)pEntity->AddSegment( *pNextSeg );

				// Zaroven konstruujeme i spojovy seznam
				pSeg->NextSegIndex[0] = pSeg->NextSegIndex[1] = Index;

				// Posun pSeg na nalezeny segment (ukazuje nekam do pEntity.Field)
				pSeg = & pEntity->Segments[Index];
				
				if( pEntity->SegmentsUsed == MaxSegments )
					return( pEntity->SegmentsUsed );

				// zmena smeru hledani
				RandMod2 = rand() % 2;				
			}		
		}	

		// cyklus pres hledani segmentu k pridani - dozadu
		pSeg = & pEntity->Segments[0];
		pPrevSeg = pStartingSeg;
		while(  pPrevSeg->PrevSegIndex[0] >= 0  &&  pPrevSeg->PrevSegIndex[1] >= 0  )
		{		
			// vyber dalsiho segmentu podle puvodni nahodne volby smeru
			pPrevSeg = &Field[ pPrevSeg->PrevSegIndex[RandMod2] ];

			// Pokud je segment v pohode...
			if(  pPrevSeg->End.X < pSeg->Start.X  &&  pPrevSeg->End.Y < pSeg->Start.Y )
			{
				Index = (int)pEntity->AddSegment( *pPrevSeg );

				// Zaroven konstruujeme i spojovy seznam
				pSeg->PrevSegIndex[0] = pSeg->PrevSegIndex[1] = Index;

				// Posun pSeg na nalezeny segment (ukazuje nekam do pEntity.Field)
				pSeg = & pEntity->Segments[Index];
				
				if( pEntity->SegmentsUsed == MaxSegments )
					return( pEntity->SegmentsUsed );

				// zmena smeru hledani
				RandMod2 = rand() % 2;				
			}							
		}

		return( pEntity->SegmentsUsed );

	}

	
}; // class SegmentsPool







// Populace rodu
class SolutionEntitiesPopulation {
	public:

	ReAllocField<SolutionEntity> Entities;

	// A na tomhle populace roste
	CorrespondingLines Lines;

	// Metody
	inline void Initialize()
	{
		Entities.Initialize();
		Lines.pMasterLine = NULL;
		Lines.pSlaveLine = NULL;
	}
	inline void Release()
	{
		Entities.Release();		
		if( Lines.pMasterLine )
			free( Lines.pMasterLine );
		if( Lines.pSlaveLine )
			free( Lines.pSlaveLine );
		Initialize();
	}

	SolutionEntitiesPopulation()	{ Initialize(); }
	~SolutionEntitiesPopulation()	{ Release(); }


	bool Create(unsigned int EntitiesCount, const SimilarSegmentParameters &Params, const CorrespondingLines &_Lines)
	{
		SegmentsPool Pool;
		SolutionEntity Entity;
		unsigned int i;

		Release();
	
		// Alokace rodu
		if( !Entities.ReAlloc(EntitiesCount) )
			return( false );

		// Alokace korespondentnich car
		Lines.MasterLength = _Lines.MasterLength;
		Lines.SlaveLength = _Lines.SlaveLength;
		Lines.pMasterLine = (Color32 *)malloc( _Lines.MasterLength * sizeof(Color32) );
		Lines.pSlaveLine = (Color32 *)malloc( _Lines.SlaveLength * sizeof(Color32) );
		if( !Lines.pMasterLine || !Lines.pSlaveLine )
		{
			Release();
			return( false );		
		}
		// a jejich prekopirovani
		memcpy( Lines.pMasterLine, _Lines.pMasterLine, _Lines.MasterLength * sizeof(Color32) );
		memcpy( Lines.pSlaveLine, _Lines.pSlaveLine, _Lines.SlaveLength * sizeof(Color32) );

		// Vygenerujeme pool pro vytvoreni pocatecni populace
		if( !Pool.Generate(_Lines, Params) )
		{
			Release();
			return( false );		
		}

		// Pool setridime pro jednoduche vyhledavani jedincu
		if( !Pool.SortIt() )
		{
			Release();
			return( false );		
		}

		// a vygenerujeme pocatecni populaci
		for(i=0; i<EntitiesCount; i++)
			if( !Pool.GetRandomEntity(&Entity, Params.Entity_MaxSegments) )
			{
				Release();
				return( false );		
			}	
		
		return( true );
	}


	bool Exists()
	{
		return( Lines.pMasterLine && Lines.pSlaveLine && Entities.Field );
	}

/*	
	bool NextGeneration( int WhichStock = -1 )	
	{	
		if( !Exists() )
			return( false ); // zadna populace neexistuje		
		
		if( WhichStock < 0 )
		{
			// vsechny rody
			if( WhichStock == -1 )
			{				
				for(WhichStock=0; WhichStock<StocksCount; WhichStock++)
					if( !NextGeneration(WhichStock) )
						return( false );				
				return( true );
			}
			return( false );	
		}

		int Rate, ParentsWorstRate;
		unsigned int ParentsUsed, EntityIndex, ParentIndex;

		unsigned int   EntitiesInStockCount = Field[ WhichStock ].ItemsAllocated;
		pCurveEntity & EntitiesInStock      = Field[ WhichStock ].Field;
		unsigned int   MaxParents           = Field[ WhichStock ].ItemsAllocated / 4;			


		// vybereme rodice z rodu
		ParentsWorstRate = 1000000;
		ParentsUsed = 0;
		for(EntityIndex=0; EntityIndex<EntitiesInStockCount; EntityIndex++ )
		{
			Rate = EntitiesInStock[ EntityIndex ].CalculateRate( Lines, CurveParams );
			if( Rate <= 0 )
				continue;
			if( ParentsUsed < MaxParents )
			{
				// rodicu je jeste malo - bereme kazdeho
				Parents[ ParentsUsed ].Rate = Rate;
				Parents[ ParentsUsed ].Entity = EntityIndex;
				ParentsUsed++;

				if( Rate < ParentsWorstRate )
					ParentsWorstRate = Rate;
			}
			else
				if( Rate>ParentsWorstRate )
				{
					bool Placed = false;
					int PrevRate;
					int NewParentsWorstRate = 1000000;

					// rodicu je dost - bereme jen kvalitni
					// rovnou nalezneme noveho nejhorsiho
					for(ParentIndex=0; ParentIndex<ParentsUsed; ParentIndex++)
					{
						if( Rate > Parents[ParentIndex].Rate && !Placed )
						{
							Placed = true;

							PrevRate = Parents[ParentIndex].Rate;

							Parents[ParentIndex].Rate = Rate;
							Parents[ParentIndex].Entity = EntityIndex;

							// pokud se nevyhodil nejhorsi, nejhorsi tam stale je
							if( PrevRate != ParentsWorstRate )
								break;
						}

						if( Parents[ParentIndex].Rate < NewParentsWorstRate )
							NewParentsWorstRate = Parents[ParentIndex].Rate;
					}

					if( NewParentsWorstRate < ParentsWorstRate )
						ParentsWorstRate = NewParentsWorstRate;					
				}		

		} // for(EntityIndex...

		if( ParentsUsed == 0 )
			return( false );
		
		// do pomocneho pole jedincu Field[StocksCount] si prekopirujeme rodice
		for(ParentIndex=0; ParentIndex<ParentsUsed; ParentIndex++)
			Field[StocksCount].Field[ParentIndex] = EntitiesInStock[ Parents[ParentIndex].Entity ];

		// 1/4 populace
		// ted to prekopirujeme zpet do EntitiesInStock, zbytek prace se bude odehravat prave zde
		memcpy( EntitiesInStock, Field[StocksCount].Field, ParentsUsed * sizeof(CurveEntity) );
	
		// 1/4 populace
		// mutace rodicu
		for( EntityIndex = 0; EntityIndex < ParentsUsed; EntityIndex++ )
		{
			EntitiesInStock[ ParentsUsed + EntityIndex ] = EntitiesInStock[ EntityIndex ];
			EntitiesInStock[ ParentsUsed + EntityIndex ].Mutate( ParentMutation );
		}

		// 1/2 populace
		// a zbytek novych jedincu vznikne krizenim rodicu
		for( EntityIndex = 2*ParentsUsed; EntityIndex<EntitiesInStockCount; EntityIndex++ )
		{
			EntitiesInStock[ EntityIndex ].ChildOf( EntitiesInStock[rand()%ParentsUsed], EntitiesInStock[rand()%ParentsUsed] );
			EntitiesInStock[ EntityIndex ].Mutate( ChildMutation );
		}
		
		return( true );
		
	} //void NextGeneration(...

  



};




*/

bool ShowSimilarityMap(
	RenderedPicture             * pPicture,
	int                         Sensitivity,
	const Color32               * pMasterLine,
	const Color32               * pSlaveLine,
	int                         MasterLength,	
	int                         SlaveLength,
	int                         StripSize,
	int                         BarlettEyeSize
	);


bool CutTwoPhotos(
	Point2D MasLeftStartPos,
	const Camera &MasCam,
	const Camera &SlaCam,
	GeneralPhotosCut *pCut );






#endif // #ifndef _DERENDAI_H_INCLUDED_
