


#include "DeRender.h"
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

#include <crtdbg.h>
// Pokud selze #include<crtdbg.h>, vkladani souboru zruste, je potreba jen kvuli makru _ASSERT
#ifndef _ASSERT
	#define _ASSERT
#endif


// Deklarace funkci
bool DerenderPointEx(Point3D *pMasterPoint, Point3D *pSlavePoint, const DeRenderInfo &Info, Point2D MasterCameraPixel, Point2D SlaveCameraPixel);



// Funkce provede kalibraci kamery (zjisteni zoomu-ViewPlaneDistance) na zaklade informaci od uzivatele (parametr Info), 
// vysledek se vraci v promenne ViewPlaneDistance, hodnota parametru Info se nemeni. Pokud funkce uspeje, vraci true,
// jinak false.
bool GetViewPlaneDistance(double &ViewPlaneDistance, const CameraCalibrationInfo &Info)
{
	double E, F, G, H, I, J, K, Help1, Help2, ResXDiv2, ResYDiv2;

	// Zbezna kontrola parametru
	if((Info.AToBDistance <= 0) || (Info.ViewPointToADistance <= 0) || (Info.ViewPointToBDistance <= 0)
		|| (Info.Resolution.X <= 0) || (Info.Resolution.Y <= 0) 
		|| !IsTriangle(Info.AToBDistance, Info.ViewPointToADistance, Info.ViewPointToBDistance))
		return( false );

	ResXDiv2 = Info.Resolution.X / 2.0;
	ResYDiv2 = Info.Resolution.Y / 2.0;

	E = (sqr(Info.ViewPointToADistance) + sqr(Info.ViewPointToBDistance) - sqr(Info.AToBDistance)) 
		/ (2*Info.ViewPointToADistance*Info.ViewPointToBDistance);
	F = (Info.PixelA.X - ResXDiv2)*(Info.PixelB.X - ResXDiv2) + (Info.PixelA.Y - ResYDiv2)*(Info.PixelB.Y - ResYDiv2);
	G = sqr(Info.PixelA.X - ResXDiv2) + sqr(Info.PixelA.Y - ResYDiv2);
	H = sqr(Info.PixelB.X - ResXDiv2) + sqr(Info.PixelB.Y - ResYDiv2);
	I = E*G + E*H - 2*F;
	J = 2*(E - 1);
	K = sqr(I) - 4*(E-1)*(E*G*H - sqr(F));

	if(InBall(J, 0, SmallNum) || (K<0))
		return( false ); 

	K = sqrt(K);

	Help1 = (-I + K) / J;
	Help2 = (-I - K) / J;

	// A vybereme spravne reseni
	if((Help1 > 0) && (Help2 <= 0))
	{
		ViewPlaneDistance = sqrt(Help1);
		return( true );
	}
	else 
		if((Help1 <= 0) && (Help2 > 0))
		{
			ViewPlaneDistance = sqrt(Help2);
			return( true );
		}
		else
			return( false ); // Dve reseni (nebo zadne) je take chyba

}


bool GetSynchroTrianglePosition(Triangle3D &SynchroTriangle,
                                Point2D_Ex<double> MainControlPointPixel,
                                Point2D_Ex<double> HelpControlPointPixel,
                                Point2D_Ex<short> Resolution,
                                double ViewPlaneDistance,
                                double TrianSynchroPointToBackPointDist,
                                double TrianSynchroPointToHelpControlPointDist,
                                double TrianHelpControlPointToBackPointDist)
{
	Point3D MainControlPoint, HelpControlPoint;
	double ResXDiv2, ResYDiv2;
	double A, B, C, D, E, F, G, H1, H2, H3; // Pomocne promenne
	Point3D NullPoint; // Bod [0,0,0]
	struct { double r, s, t; } Params[4]; // Soustava rovnic pro nalezeni polohy synchronizacniho trojuhelniku ma 4 reseni, pouze jedno je spravne
	int i; // Pocitadlo do cyklu


	NullPoint.X = NullPoint.Y = NullPoint.Z = 0;	

	ResXDiv2 = (double)Resolution.X / 2.0;
	ResYDiv2 = (double)Resolution.Y / 2.0;

	// Prevedeme uzivatelske 2D informace do 3D - "fotku" jakoby posuneme po ose Z o jeji "zoom" (stred fotky take 
	// umistime na osu Z, rovina fotky je rovnobezna s rovinou os XY), aby zorny uhel (z bodu [0,0,0]) ve 3D 
	// prostoru odpovidal realnym parametrum kamery (to zavisi na spravne ViewPlaneDistance)
	// Pro spravnou funkci celeho derenderingu take bude potreba, aby kamery zachovavaly linearitu, 
	// t.j. obraz rovne cary na fotce bude opet rovny. 
	MainControlPoint.X = (double)MainControlPointPixel.X - ResXDiv2 + 0.5;
	MainControlPoint.Y = -((double)MainControlPointPixel.Y - ResYDiv2 + 0.5);
	MainControlPoint.Z = ViewPlaneDistance;  
	HelpControlPoint.X = (double)HelpControlPointPixel.X - ResXDiv2 + 0.5;
	HelpControlPoint.Y = -((double)HelpControlPointPixel.Y - ResYDiv2 + 0.5);
	HelpControlPoint.Z = ViewPlaneDistance;
 


	// *** Zjisteni polohy synchronizacniho trojuhleniku
	// Synchronizacni trojuhelnik je na fotce videt jako usecka, oba jeji krajni body zname (Main- a
	// HelpControlPoint)
	// velikost stran synchr. trojuhelniku take zname (parametry Trian...Dist), tedy vyresenim prislusne soustavy
	// tri kvadratickych rovnic (o trech neznamych-parametrech pro rovnici primky se zakladnim bodem [0,0,0]
	// a smerovymi vektory z [0,0,0] do prislusnych krajnich bodu usecky na fotce ve 3D 
	// (na promitaci rovine, tj. ViewPlane)) ziskame jeho polohu.
  
	// Priprava pomocnych promennych pro vypocet, body muzeme brat vzhledem k hlavnimu bodu pohledu [0,0,0]
	// i jako vektory, davame pozor, aby se nedelilo nulou nebo cislem blizkym k nule a aby se neodmocnovalo zaporne cislo.
	B = ScalarMultiply(MainControlPoint);
	C = ScalarMultiply(MainControlPoint, HelpControlPoint);
	A = ScalarMultiply(HelpControlPoint);  

	if(InBall(B, 0.0, SmallNum) || InBall(C, 0.0, SmallNum) || InBall(A, 0.0, SmallNum)) return( false );

	D = TrianSynchroPointToBackPointDist / sqrt( B );
	E = sqr(C);
	F = ( A*B - E ) / C;

	if(InBall(D, 0, SmallNum)) return( false );

	G = A * ( B*sqr(D)+sqr(TrianSynchroPointToHelpControlPointDist)-sqr(TrianHelpControlPointToBackPointDist) ) / ( 2*C*D );  
	H3 = E - A*B - sqr(F);  
	H1 = F*G;  
	H2 = sqr(H1) - H3*(A*sqr(TrianSynchroPointToHelpControlPointDist) - sqr(G));  

	if(InBall(H3, 0, SmallNum) || (H2 < 0)) return( false );

	H2 = sqrt(H2);

	// A vypocitame 4 mozna reseni, ty ulozime do Params...
	Params[0].s = Params[1].s = (H1 + H2) / H3;
	Params[2].s = Params[3].s = (H1 - H2) / H3;
	Params[0].t = Params[1].t = D + Params[0].s;
	Params[2].t = Params[3].t = D + Params[2].s;
  
	for(i=0; i<=2; i=i+2)
	{
		if(InBall(Params[i].s, 0, SmallNum)) return( false );  

		H2 = E - A*(B - sqr(TrianSynchroPointToHelpControlPointDist/Params[i].s)) ;
		H1 = Params[i].s / A;

		// Pokud by se melo odmocnovat zaporne cislo, je cela tato dvojice reseni neplatna, tedy do r ulozime -1 a pri vyberu
		// spravneho reseni nize se tato dvojice zamitne.
		if( H2 < 0 )
			Params[i].r = Params[i+1].r = -1; 		
		else
		{
			H2 = sqrt(H2);
			Params[i].r = H1 * (C + H2);
			Params[i+1].r = H1 * (C - H2);
		}
	}

	// Ze 4 reseni vybereme to jedine spravne
	for(i=0; i<=4; i++)
	{
		if( i==4 )
			return( false ); // Ani jedno z reseni neproslo = chyba

		if((Params[i].r > 0) && (Params[i].s > 0) && (Params[i].t > 0) && (Params[i].t > Params[i].s))
		{
			// Ted zjistime z vypocitanych parametru pro rovnici primky bodovou polohu synchronizacniho trojuhelniku		
			SynchroTriangle.A = AddVector(NullPoint, MainControlPoint, Params[i].s);    
			SynchroTriangle.B = AddVector(NullPoint, HelpControlPoint, Params[i].r);
			SynchroTriangle.C = AddVector(NullPoint, MainControlPoint, Params[i].t);
			// A zkontrolujeme, jestli ma spravne rozmery
			if(TriangleConformity(SynchroTriangle, 
			                      TrianHelpControlPointToBackPointDist,
			                      TrianSynchroPointToBackPointDist,
			                      TrianSynchroPointToHelpControlPointDist, SmallNum))
				break; // Ma spravne rozmery -> konec
		}     
	}  

	return(true);

}


// Funkce vytvori z informaci od uzivatele (SynchroInfo, MasterCalibrationInfo, SlaveCalibrationInfo, ...)
// zakladni parametry derender-sceny (Sol), hlavne vypocita vzajemnou polohu hlavni a vedlejsi kamery pomoci 
// synchronizacniho trojuhelniku (podle jeho obrazu a realnych prostorovych rozmeru). Vysledek se uklada
// do pSol. Pokud funkce uspeje, vraci se true, jinak false.
// Pokud MasterViewPlaneDistance (resp. SlaveViewPlaneDistance) je 0, potom se spravna hodnota
// vypocita z MasterCalibrationInfo (resp. SlaveCalibrationInfo)
bool CreateDeRenderInfo(DeRenderInfo *pSol, 
						const SynchronizationInfo &SynchroInfo, 					
						const CameraCalibrationInfo &MasterCalibrationInfo,
						const CameraCalibrationInfo &SlaveCalibrationInfo,
						double MasterViewPlaneDistance,
						double SlaveViewPlaneDistance)
{	
	DeRenderInfo &Sol = *pSol;
	Point3D NullPoint; // Bod [0,0,0]
	PointRTT ViewPRTT; // Relativni poloha bodu pohledu vedlejsi kamery vzhledem k synchr. troj.
	Triangle3D HelpTrian3D;

	NullPoint.X = NullPoint.Y = NullPoint.Z = 0;	
 
	// Zbezna kontrola parametru
	if((SynchroInfo.MCHCPToSCHCPDistance <= 0) 
	   || (SynchroInfo.SynchroPointToMCHCPDistance <= 0) 
	   || (SynchroInfo.SynchroPointToSCHCPDistance <= 0)
	   || !IsTriangle(SynchroInfo.MCHCPToSCHCPDistance, SynchroInfo.SynchroPointToMCHCPDistance, SynchroInfo.SynchroPointToSCHCPDistance))
		return( false ); 

	// *** Nastaveni hlavni (master) kamery
	// Hlavni bod pohledu definujme souradnicemi [0,0,0]
	Sol.MasterCamera.ViewPoint = NullPoint;

	// Provedem kalibraci obou kamer (zjistime jejich zoom, neboli ViewPlaneDistance), pokud je to potreba
	if( MasterViewPlaneDistance == 0 )
		if( !GetViewPlaneDistance(MasterViewPlaneDistance, MasterCalibrationInfo) )
			return (false);
	if( SlaveViewPlaneDistance == 0 )
		if( !GetViewPlaneDistance(SlaveViewPlaneDistance, SlaveCalibrationInfo) )
			return (false);


	// Zoom a rozliseni fotek jednoduse predame
	Sol.MasterCamera.ViewPlaneDistance = MasterViewPlaneDistance;
	Sol.MasterCamera.ResXDiv2 = (double)MasterCalibrationInfo.Resolution.X / 2.0;
	Sol.MasterCamera.ResYDiv2 = (double)MasterCalibrationInfo.Resolution.Y / 2.0;
	Sol.MasterCamera.Resolution = MasterCalibrationInfo.Resolution;

	// Zjistime polohu synchronizacniho trojuhelnika
	if( !GetSynchroTrianglePosition( Sol.SynchroTriangle,
	                                 SynchroInfo.MasterCameraMainControlPoint,
	                                 SynchroInfo.MasterCameraHelpControlPoint,
	                                 MasterCalibrationInfo.Resolution,
	                                 MasterViewPlaneDistance,
	                                 SynchroInfo.SynchroPointToSCHCPDistance,
	                                 SynchroInfo.SynchroPointToMCHCPDistance,
	                                 SynchroInfo.MCHCPToSCHCPDistance ) )	return( false );

	Sol.MasterCamera.BasicSynchroTriangle = Sol.SynchroTriangle;

	// *** Nastaveni vedlejsi (slave) kamery 
	// Pro vedlejsi kameru vypocitame polohu synchronizacniho trojuhelniku stejnym zpusobem jako u hlavni a
	// oba vysledky pote pres synchr. trojuhelnik slepime (hlavni zustane stejna, vedlejsi polohu zmeni)
	
	Sol.SlaveCamera.ViewPlaneDistance = SlaveViewPlaneDistance;

	Sol.SlaveCamera.ResXDiv2 = (double)SlaveCalibrationInfo.Resolution.X / 2.00;
	Sol.SlaveCamera.ResYDiv2 = (double)SlaveCalibrationInfo.Resolution.Y / 2.00;
	Sol.SlaveCamera.Resolution = SlaveCalibrationInfo.Resolution;

	if( !GetSynchroTrianglePosition( HelpTrian3D,
	                                 SynchroInfo.SlaveCameraMainControlPoint,
	                                 SynchroInfo.SlaveCameraHelpControlPoint,
	                                 SlaveCalibrationInfo.Resolution, 
	                                 SlaveViewPlaneDistance,
	                                 SynchroInfo.SynchroPointToMCHCPDistance,
	                                 SynchroInfo.SynchroPointToSCHCPDistance,
	                                 SynchroInfo.MCHCPToSCHCPDistance ) ) return( false );

	Sol.SlaveCamera.BasicSynchroTriangle.A = HelpTrian3D.A;
	Sol.SlaveCamera.BasicSynchroTriangle.B = HelpTrian3D.C;
	Sol.SlaveCamera.BasicSynchroTriangle.C = HelpTrian3D.B;
  


	// *** Slepeni vysledku 
	// Mame 2 synchr. troj. na ruznych mistech, 2 kamery v bode [0,0,0], chceme jeden synchr. troj. 
	// a dve kamery na ruznych mistech

	// Zjistime relativni polohu bodu pohledu slave-kamery v zakladni poloze k Sol.SlaveCamera.BasicSynchroTriangle, 
	// tuto relativni polohu pak aplikujeme na Sol.SynchroTriangle, a mame pozadovanou novou polohu bodu pohledu Slave kamery.
	if( !GetPointRelationToTriangle(Sol.SlaveCamera.BasicSynchroTriangle, NullPoint, &ViewPRTT))
		return( false );

	SetPointRelationToTriangle(Sol.SynchroTriangle, &Sol.SlaveCamera.ViewPoint, ViewPRTT);


	// A jeste do Sol pridame geometricky popis pohledove roviny master-kamery
	Sol.MasterCamera.Pixel00_Point.X = -Sol.MasterCamera.ResXDiv2 + 0.5;
	Sol.MasterCamera.Pixel00_Point.Y = -Sol.MasterCamera.ResYDiv2 + 0.5;
	Sol.MasterCamera.Pixel00_Point.Z = Sol.MasterCamera.ViewPlaneDistance;

	Sol.MasterCamera.XAxis_Vector.X = 1;
	Sol.MasterCamera.XAxis_Vector.Y = 0;
	Sol.MasterCamera.XAxis_Vector.Z = 0;

	Sol.MasterCamera.YAxis_Vector.X = 0;
	Sol.MasterCamera.YAxis_Vector.Y = 1;
	Sol.MasterCamera.YAxis_Vector.Z = 0;


	// ...a ted to same pro slave-kameru, to jiz bude trochu obtiznejsi
	Point3D Pixel_00, Pixel_10, Pixel_01; // Pomocne body v prostoru odpovidajici pixelum ze slave kamery v zakladni poloze
		
	Pixel_00.X = -Sol.SlaveCamera.ResXDiv2 + 0.5;
	Pixel_00.Y = -Sol.SlaveCamera.ResYDiv2 + 0.5;
	Pixel_00.Z = Sol.SlaveCamera.ViewPlaneDistance;

	Pixel_10 = Pixel_00;
	Pixel_10.X += 1.0;

	Pixel_01 = Pixel_00;
	Pixel_01.Y += 1.0;

	// Prenos pomocnych bodu ze zakladni do normalni polohy
	if( !GetPointRelationToTriangle(Sol.SlaveCamera.BasicSynchroTriangle, Pixel_00, &ViewPRTT) )
		return( false );
	SetPointRelationToTriangle(Sol.SynchroTriangle, &Pixel_00, ViewPRTT);
	
	if( !GetPointRelationToTriangle(Sol.SlaveCamera.BasicSynchroTriangle, Pixel_10, &ViewPRTT) )
		return( false );
	SetPointRelationToTriangle(Sol.SynchroTriangle, &Pixel_10, ViewPRTT);
	
	if( !GetPointRelationToTriangle(Sol.SlaveCamera.BasicSynchroTriangle, Pixel_01, &ViewPRTT) )
		return( false );
	SetPointRelationToTriangle(Sol.SynchroTriangle, &Pixel_01, ViewPRTT);		

	Sol.SlaveCamera.Pixel00_Point = Pixel_00;
	Sol.SlaveCamera.XAxis_Vector = GetVector(Pixel_00, Pixel_10);
	Sol.SlaveCamera.YAxis_Vector = GetVector(Pixel_00, Pixel_01);


	return( true );

}


// Funkce vypocita na zaklade 2D-poloh obrazu bodu Point na fotkach z MasterCamera a SlaveCamera nejpribliznejsi puvodni
// polohu bodu Point ve 3D prostoru. vyznam jednotlivych parametru je:
//      Point...............Sem vraci funkce vysledek-polohu bodu ve 3D
//      Info................Inforamce o derender-scene ziskane funkci CreateDerenderInfo
//      MasterCameraPixel...Souradnice obrazu bodu Point na fotce z MasterCamera   
//      SlaveCameraPixel....Souradnice obrazu bodu Point na fotce ze SlaveCamera
// Pokud funkce uspeje (derendering se povedl), vraci se true, jinak false.

bool DerenderPoint(Point3D *pPoint, const DeRenderInfo &Info, Point2D MasterCameraPixel, Point2D SlaveCameraPixel)
{
	Point3D MasterPoint, SlavePoint;

	if( !DerenderPointEx(&MasterPoint, &SlavePoint, Info, MasterCameraPixel, SlaveCameraPixel) )
		return( false );
	
	// Derenderovany bod necht je prumer dvou bodu, ktere jsou na primkach k sobe nejblize
	pPoint->X = (MasterPoint.X + SlavePoint.X) / 2.0;
	pPoint->Y = (MasterPoint.Y + SlavePoint.Y) / 2.0;
	pPoint->Z = (MasterPoint.Z + SlavePoint.Z) / 2.0;

	return( true );	
}

bool DerenderPointEx(Point3D *pMasterPoint, Point3D *pSlavePoint, const DeRenderInfo &Info, Point2D MasterCameraPixel, Point2D SlaveCameraPixel)
{
	StraightLine MasterStraightLine, SlaveStraightLine; // Primky z bodu pohledu jednotlivych kamer, na jejich pruniku by mel (teoreticky) byt bod Point
	Point3D HelpPoint; // Pomocnne body

	// *** Pripravime primky odpovidajici paprskum svetla leticim skrze MasterCameraPixel a SlaveCameraPixel
	MasterStraightLine.Point = Info.MasterCamera.ViewPoint;
	HelpPoint = AddVector( Info.MasterCamera.Pixel00_Point, Info.MasterCamera.XAxis_Vector, MasterCameraPixel.X );
	HelpPoint = AddVector( HelpPoint, Info.MasterCamera.YAxis_Vector, MasterCameraPixel.Y );
	MasterStraightLine.Vector = GetVector(MasterStraightLine.Point, HelpPoint);

	SlaveStraightLine.Point = Info.SlaveCamera.ViewPoint;
	HelpPoint = AddVector( Info.SlaveCamera.Pixel00_Point, Info.SlaveCamera.XAxis_Vector, SlaveCameraPixel.X );
	HelpPoint = AddVector( HelpPoint, Info.SlaveCamera.YAxis_Vector, SlaveCameraPixel.Y );
	SlaveStraightLine.Vector = GetVector(SlaveStraightLine.Point, HelpPoint);

	// *** Nalezneme bod, ktery je nejblize k Master- i SlaveStraightLine  
	if( !GetNearestPoints(MasterStraightLine, SlaveStraightLine, *pMasterPoint, *pSlavePoint) )
		return( false );

	return( true );
} 


/*
// Toto je puvodni verze funkce DerenderPoint(), vraci stejne vysledky jako pouzivana verze, jen je pomalejsi.
//
bool DerenderPoint(Point3D &Point, const DeRenderInfo &Info, Point2D MasterCameraPixel, Point2D SlaveCameraPixel)
{
	StraightLine MasterStraightLine, SlaveStraightLine; // Primky z bodu pohledu jednotlivych kamer, na jejich pruniku by mel (teoreticky) byt bod Point
	Point3D MasterPoint, SlavePoint, HelpPoint; // Pomocnne body
	PointRTT Relation;

	// *** Pripravime MasterStraightLine
	// MasterCamera je v zakladni poloze, hledame tedy primku prochazejici bodem [0,0,0] a bodem 
	// na fotce z teto kamery (ve 3D-viz. CreateDerenderInfo), ktery odpovida pixelu MasterCameraPixel.

	MasterStraightLine.Point.X = MasterStraightLine.Point.Y = MasterStraightLine.Point.Z = 0;
	MasterStraightLine.Vector.X = (double)MasterCameraPixel.X - Info.MasterCamera.ResXDiv2;
	MasterStraightLine.Vector.Y = -((double)MasterCameraPixel.Y - Info.MasterCamera.ResYDiv2); // Osa Y pro indexovani pixelu ma smer ze zhora dolu, osa Y v nasem 3D prostoru ma smer ze zdola nahoru
	MasterStraightLine.Vector.Z = Info.MasterCamera.ViewPlaneDistance;

	// *** Pripravime SlaveStraightLine
	// SlaveCamera neni v zakladni poloze, hledame tedy primku prochazejici bodem 
	// SlaveCamera.ViewPoint a bodem na fotce z teto kamery (ve 3D) odpovidajicim
	// pixelu SlaveCameraPixel. Ten nalezneme tak, ze zrelativizujeme jeho zakladni polohou k synchronizacnimu
	// trojuhelniku v zakladni poloze, a parametry teto relativizace pak aplikujeme na skutecny
	// synchronizacni trojuhelnik.

	SlaveStraightLine.Point = Info.SlaveCamera.ViewPoint;

	HelpPoint.X = (double)SlaveCameraPixel.X - Info.SlaveCamera.ResXDiv2;
	HelpPoint.Y = -((double)SlaveCameraPixel.Y - Info.SlaveCamera.ResYDiv2);
	HelpPoint.Z = Info.SlaveCamera.ViewPlaneDistance;
 
	if( !GetPointRelationToTriangle(Info.SlaveCamera.BasicSynchroTriangle, HelpPoint, &Relation) )
		return( false );

	SetPointRelationToTriangle(Info.SynchroTriangle, &HelpPoint, Relation);

	SlaveStraightLine.Vector = GetVector(SlaveStraightLine.Point, HelpPoint);

	// *** Nalezneme bod, ktery je nejblize k Master- i SlaveStraightLine
  
	if( !GetNearestPoints(MasterStraightLine, SlaveStraightLine, MasterPoint, SlavePoint) )
		return( false );

	double XXX=GetDistance(MasterPoint, SlavePoint);
//	if(XXX > max_odch)
//		max_odch = XXX;

//	prum_odch += XXX;
//	pocet++;

	// Derenderovany bod necht je prumer dvou bodu, ktere jsou na primkach k sobe nejblize
	Point.X = (MasterPoint.X + SlavePoint.X) / 2.0;
	Point.Y = (MasterPoint.Y + SlavePoint.Y) / 2.0;
	Point.Z = (MasterPoint.Z + SlavePoint.Z) / 2.0;

	return( true );
} //*/



struct MutationParameters {
	double MaxSpaceDisplacement, MaxVPDistanceDifference, MaxPixelDisplacement;
};

// Jedinec z populace pro genetickou kalibraci
class GeneticCalibrationEntity {

public:
	SynchronizationInfo SynchroInfo;
	double MasterViewPlaneDistance, SlaveViewPlaneDistance;
	bool ViewPlaneDistancesEqual;

	double Rate;

	bool CalculateRate( const DeRenderScene &DRScene, double *pRate );

	void ChildOf( const GeneticCalibrationEntity &Mother, const GeneticCalibrationEntity &Father );

	void Mutate( const MutationParameters &Params );
	
};


#define RATE_UNDEFINED	(1e10)
#define PARENTS_COUNT	(10)

class Population {

public:

	// Promenne
	GeneticCalibrationEntity *Entities;
	GeneticCalibrationEntity *NewEntities, *OldEntities;

	unsigned int EntitiesCount;

	GeneticCalibrationEntity FirstEntityInPopulation;

	// Metody
	inline void Initialize()
		{ Entities = NewEntities = OldEntities = NULL; EntitiesCount = 0;	}
	inline void Release()
		{ if( Entities ) free( Entities ); Initialize(); }

	Population()
	{
		Initialize();
	}

	~Population()
	{
		Release();
	}

	bool Create(unsigned int Count)
	{
		if( Count < PARENTS_COUNT*2 )
			return( false );

		Release();


		Entities = (GeneticCalibrationEntity *)malloc( 2 * Count * sizeof(GeneticCalibrationEntity) );		
		if( !Entities )
			return( false );
		NewEntities = Entities;
		OldEntities = Entities + Count;
		EntitiesCount = Count;
		return( true );
	}

	void MakeInitialPopulation(const GeneticCalibrationEntity &Pattern, const MutationParameters &Params)
	{	
		NewEntities[0] = Pattern;

		for(int Index=1; Index<EntitiesCount; Index++)
		{
			NewEntities[ Index ] = Pattern;
			NewEntities[ Index ].Mutate( Params );
		}
		FirstEntityInPopulation = Pattern;
	}

	void NextGeneration( const DeRenderScene &DRScene,
	                     MutationParameters &LowMutation,
	                     MutationParameters &MediumMutation,
	                     MutationParameters &HighMutation )
	{
		///const ParentsCount = 10;

		MySwap( NewEntities, OldEntities );

		struct { int Index; double Rate; } Parents[ PARENTS_COUNT ];
		double WorstParentRate;
		double Rate, PrevRate;

		unsigned int i,j,k;

		for(i=0; i<PARENTS_COUNT; i++)
			Parents[ i ].Rate = RATE_UNDEFINED;
		WorstParentRate = RATE_UNDEFINED;
	
		// *** Vybereme nejlepsi jedince teto generace
		for(i=0; i<EntitiesCount; i++)
		{
			if( !OldEntities[ i ].CalculateRate( DRScene, &Rate ) )
				continue;
		
			// Tato entita se zaradi mezi rodice
			if( Rate < WorstParentRate )
			{
				// Nalezneme koho nahradi
				for(j=0; j<PARENTS_COUNT; j++)
				{
					// Pokud je tento rodic horsi, nahradme ho
					if( Rate < Parents[j].Rate )
					{
						PrevRate = Parents[j].Rate;

						// Nahrada
						Parents[j].Rate  = Rate;
						Parents[j].Index = i;

						// Vyhazujeme nejhorsiho rodice ?
						if( InBall(PrevRate,WorstParentRate,SmallNum) )
						{
							WorstParentRate = 0; 

							// Nalezneme noveho nejhorsiho rodice
							for(k=0; k<PARENTS_COUNT; k++)
								if( Parents[k].Rate > WorstParentRate )
									WorstParentRate = Parents[k].Rate;
						}

						break;
					}
				} // for(j=0...

				_ASSERT( j != PARENTS_COUNT ); // Ze by nikdo nebyl nahrazen ???
			} // if( Rate <...

		} // for(i=0...

		_ASSERT( WorstParentRate != RATE_UNDEFINED );
		
		// *** Vytvoreni nove generace
		unsigned int Ent_ToCopy = 2*PARENTS_COUNT,
		             Ent_BestParents = ((EntitiesCount - Ent_ToCopy) * 6) / 10, // 50 %
		             Ent_RandomParents = EntitiesCount - Ent_ToCopy - Ent_BestParents;

		// Prekopirovani nejlepsich rodicu a jejich mutaci
		for(i=0; i<PARENTS_COUNT; i++)
		{
			NewEntities[i] = OldEntities[ Parents[i].Index ];			

			NewEntities[PARENTS_COUNT+i] = OldEntities[ Parents[i].Index ];
			NewEntities[PARENTS_COUNT+i].Mutate( LowMutation );
		}
	
		// Deti od nejlepsich rodicu - LowMutation
		for(i=Ent_ToCopy; i< Ent_ToCopy+Ent_BestParents; i++)
		{
			NewEntities[ i ].ChildOf( OldEntities[ Parents[ rand()%PARENTS_COUNT ].Index ],
			                          OldEntities[ Parents[ rand()%PARENTS_COUNT ].Index ] );
			NewEntities[ i ].Mutate( LowMutation );
		}

		// Deti nahodnych rodicu
		int RandomParentsTried = 0;
		for(i=Ent_ToCopy+Ent_BestParents; i< EntitiesCount; i++)
		{			
			j = rand() % EntitiesCount;
			k = rand() % EntitiesCount;
			
			if( OldEntities[j].Rate==RATE_UNDEFINED || OldEntities[k].Rate==RATE_UNDEFINED )
			{
				// Rodice nejsou korektni, vytvorime zcela noveho jedince
				// podle pocatecniho jedince - HighMutation
				NewEntities[i] = FirstEntityInPopulation;
				NewEntities[i].Mutate( HighMutation );
			}
			else
			{
				// Rodice jsou v pohode - MediumMutation
				NewEntities[i].ChildOf( OldEntities[i], OldEntities[j] );
				NewEntities[i].Mutate( MediumMutation );
			}			
		}
		
	} //void NextGeneration(...

	bool GetBestEntity(const DeRenderScene &DRScene, GeneticCalibrationEntity *pBestEntity )
	{
		unsigned int BestIndex = EntitiesCount;
		double Rate, BestRate = RATE_UNDEFINED;

		for(unsigned int i=0; i<EntitiesCount; i++)
		{
			if( !NewEntities[i].CalculateRate( DRScene, &Rate ) )
				continue;
			
			if( Rate < BestRate )
			{
				BestRate = Rate;
				BestIndex = i;
			}
		}

		if( BestIndex == EntitiesCount )
			return( false );

		FirstEntityInPopulation.CalculateRate( DRScene, &Rate );

		*pBestEntity = NewEntities[ BestIndex ];

		double x;
		pBestEntity->CalculateRate( DRScene, &x );

		return( true );
	}


};


bool GeneticCalibrationEntity::CalculateRate(const DeRenderScene &DRScene, double *pRate)
{
	int Index;
	Point3D MasterPoint, SlavePoint;
	DeRenderInfo DRInfo;
	double Val = 0;
	Point2D MasterPixel, SlavePixel;

	
	if( !CreateDeRenderInfo( &DRInfo,
	                         SynchroInfo,
	                         DRScene.MasterCalibInfo,
	                         DRScene.SlaveCalibInfo,
	                         MasterViewPlaneDistance,
	                         SlaveViewPlaneDistance ) )
	{
		Rate = RATE_UNDEFINED;
		return( false );
	}

	for(Index=0; Index<DRScene.NumOfEqualPoints; Index++)
	{	
		if( !DerenderPointEx( &MasterPoint,
			                  &SlavePoint,
			                  DRInfo,
			                  DRScene.EqualPoints[Index].MasterCameraPixel,
			                  DRScene.EqualPoints[Index].SlaveCameraPixel ) )
		{
			_ASSERT(false);
			Rate = RATE_UNDEFINED;
			return( false );
		}

		Val += GetDistance(MasterPoint, SlavePoint);
/*
Toto funguje !!!:
	Point3D p3, p5, p6;

		switch(Index)
		{
		case 2:
			p3.X = (MasterPoint.X + SlavePoint.X) / 2;
			p3.Y = (MasterPoint.Y + SlavePoint.Y) / 2;
			p3.Z = (MasterPoint.Z + SlavePoint.Z) / 2;
			break;
		case 9:
			p5.X = (MasterPoint.X + SlavePoint.X) / 2;
			p5.Y = (MasterPoint.Y + SlavePoint.Y) / 2;
			p5.Z = (MasterPoint.Z + SlavePoint.Z) / 2;
			break;
		case 8:
			p6.X = (MasterPoint.X + SlavePoint.X) / 2;
			p6.Y = (MasterPoint.Y + SlavePoint.Y) / 2;
			p6.Z = (MasterPoint.Z + SlavePoint.Z) / 2;
			break;
		} */

	}


//	Val += fabs( ScalarMultiply( GetVector(p3,p5), GetVector(p6,p5) ) );

	//Val *= RATING_LENGTH_UNIT_METER;
	*pRate = Rate = Val;	
	return( true );
}


inline Point2D_Ex<double> AveragePoint2D_Ex(const Point2D_Ex<double> &PointA, const Point2D_Ex<double> &PointB)
{
	Point2D_Ex<double> Res;
	Res.X = (PointA.X + PointB.X) / 2;
	Res.Y = (PointA.Y + PointB.Y) / 2;	
	return( Res );
}

inline double Average1D(double A, double B)
{
	return( (A+B) / 2 );
}



void GeneticCalibrationEntity::ChildOf( const GeneticCalibrationEntity &Mother, const GeneticCalibrationEntity &Father )
{
	SynchroInfo.MasterCameraHelpControlPoint = AveragePoint2D_Ex( Mother.SynchroInfo.MasterCameraHelpControlPoint, Father.SynchroInfo.MasterCameraHelpControlPoint);
	SynchroInfo.MasterCameraMainControlPoint = AveragePoint2D_Ex( Mother.SynchroInfo.MasterCameraMainControlPoint, Father.SynchroInfo.MasterCameraMainControlPoint);	
	SynchroInfo.SlaveCameraHelpControlPoint  = AveragePoint2D_Ex( Mother.SynchroInfo.SlaveCameraHelpControlPoint,  Father.SynchroInfo.SlaveCameraHelpControlPoint);
	SynchroInfo.SlaveCameraMainControlPoint  = AveragePoint2D_Ex( Mother.SynchroInfo.SlaveCameraMainControlPoint,  Father.SynchroInfo.SlaveCameraMainControlPoint);

	SynchroInfo.MCHCPToSCHCPDistance         = Average1D( Mother.SynchroInfo.MCHCPToSCHCPDistance,        Father.SynchroInfo.MCHCPToSCHCPDistance);
	SynchroInfo.SynchroPointToMCHCPDistance  = Average1D( Mother.SynchroInfo.SynchroPointToMCHCPDistance, Father.SynchroInfo.SynchroPointToMCHCPDistance);
	SynchroInfo.SynchroPointToSCHCPDistance  = Average1D( Mother.SynchroInfo.SynchroPointToSCHCPDistance, Father.SynchroInfo.SynchroPointToSCHCPDistance);
	

	MasterViewPlaneDistance = Average1D(Mother.MasterViewPlaneDistance, Father.MasterViewPlaneDistance);
	SlaveViewPlaneDistance = Average1D(Mother.SlaveViewPlaneDistance, Father.SlaveViewPlaneDistance);

	ViewPlaneDistancesEqual = Mother.ViewPlaneDistancesEqual;
}


void GeneticCalibrationEntity::Mutate( const MutationParameters &Params )
{
	SynchroInfo.MasterCameraMainControlPoint.X += (double)(rand()-RAND_MAX/2) * Params.MaxPixelDisplacement / RAND_MAX;
	SynchroInfo.MasterCameraMainControlPoint.Y += (double)(rand()-RAND_MAX/2) * Params.MaxPixelDisplacement / RAND_MAX;
	SynchroInfo.MasterCameraHelpControlPoint.X += (double)(rand()-RAND_MAX/2) * Params.MaxPixelDisplacement / RAND_MAX;
	SynchroInfo.MasterCameraHelpControlPoint.Y += (double)(rand()-RAND_MAX/2) * Params.MaxPixelDisplacement / RAND_MAX;

	SynchroInfo.SlaveCameraMainControlPoint.Y += (double)(rand()-RAND_MAX/2) * Params.MaxPixelDisplacement / RAND_MAX;
	SynchroInfo.SlaveCameraMainControlPoint.X += (double)(rand()-RAND_MAX/2) * Params.MaxPixelDisplacement / RAND_MAX;
	SynchroInfo.SlaveCameraHelpControlPoint.X += (double)(rand()-RAND_MAX/2) * Params.MaxPixelDisplacement / RAND_MAX;
	SynchroInfo.SlaveCameraHelpControlPoint.Y += (double)(rand()-RAND_MAX/2) * Params.MaxPixelDisplacement / RAND_MAX;

	SynchroInfo.MCHCPToSCHCPDistance += (double)(rand()-RAND_MAX/2) * Params.MaxSpaceDisplacement / RAND_MAX;
	SynchroInfo.SynchroPointToMCHCPDistance += (double)(rand()-RAND_MAX/2) * Params.MaxSpaceDisplacement / RAND_MAX;
	SynchroInfo.SynchroPointToSCHCPDistance += (double)(rand()-RAND_MAX/2) * Params.MaxSpaceDisplacement / RAND_MAX;

	MasterViewPlaneDistance += (double)(rand()-RAND_MAX/2) * Params.MaxVPDistanceDifference / RAND_MAX;

	if( MasterViewPlaneDistance < 100 )
		MasterViewPlaneDistance = 100;

	if( ViewPlaneDistancesEqual )
		SlaveViewPlaneDistance = MasterViewPlaneDistance;
	else
	{
		SlaveViewPlaneDistance += (double)(rand()-RAND_MAX/2) * Params.MaxVPDistanceDifference / RAND_MAX;
		if( SlaveViewPlaneDistance < 100 )
			SlaveViewPlaneDistance = 100;
	}

}



bool GeneticallyCalibrateDeRenderScene(const DeRenderScene &DRScene, unsigned int GenerationsCount, DeRenderInfo *pSolution )
{
	Population Pop;
	MutationParameters LowMutation, MediumMutation, HighMutation;
	double MasterVPDistance, SlaveVPDistance;

	GeneticCalibrationEntity Pattern, BestEntity;

	if( !GetViewPlaneDistance( MasterVPDistance, DRScene.MasterCalibInfo ) )
		return( false );
	if( !GetViewPlaneDistance( SlaveVPDistance, DRScene.SlaveCalibInfo ) )
		return( false );

	Pattern.MasterViewPlaneDistance = MasterVPDistance;
	Pattern.SlaveViewPlaneDistance = SlaveVPDistance;
	Pattern.SynchroInfo = DRScene.SynchroInfo;
	Pattern.ViewPlaneDistancesEqual = InBall( MasterVPDistance, SlaveVPDistance, 1e-5 );

	LowMutation.MaxPixelDisplacement = 0.3;
	LowMutation.MaxSpaceDisplacement = DRScene.SynchroInfo.MCHCPToSCHCPDistance / 200;
	LowMutation.MaxVPDistanceDifference = 400;

	MediumMutation.MaxPixelDisplacement = 0.5;
	MediumMutation.MaxSpaceDisplacement = DRScene.SynchroInfo.MCHCPToSCHCPDistance / 70;
	MediumMutation.MaxVPDistanceDifference = 2000;

	HighMutation.MaxPixelDisplacement = 1;
	HighMutation.MaxSpaceDisplacement = DRScene.SynchroInfo.MCHCPToSCHCPDistance / 50;
	HighMutation.MaxVPDistanceDifference = 3000;

	if( !Pop.Create( 400 ) )
		return( false );

	Pop.MakeInitialPopulation( Pattern, HighMutation );

	for(int i=0; i<GenerationsCount; i++)
	{
		Pop.NextGeneration( DRScene, LowMutation, MediumMutation, HighMutation );
	}
	
	if( !Pop.GetBestEntity(DRScene, &BestEntity) )
		return( false );

	return( CreateDeRenderInfo(pSolution, BestEntity.SynchroInfo, DRScene.MasterCalibInfo, DRScene.SlaveCalibInfo, BestEntity.MasterViewPlaneDistance, BestEntity.SlaveViewPlaneDistance) );
		

}






/* Format DRS (DeRender Scene) souboru verze 1.00 (prehled znamych vyrazu):

  # komentar ...

  # Souradnice [0,0] odpovida pixelu v levem hornim rohu, [maxx, maxy] v pravem dolnim rohu !!!

  # Cislo verze - povinne
  Version( 1.00 )	

  # Informace potrebne pro synchronizaci (urceni vzajemne polohy master- a slave-camery) - povinne 1 vyskyt
  SynchronizationInfo (
	# Souradnice urcitych pixelu na fotkach:
	524, 658,		# X-, Y-ova souradnice obrazu synchronizacniho bodu na fotce z master-camery
	102, 837,		# X-, Y-ova souradnice obrazu pomocneho ridiciho bodu na fotce z master-camery
	45,	470,		# X-, Y-ova souradnice obrazu synchronizacniho bodu na fotce ze slave-camery
	341, 541,		# X-, Y-ova souradnice obrazu pomocneho ridiciho bodu na fotce ze slave-camery

	# Realne prostorove rozmery synchronizacniho trojuhelniku. Musi to byt ve stejnych jednotkach,
	# jake jsou pouzity pro kalibraci kamer (vyraz CalibrationInfo) !!! Konvence je uvadet vse v metrech.
	0.2125 ,		# Vzdalenost synchronizacniho bodu od pomocneho ridiciho bodu master-camery
	0.187 ,			# Vzdalenost synchronizacniho bodu od pomocneho ridiciho bodu slave-camery
	0.2461			# Vzdalenost pomocnych ridicich bodu master- a slave-camery
		)
  	
  # Informace pro kalibraci jedne z kamer (zjisteni "zoomu") - povinne 2 vyskyty (pro master- a slave-kameru)
  CamerasInfo (
	1,				# Ktere kamery kalibracni informace (1=master, 2=slave)

	128, 1015,		# X-ova, Y-ova souradnice obrazu bodu A (na fotce)
	540, 912,		# X-ova, Y-ova souradnice obrazu bodu B (na fotce)

	4.564,			# Vzdalenost bodu pohledu (mista v kamere, kde se paprsky svetla krizuji, obvykle 
					# uprostred clony) od bodu A. Jednotky stejne jako ve vyrazu SynchronizationInfo !!!
	3.678,			# Vzdalenost bodu pohledu od bodu B
	4.210,			# Vzdalenost bodu A a B

	1024, 768		# X-, Y-ove rozliseni fotek	
		)


  # Volitelny vyraz, maximalne 1 vyskyt
  # Soubory s fotkami z master a slave kamery (v tomto poradi), budou uzity pro otexturovani trojuhelniku.
  # V uvozovkach jsou relativni cesty od tohoto DRS souboru k BMP souborum, kazda musi mit mene nez 80 znaku (DRS_MAXWORDLENGTH)
  # Pokud je nektery z parametru prazdny retezec (""), prislusna fotka se nebude nacitat
  UseBitmaps("..\Pictures\Master.bmp", "Slave.bmp") 


  # Body pro DeRendering, na jejich poradi zalezi, jsou indexovany vzestupne od 0 - viz vyraz EqualTriangle
  
  # bod 0:	
  EqualPoint(
	347, 841,		# X-ova, Y-ova souradnice obrazu bodu na fotce z master-camery
	412, 304		# X-ova, Y-ova souradnice obrazu bodu na fotce ze slave-camery
	3,				# Velikost bodu
	6794245			# Barva bodu ve tvaru (RED*65536 + GREEN*256 + BLUE)
					# RED, GREEN i BLUE jsou relativni intenzity barev z rozmezi 0...255
					)

  EqualPoint(647, 881, 382, 42, 4, 6794245 )			# bod 1  
  EqualPoint(643, 152, 874, 146, 4, 6794245	)			# bod 2	


  # Trojuhelniky pro DeRendering
  EqualTriangle(
	0,				# Index bodu A (cislo znacici ktery EqualPoint se pro tetno bod pouzije, 
					# 0=prvni EqualPoint v souboru, 1=druhy, 2=treti, atd...)
	2,				# Index bodu B
	1,				# Index bodu C
	1				# Ze ktere kamery se bere textura (0=bez textury, 1=master, 2=slave),
					# ma vyznam jen pokud je uzit vyraz UseBitmaps
	)

*/






// Funkce nacte ze souboru File otevreneho pro cteni (textovy rezim) dalsi zname slovo nebo specialni znak.
// Pokud se jedna o jmeno nebo cislo, je toto ulozeno do retezce Word, jeho maximalni delka (i pro 0 na konci)
// se predava parametrem MaxLength. Mezery, tabelatory a znaky noveho radku se automaticky preskakuji. 
// Funkce vraci bud 0 = chyba, nebo nektery z nize uvedenych navratovych kodu (makra RW_...):
#define RW_ERROR			0	// Chyba - neznamy znak, preteceni bufferu nebo chyba parametru
#define RW_EOF				1	// Konec souboru
#define RW_NOTHING			2	// Komentar preskocen - nic noveho nenacteno, lze pokracovat dal 
#define RW_NAME				3	// Nacteno jmeno (znaky a-zA-Z a podtrzitka) do Word
#define RW_NUMBER			4	// Nacteno cislo (znaky 0-9 a .) do Word
#define RW_LPARENT			5	// Nactena leva zavorka (
#define RW_RPARENT			6	// Nactena prava zavorka )
#define RW_COMMA			7	// Nactena carka ,
#define RW_QUOTATIONTEXT	8	// Nacten text v uvozovkach do Word
// Jakmile funkce jednou zacne cist soubor, musi v tom pokracovat (do nejake chyby nebo do uspesneho konce),
// protoze nekdy je funkci predem nacten jeden znak, ulozen do staticke promenne, a pouzit az dalsim volanim.
// Diky ni se funkce NESMI!!! volat ze dvou ruznych threadu najednou!!!
int ReadWord(FILE *File, char *Word, int MaxLength)
{
	int Character, Index = 0;
	static int PrevChar(0); // Musi byt compilerem inicializovan na 0 !!!

	if ((MaxLength <= 1) || !Word || !File)
		return (RW_ERROR);

	// Zustal nam charakter z minula ?
	if (PrevChar != 0)
	{
		Character = PrevChar; 
		PrevChar = 0;
	}
	else
		Character = ' '; // Tak zas odzacatku

	// Preskocime vsechny mezery, '\n', '\t' a komentare
	while(true)
	{
		// Preskocime mezery a znaky '\n', '\t' (tyto jsou urcite mimo komentar)
		while ((Character == ' ') || (Character == '\n') || (Character == '\t'))
			Character = fgetc( File );

		if (Character == EOF)
			return (RW_EOF);

		// Preskocime i komentar do konce radky
		if (Character == '#')
		{
			while ((Character != '\n') && (Character != EOF))
				Character = fgetc( File );
			
			if (Character == EOF)
				return (RW_EOF);					
		}
		else
			break;
	}
	

	// Neni to pismeno nebo podtrzitko ?
	while ( ((('a' <= Character) && (Character <= 'z')) || (('A' <= Character) && (Character <= 'Z')) || (Character == '_')) 
			&& (Character != EOF) && (Index < MaxLength))
	{
		Word[ Index ] = (char)Character;
		Index++;
		Character = fgetc( File );			
	}
	if (Index)
	{
		if ((Index >= MaxLength) || (Character == 0))
			return (RW_ERROR); // Prekrocena velikost Bufferu nebo zakazany znak 0		
		PrevChar = Character; // Posledni nacteny znak nechame na priste
		Word[ Index ] = 0; // Ukoncime navratovy retezec se jmenem
		return (RW_NAME);
	}


	// Neni to ani cislo ?
	while ( ((('0' <= Character) && (Character <= '9')) || (Character == '.')) 
	        && (Character != EOF) && (Index < MaxLength) )	 
	{
		Word[ Index ] = (char)Character;
		Index++;
		Character = fgetc( File );			
	}
	if (Index)
	{
		if ((Index >= MaxLength) || (Character == 0))
			return (RW_ERROR); // Prekrocena velikost bufferu nebo zakazany znak 0	
		PrevChar = Character; // Posledni znak nechame na priste
		Word[ Index ] = 0; // Ukoncime navratovy retezec s cislem
		return (RW_NUMBER);
	}


	// Ze by tedy nejaky specialni znak ?
	switch (Character)
	{
		case '(': return (RW_LPARENT);
		case ')': return (RW_RPARENT);
		case ',': return (RW_COMMA);			
			
		case '"': // Zacina nam text v uvozovkach
			Character = fgetc( File );
			while((Index < MaxLength) && (Character != '"') && (Character != EOF))
			{
				Word[Index] = (char)Character;
				Index++;
				Character = fgetc( File );
			}
			if((Index >= MaxLength) || (Character == EOF))
				return( RW_ERROR ); // Prekrocena velikost bufferu nebo predcasny konec souboru
			Word[Index] = 0; // Ukoncime navratovy retezec s textem
			return(RW_QUOTATIONTEXT);

		default: return (RW_ERROR);	// Neznamy znak
	}
}



// Funkce nacte z jiz (textove) otevreneho souboru File dalsi DRS (DeRender Scene) vyraz (ten je tvaru
// NAME ( NUM1, NUM2, ... ), nebo USEBITMAPS("SOUBOR1", "SOUBOR2") ), jeho parametry ulozi do Buffer. 
// BufferElems je maximalni pocet cisel typu double, ktere se vejdou do Bufferu.
// Funkce vraci nasledujici makra (typy vyrazu):
#define DRS_ERROR				0	// Chyba parametru, neznamy vyraz, chyba syntaxe v souboru nebo jina chyba
#define DRS_EOF					1	// Konec souboru
#define DRS_VERSION				2	// Do Bufferu[0] ulozeno cislo verze
#define DRS_CALIBRATIONINFO		3	// Do Bufferu[0-9] ulozeny kalibracni informace
#define DRS_SYNCHROINFO			4	// Do Bufferu[0-10] ulozeny synchronizacni informace
#define DRS_EQUALTRIANGLE		5	// Do Bufferu[0-11] ulozeny ekvivalentni trojuhelniky
#define DRS_EQUALPOINT			6	// Do Bufferu[0-3] ulozeny ekvivalentni body
#define DRS_USEBITMAPS			7	// Do Bufferu[0-19] ulozeny jmena dvou souboru( [0-9] prvni, [10-19] druhy )
// Makra s poctem parametru daneho vyrazu
#define DRS_VERSION_PARAMS				1
#define DRS_CALIBRATIONINFO_PARAMS		10
#define DRS_SYNCHROINFO_PARAMS			11
#define DRS_EQUALTRIANGLE_PARAMS		4
#define DRS_EQUALPOINT_PARAMS			6
#define DRS_USEBITMAPS_PARAMS			20 // Ve skutecnosti velikost pole pro ulozeni cest k souborum
// Makra s nazvem vyrazu - velkym pismem !
#define DRS_VERSION_NAME				"VERSION"
#define DRS_CALIBRATIONINFO_NAME		"CALIBRATIONINFO"
#define DRS_SYNCHROINFO_NAME			"SYNCHRONIZATIONINFO"
#define DRS_EQUALTRIANGLE_NAME			"EQUALTRIANGLE"
#define DRS_EQUALPOINT_NAME				"EQUALPOINT"
#define DRS_USEBITMAPS_NAME				"USEBITMAPS"
// Maximalni delka slova (vcetne znaku 0 na konci)
#define	DRS_MAXWORDLENGTH	80

int ReadDRSExpression(FILE *File, double *Buffer, unsigned int BufferElems)
{
	int WordType, RetValue;
	unsigned int NumOfParamsToRead, Index;
	char WordBuffer[DRS_MAXWORDLENGTH];
	double Number;

	if(!File || !Buffer || !BufferElems)
		return(DRS_ERROR);
	
	// Ted bychom pri spravnem prubehu meli nacist jmeno dalsiho vyrazu nebo je soubor na konci
	WordType = ReadWord(File, WordBuffer, DRS_MAXWORDLENGTH);
	if (WordType == RW_EOF)
		return (DRS_EOF);
	if (WordType != RW_NAME)
		return (DRS_ERROR);

	// Mala/velka pismena nehraji roli
	strupr(WordBuffer);
	
	// Ted by tam musi byt leva zavorka - pokud je, WordBuffer zustane nezmenen
	WordType = ReadWord(File, WordBuffer, DRS_MAXWORDLENGTH);
	if (WordType != RW_LPARENT)
		return (DRS_ERROR);

	// Specialni vyraz UseBitmaps
	if(!strcmp(WordBuffer, DRS_USEBITMAPS_NAME))
	{
		// Je buffer dost veliky
		if(BufferElems < 20)
			return( DRS_ERROR );

		// Dalsi slovo musi nutne byt typu RW_QUOTATIONTEXT
		if(ReadWord(File, WordBuffer, DRS_MAXWORDLENGTH)!= RW_QUOTATIONTEXT)
			return( DRS_ERROR );
		if(strlen(WordBuffer) >= 80)
			return( DRS_ERROR );
		strcpy((char *)Buffer, WordBuffer);

		// Dalsi slovo musi byt carka
		if(ReadWord(File, WordBuffer, DRS_MAXWORDLENGTH)!= RW_COMMA)
			return( DRS_ERROR );

		// Ted musi byt slovo opet typu RW_QUOTATIONTEXT
		WordType = ReadWord(File, WordBuffer, DRS_MAXWORDLENGTH);
		if(WordType != RW_QUOTATIONTEXT)
			return( DRS_ERROR );
		if(strlen(WordBuffer) >= 80)
			return( DRS_ERROR );
		strcpy((char *)(Buffer+10), WordBuffer);

		// A posledni slovo musi byt RW_RIGHTPARENT
		if (ReadWord(File, WordBuffer, DRS_MAXWORDLENGTH) != RW_RPARENT)
			return( DRS_ERROR );

		return( DRS_USEBITMAPS );
	}


	// Zjisteni typu cteneho vyrazu
	if (!strcmp(WordBuffer, DRS_EQUALTRIANGLE_NAME))
	{
		NumOfParamsToRead = DRS_EQUALTRIANGLE_PARAMS;
		RetValue = DRS_EQUALTRIANGLE;
	}
	else if (!strcmp(WordBuffer, DRS_EQUALPOINT_NAME))
		{
			NumOfParamsToRead = DRS_EQUALPOINT_PARAMS;
			RetValue = DRS_EQUALPOINT;
		}
			else
			if (!strcmp(WordBuffer, DRS_VERSION_NAME))
			{
				NumOfParamsToRead = DRS_VERSION_PARAMS;
				RetValue = DRS_VERSION;
			}	
			else
				if (!strcmp(WordBuffer, DRS_SYNCHROINFO_NAME))
				{
					NumOfParamsToRead = DRS_SYNCHROINFO_PARAMS;
					RetValue = DRS_SYNCHROINFO;
				}
				else
					if (!strcmp(WordBuffer, DRS_CALIBRATIONINFO_NAME))
					{
						NumOfParamsToRead = DRS_CALIBRATIONINFO_PARAMS;
						RetValue = DRS_CALIBRATIONINFO;
					}
					else
						return (DRS_ERROR);

	// Muze pretect Buffer ?
	if (NumOfParamsToRead > BufferElems)
		return (DRS_ERROR); 

	// Nacteme parametry DRS vyrazu do Bufferu
	for (Index = 0; Index < NumOfParamsToRead; Index++)
	{
		// Ted to musi byt cislo
		WordType = ReadWord(File, WordBuffer, DRS_MAXWORDLENGTH);
		if (WordType != RW_NUMBER)
			return (DRS_ERROR);
	
		// Tak ho prevedeme z retezce do double a ulozime do Bufferu
		if ((WordBuffer[0] == '0') && (WordBuffer[1] == 0)) 
			Number = 0.0;
		else
		{	
			Number = atof(WordBuffer); 
			if (Number == 0.0) // 0.0 indikuje chybu formatu retezce s cislem
				return (DRS_ERROR);
		}		
		Buffer[ Index ] = Number;

		// Ted to musi byt carka nebo pokud to byl jiz posledni parametr, tak prava zavorka
		WordType = ReadWord(File, WordBuffer, DRS_MAXWORDLENGTH);
		if (WordType != RW_COMMA)
			if ((WordType == RW_RPARENT) &&	(Index == (NumOfParamsToRead - 1)))
				return (RetValue);
			else
				return (DRS_ERROR);		
	}

	// Sem se lze dostat jen pokud vyraz nekonci pravou zavorkou, nebo je parametru vyrazu vic nez je povoleno
	return (DRS_ERROR);

}







// Funkce otevre novou derender-scenu ze souboru Filename. Vysledek se vraci v parametru Scene.
// Pokud vse uspeje, vraci se true, jinak false. Otevrena scena by se mela uzavirat volanim CloseDeRenderScene - viz nize
// Funkce nesmi byt v temze okamziku volana z ruznych threadu - z jineho ji lze volat az po jejim skonceni !!!
#define DRS_BUFFERELEMS		20 // Maximalni pocet parametru DRS vyrazu
#define DRS_ALLOCSTEP		32 // Po jakych blocich se prialokovavaji ruzna pole
bool OpenDeRenderScene(const char *Filename, DeRenderScene *pScene)
{
	DeRenderScene &Scene = *pScene;

	FILE *File;
	int ExpressionType, EqualPointsAllocated = 0, EqualTrianglesAllocated = 0;
	double Buffer[DRS_BUFFERELEMS];
	CameraCalibrationInfo MasterCalibrationInfo, SlaveCalibrationInfo, *CalibrationInfoPtr;
	pEqualPoint EqualPointPtr;
	pEqualTriangle EqualTrianglePtr;
	SynchronizationInfo SynchroInfo;
	bool Error = false, 
	     VersionRead = false, 
	     SynchroInfoRead = false, 
         MasterCalibrationInfoRead = false,
		 SlaveCalibrationInfoRead = false,
		 UseBitmapsRead = false;
	char MasterBitmapFile[80], SlaveBitmapFile[80];
	char MasterBitmapPath[MAXPATH+80], SlaveBitmapPath[MAXPATH+80];
	Point2D MasterBitmapResolution, SlaveBitmapResolution;
	unsigned int i, MaxPointIndex = 0; // Dosavadni maximalni index bodu nastavime na 0, pokud tedy DRS soubor neobsahuje zadny vyraz EqualPoint, zpusobi toto chybu funkce vrati false.
		
	// Uzavreme predchozi scenu
	CloseDeRenderScene(&Scene);

	// Otevrem soubor v textovem rezimu ('\r\n' ve Win32 -> '\n')
	File = fopen(Filename, "r");

	if (!File)
		return (false);

	// Zakladni inicializace
	Scene.EqualPoints = NULL;
	Scene.EqualTriangles = NULL;
	Scene.MasterBitmap = NULL;
	Scene.SlaveBitmap = NULL;
	Scene.NumOfEqualPoints = 0;
	Scene.NumOfEqualTriangles = 0;


	while (true)
	{
		// Nacteme dalsi DRS vyraz ze souboru
		ExpressionType = ReadDRSExpression(File, Buffer, DRS_BUFFERELEMS);
	
		switch(ExpressionType)
		{
			case DRS_ERROR: 
				Error = true;
				break;	

			case DRS_VERSION:
				// Kontrola spravnosti verze
				if (Buffer[0] != 1.00)
					Error = true;
				break;


			case DRS_SYNCHROINFO:
				// Vyraz SynchronizationInfo smi byt v DRS souboru pouze jednou
				if (SynchroInfoRead)
				{
					Error = true;
					break;
				}				
				// Zde lze zmenit poradi (vyznam) parametru DRS vyrazu
				SynchroInfo.MasterCameraMainControlPoint.X = Buffer[0];
				SynchroInfo.MasterCameraMainControlPoint.Y = Buffer[1];
				SynchroInfo.MasterCameraHelpControlPoint.X = Buffer[2];
				SynchroInfo.MasterCameraHelpControlPoint.Y = Buffer[3];
				SynchroInfo.SlaveCameraMainControlPoint.X = Buffer[4];
				SynchroInfo.SlaveCameraMainControlPoint.Y = Buffer[5];
				SynchroInfo.SlaveCameraHelpControlPoint.X = Buffer[6];
				SynchroInfo.SlaveCameraHelpControlPoint.Y = Buffer[7];
				SynchroInfo.SynchroPointToMCHCPDistance = Buffer[8];
				SynchroInfo.SynchroPointToSCHCPDistance = Buffer[9];
				SynchroInfo.MCHCPToSCHCPDistance = Buffer[10];
				SynchroInfoRead = true;
				break;

				
			case DRS_CALIBRATIONINFO:
				// Zjistime, kterou kameru (master/slave) tento DRS vyraz kalibruje, a nastavime pointer 
				// na prislusnou strukturu, aby se kod nemusel dvakrat opakovat.
				if (Buffer[0] == 1.00)
				{
					if (MasterCalibrationInfoRead)
						Error = true;
					CalibrationInfoPtr = &MasterCalibrationInfo;
					MasterCalibrationInfoRead = true;
				}
				else
					if (Buffer[0] == 2.00)
					{
						if (SlaveCalibrationInfoRead)
							Error = true;
						CalibrationInfoPtr = &SlaveCalibrationInfo;
						SlaveCalibrationInfoRead = true;
					}
					else
						Error = true;

				if (Error)
					break;
				// Tady lze opet zmenit vyznam a poradi parametru DRS vyrazu CalibrationInfo
				CalibrationInfoPtr->PixelA.X = Buffer[1];
				CalibrationInfoPtr->PixelA.Y = Buffer[2];
				CalibrationInfoPtr->PixelB.X = Buffer[3];
				CalibrationInfoPtr->PixelB.Y = Buffer[4];
				CalibrationInfoPtr->ViewPointToADistance = Buffer[5];
				CalibrationInfoPtr->ViewPointToBDistance = Buffer[6];
				CalibrationInfoPtr->AToBDistance = Buffer[7];
				CalibrationInfoPtr->Resolution.X = Buffer[8];
				CalibrationInfoPtr->Resolution.Y = Buffer[9];
				
				// A kontrola spravnosti rozliseni
				if( (CalibrationInfoPtr->Resolution.X <= 0)
					|| (CalibrationInfoPtr->Resolution.Y <= 0) )
				{
					CalibrationInfoPtr->Resolution.X = CalibrationInfoPtr->Resolution.Y = 0;
					Error = true;
				}
				break;

			case DRS_USEBITMAPS:
				// Tento vyraz tu smi byt pouze jednou
				if(UseBitmapsRead)
				{
					Error = true;
					break;
				}
				strncpy(MasterBitmapFile, (char *)Buffer, 80);
				strncpy(SlaveBitmapFile, (char *)(Buffer + 10), 80);

				// Pro jistotu ukoncime retezce nulou, takove by ale jiz mely byt
				MasterBitmapFile[79] = 0;
				SlaveBitmapFile[79] = 0;

				UseBitmapsRead = true;
				break;

			case DRS_EQUALPOINT:
				// Pokud je pole pro ulozeni ekvivalentnich bodu male, zvetsime ho o DRS_ALLOCSTEP prvku				
				if (Scene.NumOfEqualPoints >= EqualPointsAllocated)
				{
					EqualPointPtr = (pEqualPoint)realloc(Scene.EqualPoints, (EqualPointsAllocated + DRS_ALLOCSTEP) * sizeof(EqualPoint));
					if (!EqualPointPtr)
					{
						Error = true;
						break;
					}
					Scene.EqualPoints = EqualPointPtr;
					EqualPointsAllocated += DRS_ALLOCSTEP;
				}
				// Ulozime ziskana cisla
				Scene.EqualPoints[Scene.NumOfEqualPoints].MasterCameraPixel.X = Buffer[0];
				Scene.EqualPoints[Scene.NumOfEqualPoints].MasterCameraPixel.Y = Buffer[1];
				Scene.EqualPoints[Scene.NumOfEqualPoints].SlaveCameraPixel.X = Buffer[2];
				Scene.EqualPoints[Scene.NumOfEqualPoints].SlaveCameraPixel.Y = Buffer[3]; 
				Scene.EqualPoints[Scene.NumOfEqualPoints].BillboardSize = Buffer[4];
				Scene.EqualPoints[Scene.NumOfEqualPoints].Color = Buffer[5];

				Scene.NumOfEqualPoints++; 
				break;


			case DRS_EQUALTRIANGLE:
				if (Scene.NumOfEqualTriangles >= EqualTrianglesAllocated)
				{
					// Pokud je pole pro ulozeni ekvivalentnich trojuhelniku male, zvetsime ho o DRS_ALLOCSTEP prvku
					EqualTrianglePtr = (pEqualTriangle)realloc(Scene.EqualTriangles, (EqualTrianglesAllocated + DRS_ALLOCSTEP) * sizeof(EqualTriangle));
					if (!EqualTrianglePtr)
					{
						Error = true;
						break;
					}
					Scene.EqualTriangles = EqualTrianglePtr;
					EqualTrianglesAllocated += DRS_ALLOCSTEP;
				}
				// Ulozime ziskana cisla
				Scene.EqualTriangles[Scene.NumOfEqualTriangles].PointAIndex = Buffer[0];		
				Scene.EqualTriangles[Scene.NumOfEqualTriangles].PointBIndex = Buffer[1];
				Scene.EqualTriangles[Scene.NumOfEqualTriangles].PointCIndex = Buffer[2];
				Scene.EqualTriangles[Scene.NumOfEqualTriangles].TextureSource = Buffer[3];

				// Prubezne ukladame dosavadni maximalni index bodu
				if( MaxPointIndex < Scene.EqualTriangles[Scene.NumOfEqualTriangles].PointAIndex )
					MaxPointIndex = Scene.EqualTriangles[Scene.NumOfEqualTriangles].PointAIndex;
				if( MaxPointIndex < Scene.EqualTriangles[Scene.NumOfEqualTriangles].PointBIndex )
					MaxPointIndex = Scene.EqualTriangles[Scene.NumOfEqualTriangles].PointBIndex;
				if( MaxPointIndex < Scene.EqualTriangles[Scene.NumOfEqualTriangles].PointCIndex )
					MaxPointIndex = Scene.EqualTriangles[Scene.NumOfEqualTriangles].PointCIndex;

				// Kontrola spravnosti
				if( (Scene.EqualTriangles[Scene.NumOfEqualTriangles].TextureSource  < 0)
					|| (Scene.EqualTriangles[Scene.NumOfEqualTriangles].TextureSource  > 2)	)
				{
					Error = true;
					break;
				}
				Scene.NumOfEqualTriangles++;
				break;


				case DRS_EOF: // Jsme na konci souboru
					// Vse probehlo v poradku ?
					if ( !(MasterCalibrationInfoRead && SlaveCalibrationInfoRead && SynchroInfoRead)
						 || (MaxPointIndex >= Scene.NumOfEqualPoints) )
					{
						Error = true;
						break;
					}					

					Scene.MasterCalibInfo = MasterCalibrationInfo;
					Scene.SlaveCalibInfo = SlaveCalibrationInfo;
					Scene.SynchroInfo = SynchroInfo;				

					// Nacteme jeste bitmapy (fotky z master a slave kamery)
					if( UseBitmapsRead )
					{
						// Z cesty k DRS souboru vytvorime cestu k souborum s fotkami (BMP)
						strcpy(MasterBitmapPath, Filename);						
						CutPath(MasterBitmapPath);
						strcpy(SlaveBitmapPath, MasterBitmapPath);
						strcat(MasterBitmapPath, MasterBitmapFile);
						strcat(SlaveBitmapPath, SlaveBitmapFile);


						// Ulozime bitmapy do Scene, pokud se bitmapa nenacetla, ulozilo se do
						// Master- a/nebo SlaveBitmap NULL a do Master- a/nebo SlaveBitmapResolution [0,0]
						// Podle pripony zjistime format grafickeho souboru

						// Master bitmapa
						char *extension = strrchr(MasterBitmapPath, '.');
						if( !extension )
							{ Error = true;	break; }
						extension++;
						if( 0==stricmp(extension, "jpg") || 0==stricmp(extension, "jpeg") )
							ReadJPEG(MasterBitmapPath, &MasterBitmapResolution, &Scene.MasterBitmap);
						else
							if( 0==stricmp(extension, "bmp") )
								ReadBitmap(MasterBitmapPath, &MasterBitmapResolution, &Scene.MasterBitmap);
							else
								{ Error = true; break; }						

						// Slave bitmapa
						extension = strrchr(SlaveBitmapPath, '.');
						if( !extension )
							{ Error = true;	break; }
						extension++;
						if( 0==stricmp(extension, "jpg") || 0==stricmp(extension, "jpeg") )
							ReadJPEG(SlaveBitmapPath, &SlaveBitmapResolution, &Scene.SlaveBitmap);
						else
							if( 0==stricmp(extension, "bmp") )
								ReadBitmap(SlaveBitmapPath, &SlaveBitmapResolution, &Scene.SlaveBitmap);
							else
								{ Error = true; break; }							

						// Nyni zjistime, jestli odpovida rozliseni fotek nastaveni kamer,
						// nebo jestli nedoslo k nejake chybe pri nacitani bitmap
						if( (MasterBitmapResolution.X != Scene.MasterCalibInfo.Resolution.X)
							|| (MasterBitmapResolution.Y != Scene.MasterCalibInfo.Resolution.Y)
							|| (SlaveBitmapResolution.X != Scene.SlaveCalibInfo.Resolution.X)
							|| (SlaveBitmapResolution.Y != Scene.SlaveCalibInfo.Resolution.Y) )
						{
							Error = true;
							break;
						}
					}

					// Spravny prubeh
					fclose(File);

					// Pixely v EqualPoints maji nyni pocatek souradnic (pixel [0,0]) v levem HORNIM
					// rohu obrazku, mi chceme, aby byl v levem DOLNIM rohu, proto tento prepocet.
					for(i=0; i<Scene.NumOfEqualPoints; i++)
					{
						Scene.EqualPoints[i].MasterCameraPixel.Y = Scene.MasterCalibInfo.Resolution.Y - 1 - Scene.EqualPoints[i].MasterCameraPixel.Y;
						Scene.EqualPoints[i].SlaveCameraPixel.Y = Scene.SlaveCalibInfo.Resolution.Y - 1 - Scene.EqualPoints[i].SlaveCameraPixel.Y;	
					}

					return (true);
					break;
			
				default: 
					Error = true;
					break;
		}

		// Nastala ve switchi chyba ?
		if( Error )
			break;

	} // while

	// Sem se dostaneme jen pri chybe
	fclose(File);
	CloseDeRenderScene(&Scene);
	return (false);

}



// Funkce provede derendering sceny, tedy zjisti puvodni 3D polohy objektu. Vysledek ulozi do RenderScene,
// hodnota DRScene se nemeni. Pokud vse uspeje, vraci se true, jinak false. 
// Parametr pDRInfo urcuje jake DeRenderInfo pouzit, pokud je NULL, vytvori se nove podle 
// informaci z DRScene
bool SolveDeRenderScene(const DeRenderScene &DRScene, RenderScene *pRScene, const DeRenderInfo *pDRInfo)
{
	RenderScene &RScene = *pRScene;

	DeRenderInfo DRInfo;
	int Index;
	unsigned char TextureSource;
	Triangle2D TextureTrian;

	// Je vubec co derenderovat ?
	if ( !(DRScene.NumOfEqualPoints || DRScene.NumOfEqualTriangles) )
		return( false );

	
	if( pDRInfo )
		DRInfo = *pDRInfo;
	else
	{
		// Vytvorime si informace potrebne pro derendering (hlavne polohy kamer a jejich vlastnosti)
		if( !CreateDeRenderInfo(&DRInfo, DRScene.SynchroInfo, DRScene.MasterCalibInfo, DRScene.SlaveCalibInfo) )
			return( false );
	}

	// Zavreme puvodni render-scenu
	CloseRenderScene(&RScene);	

	// Alokujeme pamet pro objekty, pokud alokujeme pamet o nulove velikosti, pri spravnem prubehu
	// by mel malloc() vratit take nenulovy pointer
	RScene.Points = (pRSFPoint)malloc(sizeof(RSFPoint) * DRScene.NumOfEqualPoints);
	RScene.Triangles = (pRSFTriangle)malloc(sizeof(RSFTriangle) * DRScene.NumOfEqualTriangles);
	RScene.MasterTexture = (pColor32)malloc(sizeof(Color32) * DRScene.MasterCalibInfo.Resolution.X 
														   * DRScene.MasterCalibInfo.Resolution.Y);
	RScene.SlaveTexture = (pColor32)malloc(sizeof(Color32) * DRScene.SlaveCalibInfo.Resolution.X 
														   * DRScene.SlaveCalibInfo.Resolution.Y);

	if( !RScene.Points || !RScene.Triangles || !RScene.MasterTexture || !RScene.SlaveTexture )
	{	
		CloseRenderScene(&RScene);
		return( false );
	}

	// Pokud pointer ukazuje na pole o nulove velikosti, uvolnime ho a nastavime na NULL
	if( DRScene.NumOfEqualPoints==0 )
		{ free(RScene.Points); RScene.Points = NULL; }
	if( DRScene.NumOfEqualTriangles==0 )
		{ free(RScene.Triangles); RScene.Triangles = NULL; }

	if( DRScene.MasterBitmap == NULL )
		{ free(RScene.MasterTexture); RScene.MasterTexture = NULL; }
	if( DRScene.SlaveBitmap == NULL )
		{ free(RScene.SlaveTexture); RScene.SlaveTexture = NULL; }

	// Prenos zakladnich udaju
	RScene.NumOfPoints = DRScene.NumOfEqualPoints;
	RScene.NumOfTriangles = DRScene.NumOfEqualTriangles;
	RScene.MasterTextureRes = DRScene.MasterCalibInfo.Resolution;
	RScene.SlaveTextureRes = DRScene.SlaveCalibInfo.Resolution;
	RScene.MasterCameraPosition = DRInfo.MasterCamera.ViewPoint;
	RScene.SlaveCameraPosition = DRInfo.SlaveCamera.ViewPoint;
	
	Point2D MasterPixel, SlavePixel;

	// DeRendering vsech bodu a jejich ulozeni do RScene
	for(Index = 0; Index < DRScene.NumOfEqualPoints; Index++)
	{
		if (!DerenderPoint(&RScene.Points[Index].Position, 
			               DRInfo, 
						   DRScene.EqualPoints[Index].MasterCameraPixel, 
	 					   DRScene.EqualPoints[Index].SlaveCameraPixel))
		{
			CloseRenderScene(&RScene);
			return(false);
		}
		RScene.Points[Index].BillboardSize = DRScene.EqualPoints[Index].BillboardSize;
		RScene.Points[Index].Color = DRScene.EqualPoints[Index].Color;
	}

	// Ulozeni trojuhelniku do RScene a kontrola existence prislusnych textury
	for(Index = 0; Index < DRScene.NumOfEqualTriangles; Index++)
	{	
		RScene.Triangles[Index].PointAIndex = DRScene.EqualTriangles[Index].PointAIndex;
		RScene.Triangles[Index].PointBIndex = DRScene.EqualTriangles[Index].PointBIndex;
		RScene.Triangles[Index].PointCIndex = DRScene.EqualTriangles[Index].PointCIndex;		
		
		TextureSource = DRScene.EqualTriangles[Index].TextureSource;

		// Ulozime jeste polohu textury
		switch(TextureSource)
		{
			case 1: // Zdroj je fotka z master kamery
				TextureTrian.A = DRScene.EqualPoints[ DRScene.EqualTriangles[Index].PointAIndex ].MasterCameraPixel;
				TextureTrian.B = DRScene.EqualPoints[ DRScene.EqualTriangles[Index].PointBIndex ].MasterCameraPixel;
				TextureTrian.C = DRScene.EqualPoints[ DRScene.EqualTriangles[Index].PointCIndex ].MasterCameraPixel;
				RScene.Triangles[Index].TexturePosition = TextureTrian;

				if( DRScene.MasterBitmap==NULL )
					TextureSource = 0; // Master textura neni pritomna
				break;

			case 2: // Zdroj je fotka ze slave kamery
				TextureTrian.A = DRScene.EqualPoints[ DRScene.EqualTriangles[Index].PointAIndex ].SlaveCameraPixel;
				TextureTrian.B = DRScene.EqualPoints[ DRScene.EqualTriangles[Index].PointBIndex ].SlaveCameraPixel;
				TextureTrian.C = DRScene.EqualPoints[ DRScene.EqualTriangles[Index].PointCIndex ].SlaveCameraPixel;

				RScene.Triangles[Index].TexturePosition = TextureTrian;
				
				if( DRScene.SlaveBitmap==NULL )
					TextureSource = 0; // Slave textura neni pritomna
				break;

			default: // Bez textury
				TextureSource = 0;
				break;
		}
		
		RScene.Triangles[Index].TextureSource = TextureSource;
	}


	// A prekopirovani puvodnich fotek jakozto textur pro rendering
	if( DRScene.MasterBitmap )
		memcpy(RScene.MasterTexture, DRScene.MasterBitmap, sizeof(Color32) * DRScene.MasterCalibInfo.Resolution.X 
																   * DRScene.MasterCalibInfo.Resolution.Y);
	if( DRScene.SlaveBitmap )
		memcpy(RScene.SlaveTexture, DRScene.SlaveBitmap, sizeof(Color32) * DRScene.SlaveCalibInfo.Resolution.X 
																   * DRScene.SlaveCalibInfo.Resolution.Y);


//	double xxx = prum_odch / pocet;
	return( true );


}


// Procedura uzavre Scene - dealokuje pamet
void CloseDeRenderScene(DeRenderScene *pScene)
{
	if (pScene->EqualPoints)
		free(pScene->EqualPoints);
	if (pScene->EqualTriangles)
		free(pScene->EqualTriangles);	
	if (pScene->MasterBitmap)
		free(pScene->MasterBitmap);	
	if (pScene->SlaveBitmap)
		free(pScene->SlaveBitmap);	

	pScene->EqualPoints = NULL;
	pScene->EqualTriangles = NULL;
	pScene->MasterBitmap = NULL;
	pScene->SlaveBitmap = NULL;	
}



