/* 3DProcs.cpp : Soubor s funkcemi pro graficke vypocty, 3D vykreslovani, derendering ... 
   Copyright(c) 2002,2003 Jan Curn (Jan.Curn@seznam.cz) */




#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

#include "MyProcs.h"

// Pro podporu JPEG, jeste musi byt prilinkovana JPEG.LIB
extern "C" {
	#include "..\_jpeg\jpeglib.h"
}


#define HLS_MAX	(240) /* H,L, and S vary over 0-HLSMAX */
                        /* COLORHLS_MAX BEST IF DIVISIBLE BY 6 */
                        /* COLORHLS_MAX must each fit in a byte. */
#define RGB_MAX	(255)

/* Hue is undefined if Saturation is 0 (grey-scale) */
/* This value determines where the Hue scrollbar is */
/* initially set for achromatic colors */
#define H_UNDEFINED (HLS_MAX*2/3)


ColorHLS ConvertRGBtoHLS( Color32 RGB_Color )
{
	short R,G,B;          /* input RGB values */
	short cMax,cMin;      /* max and min RGB values */
	short Rdelta,Gdelta,Bdelta; /* intermediate value: % of spread from max */

	long H, L, S;
	
	R = COLOR32_GET_R(RGB_Color);
	G = COLOR32_GET_G(RGB_Color);
	B = COLOR32_GET_B(RGB_Color);

	// Vypocet svetlosti
	cMax = MAX( MAX(R,G), B);
	cMin = MIN( MIN(R,G), B);

	L = ( ((cMax+cMin)*HLS_MAX) + RGB_MAX ) / (2*RGB_MAX);

	// Pripad R=G=B (odstin sedi) ?
	if (cMax == cMin)
		return( ((ColorHLS)H_UNDEFINED << 16) | ((unsigned long)L << 8));
      
	// Pripad barvy, ktera neni odstinem sedi
    // Vypocet sytosti
	if (L <= (HLS_MAX/2))
		S = ( ((cMax-cMin)*HLS_MAX) + ((cMax+cMin)/2) ) / (cMax+cMin);
	else
		S = ( ((cMax-cMin)*HLS_MAX) + ((2*RGB_MAX-cMax-cMin)/2) )
            / (2*RGB_MAX-cMax-cMin);

	// Vypocet odstinu
	Rdelta = ( ((cMax-R)*(HLS_MAX/6)) + ((cMax-cMin)/2) ) / (cMax-cMin);
	Gdelta = ( ((cMax-G)*(HLS_MAX/6)) + ((cMax-cMin)/2) ) / (cMax-cMin);
	Bdelta = ( ((cMax-B)*(HLS_MAX/6)) + ((cMax-cMin)/2) ) / (cMax-cMin);
	
	if (R == cMax)
		H = Bdelta - Gdelta;
	else if (G == cMax)
		H = (HLS_MAX/3) + Rdelta - Bdelta;
	else /* B == cMax */
		H = ((2*HLS_MAX)/3) + Gdelta - Rdelta;

	if (H < 0)
		H += HLS_MAX;
	if (H > HLS_MAX)
		H -= HLS_MAX;   

   return( ((unsigned long)H << 16) | ((unsigned long)L << 8) | (unsigned long)S );
}

// Pomocna funkce pro ConvertHLStoRGB()
inline long HueToRGB(long n1, long n2, short Hue)
{    
   // Kontrola mezi - pri volani ve funkci ConvertHLStoRGB() se pricita/odecita tretina HLS_MAX, proto je toto potreba
   if (Hue < 0)
      Hue += HLS_MAX;
   if (Hue > HLS_MAX)
      Hue -= HLS_MAX;

   /* return r,g, or b value from this tridrant */
   if( Hue < (HLS_MAX/6) )
      return( n1 + (((n2-n1)*Hue+(HLS_MAX/12))/(HLS_MAX/6)) );

   if( Hue < (HLS_MAX/2) )
      return( n2 );

   if( Hue < ((HLS_MAX*2)/3) )
      return( n1 + (((n2-n1)*(((HLS_MAX*2)/3)-Hue)+(HLS_MAX/12))/(HLS_MAX/6))); 
   else
      return( n1 );
}

// Funkce prevede barvu z formatu HLS do RGB
Color32 ConvertHLStoRGB( ColorHLS HLS_Color )
{
	long R,G,B; // RGB slozky bravy
	long H,L,S; // HLS slozky barvy
	long Magic1, Magic2; // Magicka cisla

	H = COLORHLS_GET_H( HLS_Color );
	L = COLORHLS_GET_L( HLS_Color );
	S = COLORHLS_GET_S( HLS_Color );

	// Jedna se o odstin sedi ?
	if( S == 0 )
		R=G=B=(L*RGB_MAX)/HLS_MAX;	
	else
	{
		// Barva neni odstinem sedi
		// Vypocet magickych cisel
		if (L <= (HLS_MAX/2))
			Magic2 = (L*(HLS_MAX + S) + (HLS_MAX/2)) / HLS_MAX;
		else
			Magic2 = L + S - ((L*S) + (HLS_MAX/2)) / HLS_MAX;
		Magic1 = 2*L - Magic2;

		// Ziskame RGB slozky barvy, zmenime jednotky z HLS_MAX na RGB_MAX 
		R = ( HueToRGB(Magic1,Magic2,H+(HLS_MAX/3)) *RGB_MAX + (HLS_MAX/2) ) / HLS_MAX; 
		G = ( HueToRGB(Magic1,Magic2,H            ) *RGB_MAX + (HLS_MAX/2) ) / HLS_MAX;
		B = ( HueToRGB(Magic1,Magic2,H-(HLS_MAX/3)) *RGB_MAX + (HLS_MAX/2) ) / HLS_MAX; 
   }

   return( COLOR32_CREATE(R,G,B) );
}




// Pomocny objekt pro funkci GetPenetrationPoint
typedef struct {
	double *pA1, *pB1, *pP1, *pU1;
	double *pA2, *pB2, *pP2, *pU2;
} GPP_Permutation ;

// Funkce zisti prunik primky Line s primkou urcenou body PointA a PointB, vysledek - koeficient pro
// parametrickou rovnici primky Line - vraci pomoci pResultCoeff (Prunik pak tedy lezi 
// v bode AddVector(Line.Point, Line.Vector, *pResultCoeff).
// Funkce neoveruje, jestli se primky skutecne protnou.
bool GetPenetrationPoint(
	double *pResultCoeff,
	const StraightLine &Line,
	const Point3D &PointA,
	const Point3D &PointB)
{
	GPP_Permutation Perm[3];

	// Permutace 0 - 1=X, 2=Y
	Perm[0].pA1 = (double *)&PointA.X;
	Perm[0].pB1 = (double *)&PointB.X;
	Perm[0].pP1 = (double *)&Line.Point.X;
	Perm[0].pU1 = (double *)&Line.Vector.X;
	Perm[0].pA2 = (double *)&PointA.Y;
	Perm[0].pB2 = (double *)&PointB.Y;
	Perm[0].pP2 = (double *)&Line.Point.Y;
	Perm[0].pU2 = (double *)&Line.Vector.Y;

	// Permutace 0 - 1=X, 2=Z
	Perm[1].pA1 = (double *)&PointA.X;
	Perm[1].pB1 = (double *)&PointB.X;
	Perm[1].pP1 = (double *)&Line.Point.X;
	Perm[1].pU1 = (double *)&Line.Vector.X;
	Perm[1].pA2 = (double *)&PointA.Z;
	Perm[1].pB2 = (double *)&PointB.Z;
	Perm[1].pP2 = (double *)&Line.Point.Z;
	Perm[1].pU2 = (double *)&Line.Vector.Z;

	// Permutace 0 - 1=Y, 2=Z
	Perm[2].pA1 = (double *)&PointA.Y;
	Perm[2].pB1 = (double *)&PointB.Y;
	Perm[2].pP1 = (double *)&Line.Point.Y;
	Perm[2].pU1 = (double *)&Line.Vector.Y;
	Perm[2].pA2 = (double *)&PointA.Z;
	Perm[2].pB2 = (double *)&PointB.Z;
	Perm[2].pP2 = (double *)&Line.Point.Z;
	Perm[2].pU2 = (double *)&Line.Vector.Z;

	double Val1, Val2, Denominator;

	for(int i=0; i<3; i++)
	{
		Val1 = *Perm[i].pA1 - *Perm[i].pB1;
		Val2 = *Perm[i].pA2 - *Perm[i].pB2;
		Denominator = *Perm[i].pU2*Val1 - *Perm[i].pU1*Val2;

		if( Denominator == 0 )
			continue;

		*pResultCoeff = (*Perm[i].pP1)*Val2 - (*Perm[i].pP2)*Val1 + (*Perm[i].pA1)*(*Perm[i].pB2) - (*Perm[i].pA2)*(*Perm[i].pB1);
		*pResultCoeff /= Denominator;

		return( true );
	}

	return( false );
	
}







// Funkce pocita relativni polohu (stav) Coeff bodu Point ke trojuhelniku Trian, zname-li tedy stav 
// i trojuhelnik, jsme schopni urcit souradnice bodu (i pokud se poloha trojuhelniku zmeni), toto dela
// funkce SetPointRelationToTriangle. Parametry maji vyznam:
//		Trian.....Souradnice ridiciho trojuhelniku, hodnota se nemeni
//		Point.....Souradnice bodu, hodnota se nemeni
//		Coeffs....Vysledek, obsahuje relativni vztah bodu k trojuhelniku, hodnota se tedy meni
// Pokud vse uspeje, vraci se true, pokud jsou body trojuhelniku v jedne primce ci dojde
// k jine chybe, vraci se false. Pomocne objekty pro vnitrni uziti funkce jsou:
typedef struct { double *B, *C, *W, *P; } GPRTTElem, *pGPRTTElem;
typedef struct { pGPRTTElem Elem1, Elem2, Elem3; } GPRTTPermutation;
bool GetPointRelationToTriangle(const Triangle3D &Trian, const Point3D &Point, PointRTT *pCoeffs)
{
	Vector3D B, C, W, P;
	GPRTTElem ElemX, ElemY, ElemZ;
	GPRTTPermutation Perms[6];
	double dHelp1, dHelp2, dHelp3, dHelp4, dHelp5;
	bool bResFound = false;

	// Pripravime vektory, pozdeji budeme hledat pCoeffs tak, aby resily linearni
	// rovnici P = pCoeffs->Z * B + pCoeffs->Y * C + pCoeffs->X * W, vektory B a C musi
	// nutne byt linearne nezavisle
	B = GetVector(Trian.A, Trian.C);
	C = GetVector(Trian.A, Trian.B);
	W = VectorMultiply(B, C);
	P = GetVector(Trian.A, Point);
  
	// Pripravime slozky, ktere se budou permutovat a nakonec samotne permutace
	ElemX.B = &B.X;  ElemY.B = &B.Y;  ElemZ.B = &B.Z;
	ElemX.C = &C.X;  ElemY.C = &C.Y;  ElemZ.C = &C.Z;
	ElemX.W = &W.X;  ElemY.W = &W.Y;  ElemZ.W = &W.Z;
	ElemX.P = &P.X;  ElemY.P = &P.Y;  ElemZ.P = &P.Z;
  
	Perms[0].Elem1 = &ElemX; Perms[0].Elem2 = &ElemY; Perms[0].Elem3 = &ElemZ; 
	Perms[1].Elem1 = &ElemX; Perms[1].Elem2 = &ElemZ; Perms[1].Elem3 = &ElemY; 
	Perms[2].Elem1 = &ElemY; Perms[2].Elem2 = &ElemX; Perms[2].Elem3 = &ElemZ; 
	Perms[3].Elem1 = &ElemY; Perms[3].Elem2 = &ElemZ; Perms[3].Elem3 = &ElemX; 
	Perms[4].Elem1 = &ElemZ; Perms[4].Elem2 = &ElemX; Perms[4].Elem3 = &ElemY;   
	Perms[5].Elem1 = &ElemZ; Perms[5].Elem2 = &ElemY; Perms[5].Elem3 = &ElemX;   
  
	// Rovnice pro reseni soustavy 3 linearnich rovnic nezavisi na permutaci stejnych slozek
	// vektoru B, C, W a bodu P, zkousime tedy vsechny tyto permutace dokud nenalezneme 1 reseni
	for(int i=0; i<6; i++)
	{
		dHelp1 = (*Perms[i].Elem2->C)*(*Perms[i].Elem1->B) - (*Perms[i].Elem1->C)*(*Perms[i].Elem2->B);
		dHelp2 = (*Perms[i].Elem3->C)*(*Perms[i].Elem1->B) - (*Perms[i].Elem1->C)*(*Perms[i].Elem3->B);
		dHelp3 = (*Perms[i].Elem2->W)*(*Perms[i].Elem1->B) - (*Perms[i].Elem1->W)*(*Perms[i].Elem2->B);
		
		dHelp4 = ( (*Perms[i].Elem3->W)*(*Perms[i].Elem1->B) - (*Perms[i].Elem1->W)*(*Perms[i].Elem3->B) ) 
				    * dHelp1 - dHelp3 * dHelp2;

		// Pokud by se melo delit nulou, zkusime dalsi permutaci
		if( (dHelp4==0) || (dHelp1==0) || (*Perms[i].Elem1->B==0) )
			continue;
  
		// A vypocitame koeficienty
		dHelp5 = (*Perms[i].Elem2->P)*(*Perms[i].Elem1->B) - (*Perms[i].Elem1->P)*(*Perms[i].Elem2->B);
		pCoeffs->X = ( ( (*Perms[i].Elem3->P)*(*Perms[i].Elem1->B) - (*Perms[i].Elem1->P)*(*Perms[i].Elem3->B) )*dHelp1 - dHelp5*dHelp2 )
					/ dHelp4;
		pCoeffs->Y = (dHelp5 - dHelp3*pCoeffs->X) 
					/ dHelp1;
		pCoeffs->Z = ( (*Perms[i].Elem1->P) - (*Perms[i].Elem1->W)*pCoeffs->X - (*Perms[i].Elem1->C)*pCoeffs->Y )
					/ (*Perms[i].Elem1->B);

		bResFound = true;
		break;
	}

	return( bResFound );
  
}

// Funkce vypocita polohu bodu pPoint podle jeho relativnich koeficientu Coeffs vzhledem
// ke trojuhelniku Trian. Hodnota se meni pouze u parametru pPoint. Tato funkce je
// "inverzni" k funkci GetPointRelationToTriangle.
void SetPointRelationToTriangle(const Triangle3D &Trian, Point3D *pPoint, const PointRTT &Coeffs)
{
	Vector3D B, C, W;
  
	// Pripravime vektory, pote bod nalezneme podle linearni rovnice 
	// Point = Trian.A + Coeffs.Z * B + Coeffs.Y * C + Coeffs.X * W
	B = GetVector(Trian.A, Trian.C);
	C = GetVector(Trian.A, Trian.B);
	W = VectorMultiply(B, C);
 
	pPoint->X = Trian.A.X + Coeffs.Z*B.X + Coeffs.Y*C.X + Coeffs.X*W.X;
	pPoint->Y = Trian.A.Y + Coeffs.Z*B.Y + Coeffs.Y*C.Y + Coeffs.X*W.Y;
	pPoint->Z = Trian.A.Z + Coeffs.Z*B.Z + Coeffs.Y*C.Z + Coeffs.X*W.Z;

	return;  
}








// Funkce najde na primkach Line1 a Line2 body Point1 a Point2, ktere jsou k sobe nejblize 
// ze vsech bodu primek. Vysledek se vraci v parametrech Point1 a Point2 predavanych odkazem,
// ostatni parametry funkce nemeni. Pokud funkce uspeje, vraci true, pokud jsou primky
// rovnobezne, nebo dojde k jine chybe, vraci se false. Maji-li primky prunik, 
// pak Point1 (=Point2) je bod pruniku. Bod Point1 lezi na Line1, Point2 na Line2.
bool GetNearestPoints(const StraightLine &Line1, const StraightLine &Line2, Point3D &Point1, Point3D &Point2)
{
	double D, E, F, G, H, I, t, r;
  
	// Hledame minimum funkce vzdalenosti bodu primek na R^2 (parametry r a t), tedy jeji 
	// totalni diferencial (resp. parcialni derivace) jsou rovny nule, dostaneme soustavu dvou
	// linearnich rovnic o dvou neznamych a mame vysledek.
	D = ScalarMultiply(Line1.Vector, Line1.Vector);
	E = ScalarMultiply(Line2.Vector, Line2.Vector);
	F = 2 * (ScalarMultiply(Line1.Point, Line1.Vector) - ScalarMultiply(Line2.Point, Line1.Vector));
	G = 2 * (ScalarMultiply(Line2.Point, Line2.Vector) - ScalarMultiply(Line1.Point, Line2.Vector));
	H = (-2) * ScalarMultiply(Line1.Vector, Line2.Vector);
	I = 4*D*E - H*H;
  
	// Pokud I=0, pak z Cauchy-Schwartzovy nerovnosti Line1.Vector a Line2.Vector jsou linearne
	// zavisle, tedy Line1 a Line2 rovnobezne.
	if((I == 0.0) || (E == 0.0))
		return( false ); 

	t = (G*H - 2*E*F) / I;
	r = -(G + H*t) / (2*E);

	Point1 = AddVector(Line1.Point, Line1.Vector, t);
	Point2 = AddVector(Line2.Point, Line2.Vector, r);

	return( true );
}



// Funkce nacte bitmapu ze souboru Filename (typicky *.BMP), do Bitmap alokuje pamet pro pixely pomoci malloc() 
// a ulozi do ni prislusny obrazek. Rozliseni obrazku vraci funkce funkce v parametru Resolution.
// Zatim jsou podporovany pouze bitmapy bez komprese a bez palety s 24 bity na pixel. Pokud vse uspeje,
// funkce vraci true. Pri chybe vraci false, do Bitmap ulozi NULL, a rozliseni nastavi na [0,0]. 
// Jakmile jiz Bitmap neni potreba, musi byt pridelena pamet uvolnena pomoci free().
bool ReadBitmap(const char *Filename, Point2D *pResolution, pColor32 *ppBitmap)
{
	pResolution->X = pResolution->Y = 0;
	*ppBitmap = NULL;

	Point2D &Resolution = *pResolution;
	pColor32 &Bitmap = *ppBitmap;


	MYBITMAPFILEHEADER FileHeader;
	MYBITMAPINFOHEADER InfoHeader;
	int Index, MaxIndex;
	unsigned int Buffer = 0;
	FILE *File;

	unsigned char *ReadBuffer, *ReadPtr;
	unsigned int LineSize;


	Resolution.X = Resolution.Y = NULL;
	Bitmap = NULL;

	File = fopen(Filename, "rb");

	if( !File )
		return( false );

	// Nacteme hlavicky
	if(fread(&FileHeader, SIZEOF_MYBITMAPFILEHEADER, 1, File) != 1)
	{
		fclose(File);
		return( false );
	}
	if( fread(&InfoHeader, SIZEOF_MYBITMAPINFOHEADER, 1, File) != 1) 
	{
		fclose(File);
		return( false );
	}


	// Kontrola udaju
	if( (FileHeader.bfType != 19778) 
		|| (InfoHeader.biSize != 40)
		|| (InfoHeader.biPlanes != 1)
		|| (InfoHeader.biBitCount != 24) // Zatim 
		|| (InfoHeader.biCompression != 0)
		|| (InfoHeader.biWidth == 0)
		|| (InfoHeader.biHeight == 0)
		|| (InfoHeader.biClrUsed != 0) )
	{
		fclose(File);
		return( false );
	}

	Resolution.X = (signed short)InfoHeader.biWidth;
	Resolution.Y = (signed short)InfoHeader.biHeight;
		
	MaxIndex = Resolution.X * Resolution.Y - 1;

	// Alokace pameti pro pixely
	Bitmap = (pColor32)malloc(sizeof(Color32) * Resolution.X * Resolution.Y);
	if( !Bitmap )
	{
		fclose(File);
		Resolution.X = Resolution.Y = NULL;	
		return( false );
	}

	// Radky jsou v BMP souboru zarovnany na ctyri byty
	if( Resolution.X*3 % 4 )
		LineSize = Resolution.X*3 + 4 - (Resolution.X*3 % 4);
	else
		LineSize = Resolution.X*3;

	// Alokacu cteciho bufferu
	ReadBuffer = (unsigned char *)malloc( LineSize );
	if( !ReadBuffer )
	{
		free(Bitmap);
		Bitmap = NULL;
		fclose(File);
		Resolution.X = Resolution.Y = NULL;
		return( false );
	}

	Color32  * WritePtr     = Bitmap;
	Color32  * MaxWritePtr  = Bitmap + Resolution.X - 1;
		

	// Pro kazdou radku...
	for( Index=0; Index<Resolution.Y; Index++ )
	{
		// Nacteme jednu radku
		if( fread(ReadBuffer, LineSize, 1, File) != 1 )
		{
			// Nastala chyba
			free(Bitmap);
			free(ReadBuffer);
			Bitmap = NULL;
			fclose(File);
			Resolution.X = Resolution.Y = NULL;
			return( false );
		}

		
		// a radku ulozime do Bitmap
		ReadPtr = ReadBuffer;
		for(; WritePtr <= MaxWritePtr; WritePtr++)
		{
			*WritePtr = COLOR32_CREATE( ReadPtr[2], ReadPtr[1], ReadPtr[0] );
			ReadPtr += 3;
		}

		MaxWritePtr += Resolution.X;
	}

	fclose(File);
	free(ReadBuffer);

	return( true );

}


// funkce nacte BMP neno JPEG ze souboru, alokuje se funkci malloc()
bool ReadPictureFromFile(const char *Filename, MyPicture *pPicture)
{	
	bool retval;
	char *extension = strrchr(Filename, '.');
	if( !extension )
		return( false );
	extension++;

	pPicture->Release();

	if( 0==stricmp(extension, "jpg") || 0==stricmp(extension, "jpeg") )
		retval = ReadJPEG(Filename, &pPicture->Resolution, &pPicture->AllocatedSpace);
	else
		if( 0==stricmp(extension, "bmp") )
			retval = ReadBitmap(Filename, &pPicture->Resolution, &pPicture->AllocatedSpace);
		else
			return( false );
	if( !retval )
	{
		pPicture->Release();
		return( false );
	}

	pPicture->Rows = (pColor32 *)malloc( sizeof(pColor32) * pPicture->Resolution.Y );
	if( !pPicture->Rows )
	{
		pPicture->Release();
		return( false );
	}

	// A ukazatele na radky v poli Rows nasmerujeme do AllocatedSpace, aby to odpovidalo
	Color32 *ColorPtr = pPicture->AllocatedSpace;
	for( int i=0; i<pPicture->Resolution.Y; i++ )
	{
		pPicture->Rows[i] = ColorPtr;
		ColorPtr += pPicture->Resolution.X;
	}
	return( true );
}



extern "C" {
	#include "..\_jpeg\jpeglib.h"
}

/*
 * <setjmp.h> is used for the optional error recovery mechanism shown in
 * the second part of the example.
 */

#include <setjmp.h>


struct my_error_mgr {
  struct jpeg_error_mgr pub;	/* "public" fields */
  jmp_buf setjmp_buffer;	/* for return to caller */
};

typedef struct my_error_mgr * my_error_ptr;

/*
 * Here's the routine that will replace the standard error_exit method:
 */
METHODDEF(void)
my_error_exit (j_common_ptr cinfo)
{
  /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
  my_error_ptr myerr = (my_error_ptr) cinfo->err;

  /* Always display the message. */
  /* We could postpone this until after returning, if we chose. */
  (*cinfo->err->output_message) (cinfo);

  /* Return control to the setjmp point */
  longjmp(myerr->setjmp_buffer, 1);
}



// Funkce nacte JPG obrazek ze souboru Filename, alokuje ppData funkci malloc() a
// ulozi tam obrazek a do pResolution rozliseni. Volajci pote musi pridelenou 
// pamet uvolnit pomoci free()
bool ReadJPEG(const char *Filename, Point2D *pResolution, Color32 **ppData)
{
	pResolution->X = pResolution->Y = 0;
	*ppData = NULL;

	/* This struct contains the JPEG decompression parameters and pointers to
	* working space (which is allocated as needed by the JPEG library).
	*/
	struct jpeg_decompress_struct cinfo;
	/* We use our private extension JPEG error handler.
	* Note that this struct must live as long as the main JPEG parameter
	* struct, to avoid dangling-pointer problems.
	*/
	struct my_error_mgr jerr;
	/* More stuff */
	FILE * infile;		/* source file */
	JSAMPARRAY buffer;		/* Output row buffer */
	int row_stride;		/* physical row width in output buffer */

	/* In this example we want to open the input file before doing anything else,
	* so that the setjmp() error recovery below can assume the file is open.
	* VERY IMPORTANT: use "b" option to fopen() if you are on a machine that
	* requires it in order to read binary files.
	*/

	if ((infile = fopen(Filename, "rb")) == NULL)
	  return( false );	  

	/* Step 1: allocate and initialize JPEG decompression object */

	/* We set up the normal JPEG error routines, then override error_exit. */
	cinfo.err = jpeg_std_error(&jerr.pub);
	jerr.pub.error_exit = my_error_exit;
	/* Establish the setjmp return context for my_error_exit to use. */
	if (setjmp(jerr.setjmp_buffer))
	{
		/* If we get here, the JPEG code has signaled an error.
		* We need to clean up the JPEG object, close the input file, and return.
		*/
		jpeg_destroy_decompress(&cinfo);
		fclose(infile);
		return( false );
	}
  /* Now we can initialize the JPEG decompression object. */
  jpeg_create_decompress(&cinfo);

  /* Step 2: specify data source (eg, a file) */

  jpeg_stdio_src(&cinfo, infile);

  /* Step 3: read file parameters with jpeg_read_header() */

  (void) jpeg_read_header(&cinfo, TRUE);
  /* We can ignore the return value from jpeg_read_header since
   *   (a) suspension is not possible with the stdio data source, and
   *   (b) we passed TRUE to reject a tables-only JPEG file as an error.
   * See libjpeg.doc for more info.
   */

  /* Step 4: set parameters for decompression */

  /* In this example, we don't need to change any of the defaults set by
   * jpeg_read_header(), so we do nothing here.
   */

  /* Step 5: Start decompressor */

  (void) jpeg_start_decompress(&cinfo);
  /* We can ignore the return value since suspension is not possible
   * with the stdio data source.
   */

  /* We may need to do some setup of our own at this point before reading
   * the data.  After jpeg_start_decompress() we have the correct scaled
   * output image dimensions available, as well as the output colormap
   * if we asked for color quantization.
   * In this example, we need to make an output work buffer of the right size.
   */ 
  /* JSAMPLEs per row in output buffer */
  row_stride = cinfo.output_width * cinfo.output_components;


  pResolution->X = cinfo.output_width;
  pResolution->Y = cinfo.output_height;

	*ppData = (Color32 *)malloc( pResolution->X * pResolution->Y * sizeof(Color32) );
	if( !(*ppData) || cinfo.output_components!=3 /* podpora zatim jen RGB_24 */) 
	{
		(void) jpeg_finish_decompress(&cinfo);
		jpeg_destroy_decompress(&cinfo);
		fclose(infile);
		return( false );  
	}



  /* Make a one-row-high sample array that will go away when done with image */
  buffer = (*cinfo.mem->alloc_sarray)
		((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

  /* Step 6: while (scan lines remain to be read) */
  /*           jpeg_read_scanlines(...); */

  /* Here we use the library's state variable cinfo.output_scanline as the
   * loop counter, so that we don't have to keep track ourselves.
   */
	Color32 *WritePtr = *ppData + (pResolution->Y - 1)*pResolution->X;
	JSAMPROW read_ptr = buffer[0],
	         max_read_ptr = buffer[0] + row_stride - 3;

  while (cinfo.output_scanline < cinfo.output_height) {
    (void) jpeg_read_scanlines(&cinfo, buffer, 1);
    
	// Preulozeni radky do naseho formatu
	for( ; read_ptr<=max_read_ptr ; read_ptr+=3 )
	{
		*WritePtr = COLOR32_CREATE( GETJSAMPLE(*read_ptr), GETJSAMPLE(*(read_ptr+1)), GETJSAMPLE(*(read_ptr+2)) );
		WritePtr++;
	}

	read_ptr = buffer[0];
	WritePtr -= 2*pResolution->X;   
  }


    /* Step 7: Finish decompression */

  (void) jpeg_finish_decompress(&cinfo);
  /* We can ignore the return value since suspension is not possible
   * with the stdio data source.
   */

  /* Step 8: Release JPEG decompression object */

  /* This is an important step since it will release a good deal of memory. */
  jpeg_destroy_decompress(&cinfo);

  /* After finish_decompress, we can close the input file.
   * Here we postpone it until after no more JPEG errors are possible,
   * so as to simplify the setjmp error logic above.  (Actually, I don't
   * think that jpeg_destroy can do an error exit, but why assume anything...)
   */
  fclose(infile);

  /* At this point you may want to check to see whether any corrupt-data
   * warnings occurred (test whether jerr.pub.num_warnings is nonzero).
   */

  /* And we're done! */
  return( true );







}





















// Procedura "odrizne" jmeno souboru z FilePath 
void CutPath(char *FilePath)
{
	int LastBackslash = -1;
	int Index;

	for(Index=0; FilePath[Index] != NULL; Index++)
		if(FilePath[Index] == '\\')
			LastBackslash = Index;

	FilePath[LastBackslash+1] = NULL;
}


/*
void MemorySet(unsigned long *Pointer, unsigned long Elem, unsigned long ElemsCount, unsigned long *TempBuffer, unsigned long BufferCount)
{
	signed long Index;
	// Pripravime buffer
	for(Index=0; Index<BufferCount; Index++)
		TempBuffer[Index] = Elem;	
	// Buffer postupne kopirujeme do pameti
	for(Index=0; Index<=ElemsCount-BufferCount; Index+=BufferCount)
		memcpy(Pointer+Index, TempBuffer, BufferCount*sizeof(unsigned long));
	// a zbyle elementy dojedeme pojednom
	for(;Index<ElemsCount;Index++)
		Pointer[Index] = Elem;	
}
*/



// Procedury pro vypocet car

// Funkce zkrati usecku pLine, aby byla cela uvnitr "vyrezu" s rozlisenim Resolution, pokud to lze,
// upravi pLine a vraci true, pokud je usecka mimo obraz, vraci se false (pak neni stav pLine definovan)
// Smer usecky (tj. poradi pLine->A a pLine->B) zustava nezmenen.
bool ShortenLineUsingResolution(Line2D *pLine, Point2D Resolution)
{
	Point2D HelpPoint;
	Point2D &A = pLine->A,
	        &B = pLine->B;
	int DeltaX, DeltaY, NewDeltaX, NewDeltaY;
	int SwapCount = 0;
	Line2D NoLine;

	NoLine.A.X = NoLine.A.Y = NoLine.B.X = NoLine.B.Y = -10000;

	// Specialni pripad
	if( A.X == B.X )
	{
		// Je usecka mimo obraz ?
		if( (A.X >= Resolution.X) || (A.X < 0) )
		{
			*pLine = NoLine;
			return( false );
		}
		// Zaridime, aby A.Y <= B.Y
		if( A.Y > B.Y )
		{
			SwapCount++;
			MySwap(A, B);		
		}
		if( (A.Y >= Resolution.Y) || (B.Y < 0) )
		{
			*pLine = NoLine;
			return( false );
		}		
		if( B.Y >= Resolution.Y )
			B.Y = Resolution.Y - 1;
		if( A.Y < 0 )
			A.Y = 0;
		if( SwapCount%2 == 1 )
			MySwap(A, B);
		return( true );
	}

	// Specialni pripad
	if( A.Y == B.Y )
	{
		// Je usecka mimo obraz ?
		if( (A.Y >= Resolution.Y) || (A.Y < 0) )
		{
			*pLine = NoLine;
			return( false );
		}
		// Zaridime, aby A.X <= B.X
		if( A.X > B.X )
		{
			SwapCount++;
			MySwap(A, B);
		}
		if( (A.X >= Resolution.X) || (B.X < 0) )
		{
			*pLine = NoLine;
			return( false );
		}	
		if( B.X >= Resolution.X )
			B.X = Resolution.X - 1;
		if( A.X < 0 )
			A.X = 0;
		if( SwapCount%2 == 1 )
			MySwap(A, B);
		return( true );
	}

	// Oriznuti podle Resolution.X

	// Zaridime, aby A.X <= B.X
	if( A.X > B.X )
	{
		SwapCount++;
		MySwap(A, B);
	}

	if( (A.X >= Resolution.X) || (B.X < 0) )
	{
		*pLine = NoLine;
		return( false );
	}

	if( A.X < 0 )
	{
		DeltaX = B.X - A.X;
		DeltaY = B.Y - A.Y;

		NewDeltaY = (B.X * DeltaY) / DeltaX;

		A.X = 0;
		A.Y = B.Y - NewDeltaY;
	}


	if( B.X >= Resolution.X )
	{
		DeltaX = A.X - B.X;
		DeltaY = A.Y - B.Y;

		NewDeltaY = ((Resolution.X - 1 - A.X) * DeltaY) / DeltaX;

		B.X = Resolution.X - 1;
		B.Y = A.Y + NewDeltaY;
	}


	// Oriznuti podle Resolution.Y

	// Zaridime, aby A.Y <= B.Y
	if( A.Y > B.Y )
	{
		SwapCount++;
		MySwap(A, B);
	}

	if( (A.Y >= Resolution.Y) || (B.Y < 0) )
	{
		*pLine = NoLine;
		return( false );
	}

	if( A.Y < 0 )
	{
		DeltaY = B.Y - A.Y;
		DeltaX = B.X - A.X;

		NewDeltaX = (B.Y * DeltaX) / DeltaY;

		A.Y = 0;
		A.X = B.X - NewDeltaX;
	}


	if( B.Y >= Resolution.Y )
	{
		DeltaY = A.Y - B.Y;
		DeltaX = A.X - B.X;

		NewDeltaX = ((Resolution.Y - 1 - A.Y) * DeltaX) / DeltaY;

		B.Y = Resolution.Y - 1;
		B.X = A.X + NewDeltaX;
	}

	if( SwapCount%2 == 1 )
		MySwap(A, B);

	return( true );
}


Point2D __stdcall LineCalculation_INF_DIRECTION(LineCalculationState *pState);
Point2D __stdcall LineCalculation_0_DIRECTION_1(LineCalculationState *pState);
Point2D __stdcall LineCalculation_M1_DIRECTION_0(LineCalculationState *pState);
Point2D __stdcall LineCalculation_1_DIRECTION(LineCalculationState *pState);
Point2D __stdcall LineCalculation_DIRECTION_M1(LineCalculationState *pState);

// Funkce pripravi vykreslovani usecky mezi body A a B, tj. inicializuje stav vypoctu pState
// a pro dany typ usecky vybere nejvhodnejsi (nejrychlejsi) funkci z nektere LineCalculation_...,
// kterou lze usecka rasterizovat. Vraci ukazatel na tuto funkci. Parameter Resolution udava 
// rozliseni plochy, na ktere usecku rasterizujeme, body ziskane algoritmem lezi vzdy uvnitr 
// tohoto rozliseni. Pokud je Resolution.X<0, volajici chce vypocitat celou usecku nezavisle na rozliseni.
// Principem vypoctu bodu usecky je Bresenhamuv algoritmus.
//
// Standartni pouziti tedy vypada asi nasledovne:
// 
//
//	Line2D                     line;            // Usecka, ktera se ma kreslit
//	Point2D                    resolution;      // Rozliseni obrazovky, na kterou vykreslujeme
//	LineCalculationState       state;           // Stav kresleni usecky
//	LineCalculationProcedure   proc;           // Procedura provadejici vypocet bodu na usecce
//	Point2D                    pixel;           // Pixel usecky spocitany v kazdem kroku
//	
//	// Nejaka usecka
//	line.A.X = -45;
//	line.A.Y = 112;
//	line.B.X = 578;
//	line.B.Y = 898;
//	
//	// Nejake rozliseni
//	resolution.X = 800;
//	resolution.Y = 600;
//
//	// Inicializace vypoctu bodu usecky	
//	proc = PrepareLineCalculation(&state, line, resolution);
//
//	// Samotny vypocet bodu na usecce
//	while( LineCalculationContinues(state) )
//	{
//		pixel = proc( &state ); // Pri kazdem volani vraci dalsi bod usecky
//		//...		
//	}
//

LineCalculationProcedure PrepareLineCalculation( LineCalculationState *pState, Line2D Line, Point2D Resolution)
{
	double Direction;
	Point2D A, B, HelpPoint;

	// Orizneme casti usecky mimo obraz (rozliseni)
	// Pokud je Resolution.X<0, volajici chce vypocitat celou usecku nezavisle na rozliseni
	if( Resolution.X>=0 )
		if( !ShortenLineUsingResolution( &Line, Resolution ) )
		{
			pState->RemainingIterations = 0;
			return( (LineCalculationProcedure)NULL );
		}

	A = Line.A;
	B = Line.B;

	if( (A.X - B.X) == 0 )
	{
		// Zaridime, aby A.Y <= B.Y
		if( A.Y > B.Y )
		{
			HelpPoint = A; A = B; B = HelpPoint;
		}

		pState->y = A.Y;
		pState->x = A.X;
		pState->RemainingIterations = B.Y - A.Y + 1;

		return( LineCalculation_INF_DIRECTION );
	}

	// Vypocet smernice usecky
	Direction = (double)(A.Y - B.Y) / (A.X - B.X);

	// Zaridime, aby A.X <= B.X
	if( A.X > B.X )
	{
		HelpPoint = A; A = B; B = HelpPoint;
	}

	// Zjistime, ktera funkce se nejlepe hodi pro vypocet usecky a inicializujeme pState
	if( (0 <= Direction) && (Direction <= 1) )
	{
		// A inicializujeme promenne
		pState->k1 = 2 * ( B.Y - A.Y );
		pState->k2 = 2 * ( (B.Y - A.Y) - (B.X - A.X) );
		pState->prediction = pState->k1 - ( B.X - A.X );
		pState->x = A.X;
		pState->y = A.Y;
		pState->RemainingIterations = B.X - A.X + 1;

		return( LineCalculation_0_DIRECTION_1 );
	}

	if( (-1 <= Direction) && (Direction <= 0) )
	{	
		// A inicializujeme promenne
		pState->k1 = 2 * ( A.Y - B.Y );
		pState->k2 = 2 * ( (A.Y - B.Y) - (B.X - A.X) );
		pState->prediction = pState->k1 - ( B.X - A.X );
		pState->x = A.X;
		pState->y = A.Y;
		pState->RemainingIterations = B.X - A.X + 1;

		return( LineCalculation_M1_DIRECTION_0 );
	}
	
	if( Direction > 1 )
	{		
		// A inicializujeme promenne
		pState->k1 = 2 * ( B.X - A.X );
		pState->k2 = 2 * ( (B.X - A.X) - (B.Y - A.Y) );
		pState->prediction = pState->k1 - ( B.Y - A.Y );
		pState->x = A.X;
		pState->y = A.Y;
		pState->RemainingIterations = B.Y - A.Y + 1;

		return( LineCalculation_1_DIRECTION );
	}

	// Jiz zbyva jen moznost Direction < -1
	pState->k1 = 2 * ( B.X - A.X );
	pState->k2 = 2 * ( (B.X - A.X) - (A.Y - B.Y) );
	pState->prediction = pState->k1 - ( A.Y - B.Y );
	pState->x = B.X;
	pState->y = B.Y;
	pState->RemainingIterations = A.Y - B.Y + 1;

	return( LineCalculation_DIRECTION_M1 );
}

Point2D __stdcall LineCalculation_INF_DIRECTION(LineCalculationState *pState)
{	
	Point2D RetVal;

	RetVal.X = pState->x;
	RetVal.Y = pState->y;

	// Dalsi iterace
	pState->y++;
	pState->RemainingIterations--;

	return( RetVal );
}

Point2D __stdcall LineCalculation_0_DIRECTION_1(LineCalculationState *pState)
{	
	Point2D RetVal;
	RetVal.X = pState->x;
	RetVal.Y = pState->y;

	// Dalsi iterace
	pState->x++;
	if( pState->prediction > 0 )
	{
		pState->y++;
		pState->prediction += pState->k2;
	}
	else
		pState->prediction += pState->k1;	

	pState->RemainingIterations--;

	return( RetVal );
}



Point2D __stdcall LineCalculation_M1_DIRECTION_0(LineCalculationState *pState)
{	
	Point2D RetVal;
	RetVal.X = pState->x;
	RetVal.Y = pState->y;

	// Dalsi iterace
	pState->x++;
	if( pState->prediction > 0 )
	{
		pState->y--;
		pState->prediction += pState->k2;
	}
	else
		pState->prediction += pState->k1;	

	pState->RemainingIterations--;

	return( RetVal );
}


Point2D __stdcall LineCalculation_1_DIRECTION(LineCalculationState *pState)
{	
	Point2D RetVal;
	RetVal.X = pState->x;
	RetVal.Y = pState->y;

	// Dalsi iterace
	pState->y++;
	if( pState->prediction > 0 )
	{
		pState->x++;
		pState->prediction += pState->k2;
	}
	else
		pState->prediction += pState->k1;	

	pState->RemainingIterations--;

	return( RetVal );
}


Point2D __stdcall LineCalculation_DIRECTION_M1(LineCalculationState *pState)
{	
	Point2D RetVal;
	RetVal.X = pState->x;
	RetVal.Y = pState->y;

	// Dalsi iterace
	pState->y++;
	if( pState->prediction > 0 )
	{
		pState->x--;
		pState->prediction += pState->k2;
	}
	else
		pState->prediction += pState->k1;	

	pState->RemainingIterations--;

	return( RetVal );
}



// Funkce nacte z obrazku usecku (resp. jeji barvy) a ty ulozi do linearniho pole.
// Pokud je pouzito vzorkovaci (Barlettovo) oko, musi byt usecka dostatecne daleko
// od okraje obrazku.
bool GetLineFromPicture(
	pColor32		pPicture,		// Obrazek
	const Point2D	&PictureRes,	// Rozliseni obrazku
	const Line2D	&LineSource,	// Usecka, ktera se bude nacitat. Cela musi byt uvnitr obrazku pPicture !!!
	pColor32		*ppLine,		// Sem se funkci malloc() alokuje pole a ulozi usecka s poradim barev od LineSource.A k LineSource.B. Pole je potreba pozdeji uvolnit funkci free()
	int				*pLineLength,	// Sem se ulozi delka usecky
	int				BarlettEyeDim	// Velikost strany vzorkovaciho oka, muze byt 3,5,7 nebo pri 0 se vubec nepouziva
	)
{
	LineCalculationState LState;
	LineCalculationProcedure LProc;
	Point2D Pixel;
	int Length;
	pColor32 WritePosition;
	int WriteDirection; 

	BarlettEyeProcedure *BarlettEye;

	*ppLine = NULL;

	// Jake oko pouzijeme ?
	switch( BarlettEyeDim )	
	{
		case 3: BarlettEye = (BarlettEyeProcedure *)BarlettEye3x3; break;
		case 5:	BarlettEye = (BarlettEyeProcedure *)BarlettEye5x5; break;
		case 7:	BarlettEye = (BarlettEyeProcedure *)BarlettEye7x7; break;
		default: BarlettEye = (BarlettEyeProcedure *)BarlettEye1x1; break;
	}

	// Kontrola, jestli je cela SourceLine uvnitr obrazu (i s prihlednutim
	// k velikosti okraje Barlettova oka)
	if( (LineSource.A.X < 0)
		|| (LineSource.A.Y < 0)
		|| (LineSource.B.X < 0)
		|| (LineSource.B.Y < 0)
		|| (LineSource.A.X >= PictureRes.X) 
		|| (LineSource.A.Y >= PictureRes.Y) 
		|| (LineSource.B.X >= PictureRes.X) 
		|| (LineSource.B.Y >= PictureRes.Y) 
		)
		return( false );
	
	// Vypocet delky usecky
	Length = MAX( abs(LineSource.A.X - LineSource.B.X),
	              abs(LineSource.A.Y - LineSource.B.Y) ) + 1;

	*ppLine = (pColor32)malloc( Length * sizeof(Color32) );
	if( !(*ppLine) )
		return( false );


	// A samotny "scan" usecky
	LProc = PrepareLineCalculation( &LState, LineSource, PictureRes );
	if( LineCalculationContinues(LState) )
	{
		Pixel = LProc( &LState );

		// Zjistime, jakym smerem barvy z usecky do radky ukladat, aby odpovidali
		// puvodnimu zadani SourceLine (tj. od SourceLine.A k SourceLine.B)
		if( (Pixel.X == LineSource.A.X) && (Pixel.Y == LineSource.A.Y) )
		{
			WritePosition = *ppLine;
			WriteDirection = 1;
		}
		else
		{
			WritePosition = *ppLine + (Length - 1);
			WriteDirection = -1;
		}

		*WritePosition = BarlettEye( pPicture, Pixel, PictureRes );				
		WritePosition += WriteDirection;

		// Vlastni cyklus ukladani barev z usecky
		while( LineCalculationContinues(LState) )
		{
			Pixel = LProc( &LState ); // zjistime dalsi pixel usecky						
			//*WritePosition = pPicture[ Pixel.Y*PictureRes.X + Pixel.X ];	
			*WritePosition = BarlettEye( pPicture, Pixel, PictureRes );
			WritePosition += WriteDirection;		
		} 

	}

	*pLineLength = Length;
	return( true );
}





// Funkce analyzuje trojuhelnik (Trian) na 2D plose a pripravi informace potrebne pro indexaci jeho radku
// (neboli pixelu na jeho radkach), coz se hodi napr. pro otexturovani. Resolution je rozliseni obrazu, na kterem
// trojuhelnik lezi. Vysledek se vraci parametrem Info, ktery lze uzit pro volani funkce GetTrianLineIndexes().
// Pixel se souradnicemi [0,0] je v levem dolnim rohu. Pokud vse uspeje, vraci se true, jinak false.
bool PrepareTrianLinesIndexing(Triangle2D Trian, Point2D Resolution, TrianLinesInfo &Info )
{
	Point2D Point;
	int VectMultZ;
			
	// Setridime body trojuhelniku podle polohy X vzestupne
	if(Trian.A.X > Trian.B.X)
		{ Point = Trian.A; Trian.A = Trian.B; Trian.B = Point; }
	if(Trian.A.X > Trian.C.X)
		{ Point = Trian.A; Trian.A = Trian.C; Trian.C = Point; }
	if(Trian.B.X > Trian.C.X)
		{ Point = Trian.B; Trian.B = Trian.C; Trian.C = Point; }
	
	// Overime, je-li vubec trojuhelnik v obraze - vyuzijeme setridenosti podle X
	if( (Trian.A.X >= Resolution.X) || (Trian.C.X < 0) )
		return( false );

	// Pokud jsou dva body trojuhelnika primo nad sebou (stejna X), nechame ten s mensim pismenem nize (podle Y)
	// Pripad A.X==C.X znamena, ze vsechny tri body jsou v jedne primce - odchytneme za chvili jako chybu
	if( (Trian.A.X == Trian.B.X) && (Trian.A.Y > Trian.B.Y) )
		{ Point = Trian.A; Trian.A = Trian.B; Trian.B = Point; }
	if( (Trian.B.X == Trian.C.X) && (Trian.B.Y > Trian.C.Y) )
		{ Point = Trian.B; Trian.B = Trian.C; Trian.C = Point; }			

	// Zakladni inicializace
	Info.BorderAB = false;
	Info.BorderBC = false;
	Info.BorderAC = false;
	Info.VectorAB.X = Trian.B.X - Trian.A.X;
	Info.VectorAB.Y = Trian.B.Y - Trian.A.Y;
	Info.VectorBC.X = Trian.C.X - Trian.B.X;
	Info.VectorBC.Y = Trian.C.Y - Trian.B.Y;
	Info.VectorAC.X = Trian.C.X - Trian.A.X;
	Info.VectorAC.Y = Trian.C.Y - Trian.A.Y;

	// Spocteme z-ovou souradnici vektoroveho soucinu vektoru BC a BA jakozto vektoru v prostoru
	VectMultZ =  ((int)Info.VectorBC.X*(int)Info.VectorAB.Y - (int)Info.VectorAB.X*(int)Info.VectorBC.Y);
	
	// Pokud je plocha trojuhelniku mala, zabivat se s nim nebudeme, pokud je nula, body trojuhelniku 
	// jsou v primce. Velikost konstanty vydelene dvemi urcuje maximalni plochu, musi byt vetsi nez 0.
	if( abs(VectMultZ) <= 1 )
		return( false );


	// Nalezneme hrany, ktere tvori levou hranici trojuhelnika a jeji nejnizsi a nejvyssi Y souradnici 
	// (index prvniho a posledniho radku v trojuhelniku brano zespoda).
	// Pouzijeme metodu probrani vsech moznosti, vyuzijeme predchozich uprav trojuhelnika.
	if(Trian.A.X==Trian.B.X) 
	{	
		// Vime jen ze plati A.Y<B.Y a A.X=B.X
		if(Trian.B.Y < Trian.C.Y)
			{ Info.BorderAB=true; Info.BorderBC=true; Info.StartLine=Trian.A.Y; Info.EndLine=Trian.C.Y; }
		else
			if(Trian.C.Y < Trian.A.Y)
				{ Info.BorderAB=true; Info.BorderAC=true; Info.StartLine=Trian.C.Y; Info.EndLine=Trian.B.Y; }
			else
				{ Info.BorderAB=true; Info.StartLine=Trian.A.Y; Info.EndLine=Trian.B.Y; }
	}
	else	
		if(Trian.B.X==Trian.C.X)
		{
			// Vime ze plati B.Y<C.Y a B.X=C.X
			if(Trian.B.Y >= Trian.A.Y)
				{ Info.BorderAC=true; Info.StartLine=Trian.A.Y; Info.EndLine=Trian.C.Y; }
			else
				if(Trian.A.Y >= Trian.C.Y)
					{ Info.BorderAB=true; Info.StartLine=Trian.B.Y; Info.EndLine=Trian.A.Y; }
				else
					{ Info.BorderAB=true; Info.BorderAC=true; Info.StartLine=Trian.B.Y; Info.EndLine=Trian.C.Y; }
		}
		else
		{	
			// Vime jen ze plati A.X < B.X < C.X, nic vic.
			if( (Trian.A.Y<=Trian.C.Y) && (Trian.C.Y<=Trian.B.Y) )
				{ Info.BorderAB=true; Info.StartLine=Trian.A.Y; Info.EndLine=Trian.B.Y; }
			else	
				if( (Trian.A.Y>=Trian.C.Y) && (Trian.C.Y>=Trian.B.Y) )
					{ Info.BorderAB=true; Info.StartLine=Trian.B.Y; Info.EndLine=Trian.A.Y; }
				else			
					if( (Trian.C.Y<Trian.A.Y) && (Trian.A.Y<Trian.B.Y) )
						{ Info.BorderAB=true; Info.BorderAC=true; Info.StartLine=Trian.C.Y; Info.EndLine=Trian.B.Y; }
					else
						if( (Trian.B.Y<Trian.A.Y) && (Trian.A.Y<Trian.C.Y) )
							{ Info.BorderAB=true; Info.BorderAC=true; Info.StartLine=Trian.B.Y; Info.EndLine=Trian.C.Y; }					
						else
						{   // Funkce f: { A.X, B.X, C.X } -> { A.Y, B.Y, C.Y } neklesajici/nerostouci, bijektivni
							if(Trian.A.Y < Trian.C.Y)
							{								
								// f neklesajici, je ale konvexni ?
								if( ((int)Info.VectorAB.Y*(int)Info.VectorBC.X) <= ((int)Info.VectorBC.Y*(int)Info.VectorAB.X) )
									{ Info.BorderAC=true; Info.StartLine=Trian.A.Y; Info.EndLine=Trian.C.Y; }
								else
									{ Info.BorderAB=true; Info.BorderBC=true; Info.StartLine=Trian.A.Y; Info.EndLine=Trian.C.Y; }
							}
							else
							{
								// f nerostouci, je konvexni ?
								if( ((int)Info.VectorAB.Y*(int)Info.VectorBC.X) <= ((int)Info.VectorBC.Y*(int)Info.VectorAB.X) )
									{ Info.BorderAB=true; Info.BorderBC=true; Info.StartLine=Trian.C.Y; Info.EndLine=Trian.A.Y; }
								else
									{ Info.BorderAC=true; Info.StartLine=Trian.C.Y; Info.EndLine=Trian.A.Y; }				
							}
						}
		}

	// Kontrola, je-li vubec trojuhelnik v obraze
	if( (Info.EndLine < 0) || (Info.StartLine >= Resolution.Y) )
		return( false );
	// a vynechani radku mimo obraz
	if( Info.StartLine<0 )
		Info.StartLine = 0;
	if( Info.EndLine >= Resolution.Y)
		Info.EndLine = Resolution.Y - 1;

	Info.SortedTrian = Trian;	
	Info.ResolutionX = Resolution.X;

	//	Uspech
	return( true );
}

#define TLI_INFINITY	(2000000000) // "nekonecne" velke cislo

// Funkce zjistuje x-ove souradnice zacatku a konce radku Line v trojuhelniku, ktery byl jiz analyzovan funkci
// PrepareTrianLinesIndexing() (jeho vlastnosti jsou predavany parametrem Info), vysledek (pocatecni a koncovy x-index
// radku) se vraci parametry StartIndex a EndIndex. Pokud vse uspeje, vraci se true, jinak false.
bool GetTrianLineIndexes(TrianLinesInfo &Info, int Line, int &StartIndex, int &EndIndex)
{
	// Neni radka mimo trojuhelnik ?
	if((Line<Info.StartLine) || (Line>Info.EndLine))
		return( false );
	
	// Vypocitame pruniky Line s hranami trojuhelnika, pro ulozeni vysledku pouzijeme
	// 3 slozky z Info vyhrazene jen pro toto. Kazdy z vyrazu pro nalezeni pruniku by sel
	// jeste zjednodusit o jedno nasobeni, snizila by se nam vsak uz tak dost mizerna celociselna presnost,
	// coz by mohlo zpusobit defekty na nekolika krajnich radcich pri vykreslovani povrchu trojuhelnika
	if(Info.VectorAB.Y != 0)
		Info.PenetAB = (Info.VectorAB.X*(Line-Info.SortedTrian.B.Y) + Info.SortedTrian.B.X*Info.VectorAB.Y)/Info.VectorAB.Y;
	else
		Info.PenetAB = -TLI_INFINITY; // Prunik neexistuje - nastavime "minus nekonecno"

	if(Info.VectorBC.Y != 0)
		Info.PenetBC = (Info.VectorBC.X*(Line-Info.SortedTrian.B.Y) + Info.SortedTrian.B.X*Info.VectorBC.Y)/Info.VectorBC.Y;
	else
		Info.PenetBC = -TLI_INFINITY; 

	if(Info.VectorAC.Y != 0)
		Info.PenetAC = (Info.VectorAC.X*(Line-Info.SortedTrian.C.Y) + Info.SortedTrian.C.X*Info.VectorAC.Y)/Info.VectorAC.Y;
	else
		Info.PenetAC = -TLI_INFINITY; // Prunik neexistuje - nastavime "minus nekonecno"

	// Hledana cast radku Line (ta co lezi v trojuhelniku) je mezi nejpravejsim prunikem s levymi hranicnimi 
	// hranami a nejblizsim (vpravo) dalsim prunikem
	if( Info.BorderAB )
		if( Info.BorderAC )
		{
			StartIndex = MAX(Info.PenetAB, Info.PenetAC);
			EndIndex = Info.PenetBC;
		}
		else
			if( Info.BorderBC )
			{
				StartIndex = MAX(Info.PenetAB, Info.PenetBC);
				EndIndex = Info.PenetAC;
			}
			else
			{ 
				// Pripad BorderAB
				StartIndex = Info.PenetAB;				
				EndIndex = MIN( (Info.PenetAC>=Info.PenetAB) ? Info.PenetAC : TLI_INFINITY,
				                (Info.PenetBC>=Info.PenetAB) ? Info.PenetBC : TLI_INFINITY );
			}
	else
	{
		// Pripad BorderAC
		StartIndex = Info.PenetAC;
		EndIndex = MIN( (Info.PenetAB>=Info.PenetAC) ? Info.PenetAB : TLI_INFINITY,
		                (Info.PenetBC>=Info.PenetAC) ? Info.PenetBC : TLI_INFINITY );
	}

	// Jeste kontrola, je-li reseni v obraze
	if( StartIndex > EndIndex )
		return( false );
	if( (StartIndex >= Info.ResolutionX) || (EndIndex < 0) )
		return( false );
	if( StartIndex<0 )
		StartIndex = 0;
	if( EndIndex >= Info.ResolutionX )
		EndIndex = Info.ResolutionX - 1;

	return( true ); // Uspech...
}



// Barlettovo oko 3x3, pPicture je zdrojovy obraz, Resolution jeho rozliseni a
// Where je stred oka, ten musi byt dostatecne daleko od okraje obrazu!
// Schema vzorkovani:
//	1,2,1
//	2,4,2
//	1,2,1
Color32 __stdcall BarlettEye3x3(Color32 *pPicture, Point2D Where, Point2D Resolution)
{
	unsigned long R=0, G=0, B=0;
	Color32 *Ptr;

	if( Where.X<1 || Where.X >=Resolution.X-1 || Where.Y<1 || Where.Y>=Resolution.Y-1 )
		return( pPicture[ Where.Y*Resolution.X + Where.X ] );

	// Optimalni kod ?
	// Rada 1,2,1
	Ptr = pPicture +(Where.Y-1)*Resolution.X + Where.X - 1;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );

	// Rada 2,4,2
	Ptr += Resolution.X - 2;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );

	// Rada 1,2,1
	Ptr += Resolution.X - 2;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );

	// Vypocet vysledku
	R /= 16;
	G /= 16;
	B /= 16;

	return( COLOR32_CREATE(R,G,B) );
}


// Barlettovo oko 5x5, pPicture je zdrojovy obraz, Resolution jeho rozliseni a
// Where je stred oka, ten musi byt dostatecne daleko od okraje obrazu!
// Schema vzorkovani:
//	1,2,3,2,1
//	2,4,6,4,2
//	3,6,9,6,3
//	2,4,6,4,2
//	1,2,3,2,1
Color32 __stdcall BarlettEye5x5(Color32 *pPicture, Point2D Where, Point2D Resolution)
{
	unsigned long R=0, G=0, B=0;
	Color32 *Ptr;

	if( Where.X<2 || Where.X >=Resolution.X-2 || Where.Y<2 || Where.Y>=Resolution.Y-2 )
		return( pPicture[ Where.Y*Resolution.X + Where.X ] );

	// Rada 1,2,3,2,1
	Ptr = pPicture +(Where.Y-2)*Resolution.X + Where.X - 2;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );

	// Rada 2,4,6,4,2
	Ptr += Resolution.X - 4;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	
	// Rada 3,6,9,6,3
	Ptr += Resolution.X - 4;	
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 9*COLOR32_GET_R( *Ptr ); G += 9*COLOR32_GET_G( *Ptr ); B += 9*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	
	// Rada 2,4,6,4,2
	Ptr += Resolution.X - 4;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );

	// Rada 1,2,3,2,1
	Ptr += Resolution.X - 4;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );

	// Vypocet vysledku
	R /= 81;
	G /= 81;
	B /= 81;

	return( COLOR32_CREATE(R,G,B) );
}


// Barlettovo oko 7x7, pPicture je zdrojovy obraz, Resolution jeho rozliseni a
// Where je stred oka, ten musi byt dostatecne daleko od okraje obrazu!
// Schema vzorkovani:
//	1, 2, 3, 4, 3, 2, 1
//	2, 4, 6, 8, 6, 4, 2
//	3, 6, 9,12, 9, 6, 3
//	4, 8,12,16,12, 8, 4
//	3, 6, 9,12, 9, 6, 3
//	2, 4, 6, 8, 6, 4, 2
//	1, 2, 3, 4, 3, 2, 1
Color32 __stdcall BarlettEye7x7(Color32 *pPicture, Point2D Where, Point2D Resolution)
{
	unsigned long R=0, G=0, B=0;
	Color32 *Ptr;

	if( Where.X<3 || Where.X >=Resolution.X-3 || Where.Y<3 || Where.Y>=Resolution.Y-3 )
		return( pPicture[ Where.Y*Resolution.X + Where.X ] );
	
	// Rada 1,2,3,4,3,2,1
	Ptr = pPicture +(Where.Y-3)*Resolution.X + Where.X - 3;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );

	// Rada 2,4,6,8,6,4,2
	Ptr += Resolution.X - 6;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 8*COLOR32_GET_R( *Ptr ); G += 8*COLOR32_GET_G( *Ptr ); B += 8*COLOR32_GET_B( *Ptr );
	Ptr++;	
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	
	// Rada 3,6,9,12,9,6,3
	Ptr += Resolution.X - 6;	
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 9*COLOR32_GET_R( *Ptr ); G += 9*COLOR32_GET_G( *Ptr ); B += 9*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 12*COLOR32_GET_R( *Ptr ); G += 12*COLOR32_GET_G( *Ptr ); B += 12*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 9*COLOR32_GET_R( *Ptr ); G += 9*COLOR32_GET_G( *Ptr ); B += 9*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );

	// Rada 4,8,12,16,12,8,4
	Ptr += Resolution.X - 6;	
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 8*COLOR32_GET_R( *Ptr ); G += 8*COLOR32_GET_G( *Ptr ); B += 8*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 12*COLOR32_GET_R( *Ptr ); G += 12*COLOR32_GET_G( *Ptr ); B += 12*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 16*COLOR32_GET_R( *Ptr ); G += 16*COLOR32_GET_G( *Ptr ); B += 16*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 12*COLOR32_GET_R( *Ptr ); G += 12*COLOR32_GET_G( *Ptr ); B += 12*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 8*COLOR32_GET_R( *Ptr ); G += 8*COLOR32_GET_G( *Ptr ); B += 8*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );

	// Rada 3,6,9,12,9,6,3
	Ptr += Resolution.X - 6;	
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 9*COLOR32_GET_R( *Ptr ); G += 9*COLOR32_GET_G( *Ptr ); B += 9*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 12*COLOR32_GET_R( *Ptr ); G += 12*COLOR32_GET_G( *Ptr ); B += 12*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 9*COLOR32_GET_R( *Ptr ); G += 9*COLOR32_GET_G( *Ptr ); B += 9*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );

	// Rada 2,4,6,8,6,4,2
	Ptr += Resolution.X - 6;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 8*COLOR32_GET_R( *Ptr ); G += 8*COLOR32_GET_G( *Ptr ); B += 8*COLOR32_GET_B( *Ptr );
	Ptr++;	
	R += 6*COLOR32_GET_R( *Ptr ); G += 6*COLOR32_GET_G( *Ptr ); B += 6*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );

	// Rada 1,2,3,4,3,2,1
	Ptr += Resolution.X - 6;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 4*COLOR32_GET_R( *Ptr ); G += 4*COLOR32_GET_G( *Ptr ); B += 4*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 3*COLOR32_GET_R( *Ptr ); G += 3*COLOR32_GET_G( *Ptr ); B += 3*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += 2*COLOR32_GET_R( *Ptr ); G += 2*COLOR32_GET_G( *Ptr ); B += 2*COLOR32_GET_B( *Ptr );
	Ptr++;
	R += COLOR32_GET_R( *Ptr ); G += COLOR32_GET_G( *Ptr ); B += COLOR32_GET_B( *Ptr );
	
	// Vypocet vysledku
	R /= 256;
	G /= 256;
	B /= 256;

	return( COLOR32_CREATE(R,G,B) );
}

// Procedura ktera
Color32 __stdcall BarlettEye1x1(Color32 *pPicture, Point2D Where, Point2D Resolution)
{
	return( *(pPicture + Where.Y*Resolution.X + Where.X) );
}



/*
// Funkce vraci vzdalenost bodu Point od primky Straight. Vyznam parametru HelpPoint 
// predavaneho odkazem je uveden nize. Funkce neprovadi kontrolu korektnosti parametru Straight. }
double GetDistance(StraightLine &Line, Point3D &Point, Point3D &HelpPoint)
{
	PlaneCoeff Plane;
	// Smerovy vektor primky je normalovym vektorem roviny k teto primce kolme,
	// koeficient d obecne rovnice teto roviny pak dopocitame tak, aby tato rovina
	// prochazela bodem Point. Vraci se vzdalenost pruniku teto roviny s primkou
	// Straight a bodu Point. Tento bod pruniku se jako parametr HelpPoint predava
	// odkazem zpet. 
	Plane.A = Line.Vector.X;
	Plane.B = Line.Vector.Y;
	Plane.C = Line.Vector.Z;
	Plane.D = -(Plane.A*Point.X + Plane.B*Point.Y + Plane.C*Point.Z);
	GetPenetrationPoint(Line, Plane, HelpPoint);
	return( sqrt(sqr(Point.X-HelpPoint.X) + sqr(Point.Y-HelpPoint.Y) + sqr(Point.Z-HelpPoint.Z)) );
}


*/


