/* MyProcs.h : Hlavickovy soubor s ruznymi procedurami a objekty hlavne pro graficke graficke vypocty
   Copyright(c) 2002,2003 Jan Curn (Jan.Curn@seznam.cz) 
*/


#ifndef _MYPROCS_H_INCLUDED_
#define _MYPROCS_H_INCLUDED_

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <crtdbg.h>


#ifndef MAXPATH
	#ifdef MAX_PATH
		#define MAXPATH		(MAX_PATH)
	#else
		#define MAXPATH		(260)
	#endif 
#endif


#ifndef MIN
	#define MIN(A,B)	(((A)<(B))?(A):(B))
	#define MAX(A,B)	(((A)>(B))?(A):(B)) /*
	#define MIN
	#define MAX

	template< class T >
	T MIN(const T &A, const T &B)
	{
		return( (A < B) ? A : B );
	}

	template< class T >
	T MAX(const T &A, const T &B)
	{
		return( (A > B) ? A : B );
	} // */
#endif


#ifndef PI
	#define PI	 (3.1415926535897932384626433832795)
#endif


// Hodne male cislo pro ruzna porovnavani ( viz napr. InBall() )
#define SmallNum	0.000001


// Souradnice bodu a vektor ve 2D a 3D prostoru
template <class T>
struct Point2D_Ex {
	T X, Y;
};
template <class T>
struct Point3D_Ex {
	T X, Y, Z;
};

typedef Point3D_Ex<double> Point3D, *pPoint3D, Vector3D, *pVector3D, Coord3D, *pCoord3D;
typedef Point2D_Ex<signed short> Point2D, *pPoint2D, Vector2D, pVector2D;
// Meze polozek objektu Point2D
#define MAXPOINT2D	(32700)
#define MINPOINT2D	(-32700)	


// Typ pro ulozeni 24-bitove (barvy ve formatu 0x00rrggbb, pokud pole techto objektu predstavuje pixely nejakeho obrazku,
// plati konvence ukladani od nejnizsi radky po nejvyssi, kazdou radku zleva do prava. Priklad:
//  Obrazek:          Pole objektu Color32 v tomto poradi:
//  09 10 11 12        [01,02,03,04,05,06,07,08,09,10,11,12]
//  05 06 07 08
//  01 02 03 04 
typedef unsigned long Color32, *pColor32; 

#define COLOR32_PRECISION			256


#define COLOR32_CREATE( R, G, B )	( ((Color32)(R) << 16) | ((Color32)(G) << 8) | (Color32)(B) )

// Makra vraceji po rade relativni intenzitu (0-255) cervene, zelene a modre slozky barvy COLOR (typu Color32)
#define COLOR32_GET_R(COLOR)		(((COLOR) >> 16) & 0x000000ff)
#define COLOR32_GET_G(COLOR)		(((COLOR) >> 8) & 0x000000ff)
#define COLOR32_GET_B(COLOR)		((COLOR) & 0x000000ff)


// Makra zmeni po rade relativni intenzitu (0-255) cervene, zelene a modre slozky barvy COLOR (typu Color32)
// na VAL, vysledek - upravena barva - se vraci
#define COLOR32_SET_R(COLOR, VAL)	(((COLOR) & 0x0000ffff) | ((Color32)(VAL) << 16))
#define COLOR32_SET_G(COLOR, VAL)	(((COLOR) & 0x00ff00ff) | ((Color32)(VAL) << 8))
#define COLOR32_SET_B(COLOR, VAL)	(((COLOR) & 0x00ffff00) | (Color32)(VAL))


// Typ pro ulozeni barvy ve formatu HLS (Hue=ton, Lightness=svetlost, Saturation=sytost)
// Ma tvar 0xx00HHLLSS, kde
typedef unsigned long ColorHLS, *pColorHLS;

#define COLORHLS_CREATE( H, L, S )	( ((ColorHLS)(H) << 16) | ((ColorHLS)(L) << 8) | (ColorHLS)(S) )

#define COLORHLS_GET_H(COLOR)		(((COLOR) >> 16) & 0x000000ff)
#define COLORHLS_GET_L(COLOR)		(((COLOR) >> 8) & 0x000000ff)
#define COLORHLS_GET_S(COLOR)		((COLOR) & 0x000000ff)




// Funkce pro manipulaci s barvami
inline int GetHDifference( ColorHLS Col1, ColorHLS Col2 )
	{	return(abs( (int)COLORHLS_GET_H( Col1 ) - (int)COLORHLS_GET_H( Col2 ) )); }
inline int GetLDifference( ColorHLS Col1, ColorHLS Col2 )
	{	return(abs( (int)COLORHLS_GET_L( Col1 ) - (int)COLORHLS_GET_L( Col2 ) )); }
inline int GetSDifference( ColorHLS Col1, ColorHLS Col2 )
	{	return(abs( (int)COLORHLS_GET_S( Col1 ) - (int)COLORHLS_GET_S( Col2 ) )); }

ColorHLS ConvertRGBtoHLS( Color32 RGB_Color );
Color32 ConvertHLStoRGB( ColorHLS HLS_Color );


Color32 __stdcall BarlettEye1x1(Color32 *pPicture, Point2D Where, Point2D Resolution);	
Color32 __stdcall BarlettEye3x3(Color32 *pPicture, Point2D Where, Point2D Resolution);
Color32 __stdcall BarlettEye5x5(Color32 *pPicture, Point2D Where, Point2D Resolution);
Color32 __stdcall BarlettEye7x7(Color32 *pPicture, Point2D Where, Point2D Resolution);

typedef Color32 (__stdcall BarlettEyeProcedure)(Color32 *pPicture, Point2D Where, Point2D Resolution);
typedef BarlettEyeProcedure *pBarlettEyeProcedure;




// Typy pro ulozeni trojuhelniku
typedef struct { Point3D A, B, C; } Triangle3D, *pTriangle3D;
typedef struct { Point2D A, B, C; } Triangle2D, *pTriangle2D;
 

// Typ ukladajici koeficienty obecne rovnice roviny v prostoru (AX+BY+CZ+D=0) 
typedef struct {
	Vector3D  Normal; // Normala, odpovida vektoru (A, B, C)
	double    D;       
} PlaneCoeff, *pPlaneCoeff;
   

// Matice 4x4 double
typedef double Matrix4x4[16];
typedef Matrix4x4 *pMatrix4x4;


// Typ pro ulozeni parametricky zadane primky
typedef struct {
	Point3D   Point;  // Ridici bod
	Vector3D  Vector; // Smerovy vektor 
} StraightLine, *pStraightLine;

// Typ pro ulozeni usecky ve 2D 
typedef struct {
	Point2D A, B;
} Line2D, *pLine2D;


// Sablona pro ulozeni pole objektu libovolneho typu T, pole muze menit svou velikost.
// Pri vytvareni objektu typu T se vola metoda void T::Initialize(), pri niceni
// se vola void T::Release(), obe metody musi trida T implementovat !!!
#define REALLOCFIELD_ALLOC_STEP		(16)
template <class T>
class ReAllocField
{

public:

	// Pole s polozkami pole
	T *Field;

	// Pocet alokovanych polozek, pokud je -1, zadna polozka neni alokovana, pokud je
	// 0, je alokovano pole o nulove velikosti
	unsigned long ItemsAllocated;

	// Pocet pouzitych prvku pole
	unsigned long ItemsUsed;

	// Zakladni inicializace
	void Initialize()
	{
		Field = NULL;
		ItemsAllocated = 0;
		ItemsUsed = 0;
	}

	// Uvolni pole, zavola destruktory
	void Release()
	{
		if( Field )
		{
			for(unsigned int i=0; i<ItemsAllocated; i++)
				Field[ i ].Release();		
			free( Field );
		}
		Initialize();
	}
	
	// Konstruktor
	ReAllocField()
		{ Initialize(); }

	// Destruktor
	~ReAllocField()
		{ Release(); }


	// Metoda meni velikost pole Field na NewItems, odpovidajici si objekty v nove vytvorenem
	// a puvodnim poli si odpovidaji, pro ostatni jsou zavolany T::Initialize()/T::Release() 
	// (pri ubirani polozek T::Initialize(), pri pridavani T::Release())
	// Pri uspechu vraci true
	// Pri selhani vraci false, puvodni pole muze byt zmeneno
	bool ReAlloc(unsigned long NewItems)
	{	
		T *HelpPtr;
		unsigned long i;

		if( NewItems == 0 )
		{
			Release();
			return( true );			
		}

		// Pri zmensovani pole uvolnime prebytecne objekty T
		for( i=NewItems; i<ItemsAllocated; i++)
			Field[ i ].Release();

		HelpPtr = (T *)realloc( Field, sizeof(T)*NewItems );

		if( !HelpPtr )
		{
			// Byly nektere polozky destruktovany?
			if( NewItems < ItemsAllocated )
				ItemsAllocated = NewItems;
			return( false );
		}		

		for( i=ItemsAllocated; i<NewItems; i++)
			HelpPtr[ i ].Initialize();

		Field = HelpPtr;		
		ItemsAllocated = NewItems;

		return( true );
	}

	// Prida do pole jednu polozku, pokud je prilis male, prialokuje pamet o REALLOCFIELD_ALLOC_STEP prvku.
	// Pokud vse uspeje, vraci se pointer na nove pridany prvek, jinak NULL
	T *AddItem()
	{
		if( ItemsUsed < ItemsAllocated )
			return( &Field[ ItemsUsed++ ] );	

		if( ReAlloc(ItemsUsed + REALLOCFIELD_ALLOC_STEP) )
			return( &Field[ ItemsUsed++ ] );
		else
			return( NULL ); // Chyba
	}

	// Prida do pole jednu polozku Item typu T, pouziva k tomu standartni (nebo pretypovany)
	// Pokud vse uspeje, vraci se pointer na nove pridany prvek, jinak NULL
	T *AddItem(const T &Item)
	{
		if( ItemsUsed < ItemsAllocated )
		{
			Field[ ItemsUsed ] = Item;
			return( &Field[ ItemsUsed++ ] );
		}

		if( ReAlloc(ItemsUsed + REALLOCFIELD_ALLOC_STEP) )
		{
			Field[ ItemsUsed ] = Item;
			return( &Field[ ItemsUsed++ ] );
		}
		else
			return( NULL ); // Chyba
	}

	inline bool IsFull()
	{
		return( ItemsUsed >= ItemsAllocated );
	}


	inline T *GetLastItem()
	{
		return &Field[ ItemsUsed - 1 ];
	}


};


template <class T>
inline void MySwap(T &A, T &B)
{
	T C = B; B = A; A = C;
}


template< class T > 
void MiniSort( T *Field, int *Indexes, int IndexesCount, bool __cdecl IsGreater(const void *pA, const void *pB) )
{
	switch( IndexesCount )
	{
	case 2:
		if( IsGreater( &Field[Indexes[0]], &Field[Indexes[1]] ) )
			MySwap( Indexes[0], Indexes[1] );
		return;

	case 3:
		if( IsGreater( &Field[Indexes[0]], &Field[Indexes[1]] ) )
			MySwap( Indexes[0], Indexes[1] );
		
		if( IsGreater( &Field[Indexes[0]], &Field[Indexes[2]] ) )
			MySwap( Indexes[0], Indexes[2] );

		if( IsGreater( &Field[Indexes[1]], &Field[Indexes[2]] ) )
			MySwap( Indexes[1], Indexes[2] );
		return;

	case 4:
		if( IsGreater( &Field[Indexes[0]], &Field[Indexes[1]] ) )
			MySwap( Indexes[0], Indexes[1] );

		if( IsGreater( &Field[Indexes[2]], &Field[Indexes[3]] ) )
			MySwap( Indexes[2], Indexes[3] );

		if( IsGreater( &Field[Indexes[0]], &Field[Indexes[2]] ) )
			MySwap( Indexes[0], Indexes[2] );

		if( IsGreater( &Field[Indexes[1]], &Field[Indexes[3]] ) )
			MySwap( Indexes[1], Indexes[3] );

		if( IsGreater( &Field[Indexes[1]], &Field[Indexes[2]] ) )
			MySwap( Indexes[1], Indexes[2] );
		return;
	}
}


template< class T > 
void MyBubbleSort( T *Field, int *Indexes, int IndexesCount, bool __cdecl IsGreater(const void *pA, const void *pB) )
{
	int i, j;
	for(i=1; i<IndexesCount; i++)
		for(j=IndexesCount-1; j>=i; j--)
			if( IsGreater( &Field[Indexes[j-1]], &Field[Indexes[j]] ) )
				MySwap( Indexes[j-1], Indexes[j] );
}



template< class T > 
void MyQuickSort( T *Field, int *Indexes, int IndexesCount, bool __cdecl IsGreater(const void *pA, const void *pB) )
{	
	int *StartPtr, *EndPtr, *Recurse;	

	if( IndexesCount <= 4 )
	{
		// Jiz jen velmi malo tridenych prvku, ukoncime rekurzivni vnorovani
		MiniSort( Field, Indexes, IndexesCount, IsGreater );
		return; 
	}

	T &Median = Field[ Indexes[rand()%IndexesCount] ];

	StartPtr = Indexes;
	EndPtr = Indexes + IndexesCount - 1;

	// Prehazime pole Indexes podle medianu, prvky mensi nebo rovny medianu budou na zacatku,
	// ostatni na konci
	while( StartPtr < EndPtr )
		if( ! IsGreater( &Field[*StartPtr], &Median) ) // Field[*StartPtr] <= Median			
		{
			if( IsGreater( &Field[*EndPtr], &Median) ) // Field[*EndPtr] > Median
			{
				StartPtr ++;
				EndPtr --;				
			}
			else // Field[*EndPtr] <= Median
				StartPtr ++;			
		}
		else // Field[*StartPtr] > Median	
		{
			if( IsGreater(&Field[*EndPtr], &Median) ) // Field[*EndPtr] > Median
				EndPtr --;						
			else // Field[*EndPtr] <= Median
			{
				MySwap( *StartPtr, *EndPtr );
				StartPtr ++;
				EndPtr --;
			}
		}	

	Recurse = StartPtr;	
	if( StartPtr == EndPtr )
		if( ! IsGreater(&Field[*StartPtr], &Median) )
			Recurse ++;

	// Pokud je median vetsi nebo roven vsem tridenym prvkum, pokracujeme radsi bublinkovym tridenim,
	// protoze vsechny prvky by mohly byt stejne, a quicksort by se nam tu rekurzivne zacyklil
	if( Recurse-Indexes == IndexesCount )
	{
		MyBubbleSort( Field, Indexes, IndexesCount, IsGreater );
	}
	else
	{
		MyQuickSort( Field, Indexes, Recurse - Indexes, IsGreater );
		MyQuickSort( Field, Recurse, Indexes + IndexesCount - Recurse, IsGreater );
	}
}


// Objekt pro ulozeni obrazku s pixely ve formatu Color32
struct MyPicture {

	Point2D Resolution;
	pColor32 AllocatedSpace;
	pColor32 *Rows;	

	void Initialize()
	{
		Resolution.X = Resolution.Y = 0;
		AllocatedSpace = NULL;
		Rows = NULL;
	}

	void Release()
	{
		int i;
		if( AllocatedSpace )
		{
			// Radky jsou alokovany na jednom miste - v AllocatedSpace			
			free( AllocatedSpace );
			_ASSERT( Rows != NULL );
			free( Rows );
		}
		else
		{
			// Kazda radka je alokovana zvlast v poli Lines
			if( Rows )
			{
				for(i=0; i<Resolution.Y; i++)
					free( Rows[i] );				
				free( Rows );
			}
		}		
		Initialize();
	}

	// Metoda alokuje radky pro obrazek o rozliseni NewResolution, pokud je Separately==true,
	// alokuje se kazdy radek zvlast, jinak se obrazek alokuje v jednom bloku (s pixely zacinajicimi
	// v levem spodnim rohu, pokracujici po radkach nahoru).
	// Pokud vse uspeje, vraci se true, jinak false.
	bool AllocateRows( Point2D NewResolution, bool Separately )
	{
		int i,j;
		pColor32 ColorPtr;
		Release();

		if( NewResolution.X<=0 || NewResolution.Y<=0 )
			return( false );		

		Rows = (pColor32 *)malloc( sizeof(pColor32) * NewResolution.Y );
		if( !Rows )
			return( false );

		if( Separately )
		{
			// Radky alokujeme oddelene v poli Rows
			for( i=0; i<NewResolution.Y; i++ )
			{
				Rows[i] = (pColor32)malloc( sizeof(Color32) * NewResolution.X );
				if( !Rows[i] )
				{
					// Dalsi radek nelze alokovat, musime tedy uvolnit vsechny jiz drive alokovane
					for(j=0; j<i; j++)
						free( Rows[j] );
					free( Rows );
					Rows = NULL;
					return( false );
				}
			}
		}
		else
		{
			// Radky alokujeme spolecne v AllocatedSpace, budou zde "ukladany" od spodniho k hornimu
			AllocatedSpace = (pColor32)malloc( NewResolution.X * NewResolution.Y * sizeof(Color32) );
			if( !AllocatedSpace )
			{
				free( Rows );
				Rows = NULL;
				return( false );
			}

			// A ukazatele na radky v poli Rows nasmerujeme do AllocatedSpace, aby to odpovidalo
			ColorPtr = AllocatedSpace;
			for( i=0; i<NewResolution.Y; i++ )
			{
				Rows[i] = ColorPtr;
				ColorPtr += NewResolution.X;
			}
		}
		
		Resolution = NewResolution;
		return( true );
	}
};






 

// Hlavicky BMP souboru
#define SIZEOF_MYBITMAPFILEHEADER	(14)
#define SIZEOF_MYBITMAPINFOHEADER	(40)
#define MYBITMAPFILEHEADER_BFTYPE	(19778) // Retezec "BM"
typedef struct {
	unsigned short  bfType; 
	unsigned long   bfSize; 
	unsigned short  bfReserved1; 
	unsigned short  bfReserved2; 
	unsigned long   bfOffBits; 
} MYBITMAPFILEHEADER; 
typedef struct { 
	unsigned long   biSize; 
	unsigned long   biWidth; 
	unsigned long   biHeight; 
	unsigned short  biPlanes; 
	unsigned short  biBitCount; 
	unsigned long   biCompression; 
	unsigned long   biSizeImage; 
	unsigned long   biXPelsPerMeter; 
	unsigned long   biYPelsPerMeter; 
	unsigned long   biClrUsed; 
	unsigned long   biClrImportant; 
} MYBITMAPINFOHEADER; 


// Typ pro ulozeni relativni polohy bodu vzhledem ke trojuhelniku - viz. fce GetPointRelationToTriangle
typedef Point3D PointRTT, *pPointRTT;


// Typ pro ulozeni stavu vypoctu bodu na primce - viz. fce PrepareLineCalculation
typedef struct {
	long k1, k2, prediction, x, y, i;
	long RemainingIterations;
} LineCalculationState;

// Procedura starajici se o vypocet bodu na primce - viz. fce PrepareLineCalculation
typedef Point2D (__stdcall *LineCalculationProcedure)(LineCalculationState *pState);
//typedef LineCalculationProcedure *pLineCalculationProcedure;
LineCalculationProcedure PrepareLineCalculation( LineCalculationState *pState, Line2D Line, Point2D Resolution);

// Typ pro ulozeni aktualniho stavu vypoctu car, ze kterych se sklada trojuhelnik - viz fce. PrepareTrianLinesIndexing
typedef struct {
	Triangle2D SortedTrian; // Puvodni trojuhelnik setrideny podle podle rostouci X-ove souradnice vrcholu
	int StartLine, EndLine; // Nejnizsi a nejvyssi radka (Y-souradnice) trojuhelniku
	int ResolutionX;	// X-ove rozliseni obrazu
	bool BorderAB, BorderBC, BorderAC; // Ktere hrany jsou leve hranicni
	int PenetAB, PenetBC, PenetAC; // Pomocnne promenne 
	Point2D VectorAB, VectorBC, VectorAC; // Vektory prislusejici danym stranam
} TrianLinesInfo;






// Deklarace funkci z 3DProcs.cpp
//void MemorySet(unsigned long *Pointer, unsigned long Elem, unsigned long ElemsCount, unsigned long *TempBuffer, unsigned long BufferCount);
bool ReadBitmap(const char *Filename, Point2D *pResolution, pColor32 *ppBitmap);
bool ReadJPEG(const char *Filename, Point2D *pResolution, Color32 **ppData);

bool ReadPictureFromFile(const char *Filename, MyPicture *pPicture);


bool GetPointRelationToTriangle(const Triangle3D &Trian, const Point3D &Point, PointRTT *pCoeffs);
void SetPointRelationToTriangle(const Triangle3D &Trian, Point3D *pPoint, const PointRTT &Coeffs);

bool GetNearestPoints(const StraightLine &Line1, const StraightLine &Line2, Point3D &Point1, Point3D &Point2);

void CutPath(char *FilePath);

bool ShortenLineUsingResolution(Line2D *pLine, Point2D Resolution);

bool GetLineFromPicture(pColor32 pPicture, const Point2D &PictureRes, const Line2D &LineSource, pColor32 *ppLine, int *pLineLength, int BarlettEyeDim);

bool PrepareTrianLinesIndexing(Triangle2D Trian, Point2D Resolution, TrianLinesInfo &Info );
bool GetTrianLineIndexes(TrianLinesInfo &Info, int Line, int &StartIndex, int &EndIndex);

bool GetPenetrationPoint(double *pResultCoeff, const StraightLine &Line, const Point3D &PointA, const Point3D &PointB);


// Funkce vrati jas barvy Color, ten lezi v rozmezi 0...255
inline unsigned char GetBrightness(const Color32 &Color)
{	
	unsigned char *BytePtr = (unsigned char *)&Color;
	return( ((unsigned long)BytePtr[2]*76/* red */
	         + (unsigned long)BytePtr[1]*150/* green */
	         + (unsigned long)BytePtr[0]*29/* blue */) / 256 );
}

// Funkce vraci true, pokud jeste usecka nebyla cela dokreslena, pokud je kresleni jiz u konce, vraci false.
// viz fce PrepareLineCalculation.
inline bool LineCalculationContinues( const LineCalculationState &State )
{
	return( (State.RemainingIterations > 0) ? true : false );
}


// Funkce vraci true, je-li X prvkem kruhoveho Epsilon-okoli bodu Y (nebo naopak), jinak vraci false.
// Da se tedy pouzit pro porovnavani cisel, ktera se lisi pouze na nejnizsich radech.
inline bool InBall(double X, double Y, double Epsilon)
{
	return( (fabs(X-Y) < Epsilon) ? true : false );
}

// Funkce vraci druhou mocninu argumentu X
inline double sqr(double X)
{
	return( X*X );
}

inline int sqr(int x)
{
	return( x*x );
}


// Funkce vraci vektorovy soucin UxV
inline Vector3D VectorMultiply(const Vector3D &U, const Vector3D &V)
{
	Vector3D Res;
	Res.X = (U.Y*V.Z)-(V.Y*U.Z);
	Res.Y = (U.Z*V.X)-(V.Z*U.X);
	Res.Z = (U.X*V.Y)-(V.X*U.Y);
	return( Res );
}

// Funkce vraci skalarni soucin vektoru U a V
inline double ScalarMultiply(const Vector3D &U, const Vector3D &V)
{
	return ( U.X*V.X + U.Y*V.Y + U.Z*V.Z );
}

// Funkce vraci skalarni soucin vektoru U a V
inline double ScalarMultiply(const Vector3D &V)
{
	return ( V.Y*V.Y + V.X*V.X + V.Z*V.Z );
}

// Funkce vraci normu vektoru V
inline double GetNorm(const Vector3D &V)
{
	return ( sqrt(ScalarMultiply(V)) );
}

// Funkce vraci vektor z bodu A do B
inline Vector3D GetVector(const Point3D &A, const Point3D &B)
{
	Vector3D Res;
	Res.X = B.X - A.X;
	Res.Y = B.Y - A.Y;
	Res.Z = B.Z - A.Z;
	return( Res );
}

// Funkce vraci bod, ktery vznikne prictenim vektoru Vector k bodu Point. Lze uzit i pro scitani vektoru.
inline Point3D AddVector(const Point3D &Point, const Vector3D &Vector)
{
	Point3D Res;
	Res.X = Point.X + Vector.X;
	Res.Y = Point.Y + Vector.Y;
	Res.Z = Point.Z + Vector.Z;
	return( Res );
}

// Funkce vraci bod, ktery vznikne prictenim Coeff-nasobku vektoru Vector k bodu Point,
// tedy Vysledek = Point + Coeff*Vector.
inline Point3D AddVector(const Point3D &Point, const Vector3D &Vector, const double &Coeff)
{
	Point3D Res;
	Res.X = Point.X + Coeff*Vector.X;
	Res.Y = Point.Y + Coeff*Vector.Y;
	Res.Z = Point.Z + Coeff*Vector.Z;
	return( Res );
}


// Funkce transformuje v prostoru bod Point transformacni matici Trans. Jedna se vlastne o nasobeni
// bodu (doplneneho o vahu 1) s transformacni matici.
inline Point3D TransformPoint(Matrix4x4 *Trans, const Point3D &Point)
{
	Point3D Res;
	Res.X = (*Trans)[0]*Point.X + (*Trans)[4]*Point.Y + (*Trans)[8]*Point.Z + (*Trans)[12];
	Res.Y = (*Trans)[1]*Point.X + (*Trans)[5]*Point.Y + (*Trans)[9]*Point.Z + (*Trans)[13];
	Res.Z = (*Trans)[2]*Point.X + (*Trans)[6]*Point.Y + (*Trans)[10]*Point.Z + (*Trans)[14];
	return( Res );
}


// Funkce nalezne obecnou rovnici roviny plochy trojuhelnika Trian. Neprovadi se kontrola parametru, neni-li
// tedy Trian korektni trojuhelnik (body v primce), vysledek take neni korektni
inline PlaneCoeff GetPlaneOfTriangle(const Triangle3D &Trian)
{
	PlaneCoeff Plane;	
	Plane.Normal = VectorMultiply( GetVector(Trian.A, Trian.B), GetVector(Trian.A, Trian.C) );
	Plane.D = - ScalarMultiply(Plane.Normal, Trian.A);
	return( Plane );
}


// Funkce zjisti, zda-li muze existovat trojuhelnik o stranach a, b, c, pokud ano, vraci se true, jinak false.
inline bool IsTriangle(double a, double b, double c)
{
	if(((a+b) > c) && ((a+c) > b) && ((b+c) > a))
		return( true );
	else
		return( false );
}

// Funkce vraci vzdalenost bodu A a B
inline double GetDistance(const Point3D &A, const Point3D &B)
{
	return( sqrt(sqr(A.X-B.X) + sqr(A.Y-B.Y) + sqr(A.Z-B.Z)) );
}

// Funkce overi, jestli vzdalenosti stran trojuhelnika a, b, c odpovidaji s presnosti Precision trojuhelniku Trian
// ( Trian.A-Trian.B...c, Trian.B-Trian.C...a, Trian.C-Trian.A...b ), pokud ano, vraci se true, jinak false.
// Hodnota parametru Trian se nemeni.
inline bool TriangleConformity(const Triangle3D &Trian, double a, double b, double c, double Precision)
{
	if(InBall(GetDistance(Trian.A, Trian.B), c, Precision)
	   && InBall(GetDistance(Trian.B, Trian.C), a, Precision)
	   && InBall(GetDistance(Trian.C, Trian.A), b, Precision))
		return( true );
	else
		return( false );
}



// Funkce vypocita prunik primky Line a roviny zadane parametrem Plane. Vysledek - realny parametr pro
// parametrickou rovnici primky se ulozi do parametru Coeff predavaneho odkazem. Bod pruniku je pote
// roven vyrazu AddVector(Line.Point, Line.Vector, Coeff). Pokud prunik existuje, vraci se true, jinak 
// false. Vsechny parametry krome Coeff funkce nemeni.
inline bool GetPenetrationPoint(double &Coeff, const StraightLine &Line, const PlaneCoeff &Plane)
{
	double Help;
	Help = ScalarMultiply( Plane.Normal, Line.Vector );	
	if( Help == 0 )
		return( false );	
	Coeff = -( ScalarMultiply(Plane.Normal, Line.Point) + Plane.D ) / Help;
	return( true );
}

inline bool GetPenetrationPoint(double *pCoeff, const StraightLine &Line, const PlaneCoeff &Plane)
{
	return( GetPenetrationPoint(*pCoeff, Line, Plane) );
}



// Funkce vrati koeficient rovnobeznosti vektoru U a V z intervalu <0,1>, tj.:
//    - pokud jsou U a V rovnobezne, vraci se 0
//    - pokud jsou U a V kolme, vraci se 1
//    - "cim vice" se U a V blizi rovnobeznosti, tim blize jde navratova hodnota k 0 zprava
//    - "cim vice" se U a V blizi kolmosti, tim blize jde navratova hodnota k 1 zleva
//    - pokud je U nebo V nulovy vektor, vraci se -1
inline double GetParallelityCoefficient(const Vector3D &U, const Vector3D &V)
{
	double HelpVal = GetNorm(U) * GetNorm(V);
	if( HelpVal == 0 )
		return (-1.0);
	else
		return( GetNorm(VectorMultiply(U,V)) / (HelpVal) );
}


// Prevod textu s cislem na float. Pokud vse uspeje, ulozi funkce do Number cislo z Text a vrati true.
// Pri neuspechu se vraci false. Funkce zmeni znaky ',' v retezci Text na '.'. 
inline bool TextToFloat(float *Number, char *Text)
{
	return( sscanf(Text, "%f", Number) == 1 );
}

// From < Centre < To
inline double PseudoGaussRand(double From, double Centre, double To)
{
	float rnd = (float)(rand()*2)/RAND_MAX - 1;	
	rnd = rnd * rnd * rnd;
	if( rnd >= 0 )
		return( Centre + rnd*(To-Centre) );	
	else
		return( Centre + rnd*(Centre-From) );	
}



#endif // _MYPROCS_H_INCLUDED_
