

#include "Render.h"
#include <memory.h>  
#include <string.h> 
#include <malloc.h>





// Funkce otoci bod pohledu na scenu kolem bodu Point o relativni uhly HorAngleInc a VerAngleInc,
// zaroven nastavi pohled (presneji horizontalni a vertikalni uhel pohledu) primo na Point (ten tedy bude 
// po renderingu uprostred obrazovky). Vsechny zmeny pohledu se provedou v parametru Params.
// Pokud vse uspeje, vraci se true, jinak false (v tomto pripade zustane Params nezmenen).
bool RotateViewAroundPoint(ViewParams *pParams, const Point3D &Point, double HorAngleInc, double VerAngleInc)
{
	ViewParams &Params = *pParams;

	double HorAngle1, HorAngle3;
	double VerAngle, Distance, HelpVal1;
	Point3D RelPosition;

	// Bod kolem ktereho rotujeme (Point) si "preneseme" do [0,0,0]
	RelPosition = GetVector(Point, Params.ViewPoint);

	// Toto je vzdalenost bodu pohledu (Params.ViewPoint) od Point
	Distance = sqrt( ScalarMultiply(RelPosition) );

	if( InBall(Distance, 0, SmallNum) )
		return( false );

	HelpVal1 = RelPosition.Y / Distance;	
	
	// Vypocet vertikalniho uhlu polohy bodu pohledu (Params.ViewPoint) od Point
	VerAngle = asin( HelpVal1 );
	HelpVal1 = Distance * cos( VerAngle );

	if( InBall(HelpVal1, 0, SmallNum) )
		return( false );

	// Vypocet horizontalniho uhlu polohy bodu pohledu od Point - vyjdou 4 reseni, 
	// z nichz prave dve se rovnaji - jedno z nich je pak to spravne.
	// Vsechny uhly pocitame tak, aby byly z intervalu (-PI,PI>, pro jejich jednodussi porovnani
	HorAngle1 = asin( - RelPosition.X / HelpVal1 );
	HorAngle3 = acos( RelPosition.Z / HelpVal1 );

	if( HorAngle1 < 0 )
		HorAngle3 = -HorAngle3;


	// A provedeme vyzadane otoceni
	VerAngle += VerAngleInc;
	HorAngle3 += HorAngleInc;

	if( VerAngle >= (PI/2) )
		VerAngle = PI/2 - 0.01;
	if( VerAngle <= (-PI/2) )
		VerAngle = -PI/2 + 0.01;

	// Vypocitame novy bod pohledu
	Params.ViewPoint.X = Point.X - Distance * cos(VerAngle) * sin(HorAngle3);
	Params.ViewPoint.Z = Point.Z + Distance * cos(VerAngle) * cos(HorAngle3);
	Params.ViewPoint.Y = Point.Y + Distance * sin(VerAngle);

	// Vypocitame nove uhly pohledu
	Params.HorViewAngle = -(PI - HorAngle3);
	Params.VerViewAngle = VerAngle;

	return( true );
}






// Funkce "prelozi" uzivatelske informace o pohledu na scenu (Params) do vnitrniho formatu pro rychlejsi rendering.
// Vysledek se vraci v parametru Info, hodnotu Params funkce nemeni. Pokud vse uspeje, vraci se true, jinak false.
bool CreateRenderInfo(RenderInfo *pInfo, const ViewParams &Params)
{
	Triangle3D Triangle, TransformedTriangle;
	Matrix4x4 Trans;
	double SinAlfa, SinBeta, CosAlfa, CosBeta, X, Y, Z;
	
	// Kontrola parametru
	if ((Params.ResX <= 1) || (Params.ResY <= 1) || (Params.ViewPlaneDistance <= 0))
		return( false );

	pInfo->ResXDiv2 = (double)Params.ResX / 2;
	pInfo->ResYDiv2 = (double)Params.ResY / 2;
	pInfo->ViewPoint = Params.ViewPoint;
	pInfo->ViewPlaneDistance = Params.ViewPlaneDistance;
	

	// Pohledova (promitaci) rovina v zakladni poloze je rovnobezna s rovinou os X a Y, a je posunuta
	// o ViewPlaneDistance po smeru osy Z. Mi na ni umistime nejake 3 body (ne v primce!), ty transformujeme 
	// podle uzivatelskych nastaveni (hor. a ver. uhel pohledu, posunuti) a z nich ziskame 
	// transformovanou polohu (obecnou rovnici) roviny (zakladni -> normalni poloha)
	Triangle.A.Z = Triangle.B.Z = Triangle.C.Z = Params.ViewPlaneDistance;	
	Triangle.A.X = Triangle.C.X = -pInfo->ResXDiv2;
	Triangle.B.X = pInfo->ResXDiv2;
	Triangle.A.Y = Triangle.B.Y = -pInfo->ResYDiv2;
	Triangle.C.Y = pInfo->ResYDiv2;

	// Vypocitame transformacni matici (Alfa = VerViewAngle, Beta = HorViewAngle)
	SinAlfa = sin( Params.VerViewAngle );
	SinBeta = sin( Params.HorViewAngle );
	CosAlfa = cos( Params.VerViewAngle );
	CosBeta = cos( Params.HorViewAngle );
	Trans[0] = CosBeta;
	Trans[1] = 0;
	Trans[2] = SinBeta;
	Trans[3] = 0;
	Trans[4] = - SinAlfa * SinBeta;
	Trans[5] = CosAlfa;
	Trans[6] = SinAlfa * CosBeta;
	Trans[7] = 0;
	Trans[8] = - CosAlfa * SinBeta;
	Trans[9] = - SinAlfa;
	Trans[10] = CosAlfa * CosBeta;
	Trans[11] = 0;
	Trans[12] = Params.ViewPoint.X;
	Trans[13] = Params.ViewPoint.Y;
	Trans[14] = Params.ViewPoint.Z;
	Trans[15] = 1;

	// Provedeme transformaci trojuhelnika
	TransformedTriangle.A = TransformPoint(&Trans, Triangle.A);
	TransformedTriangle.B = TransformPoint(&Trans, Triangle.B);
	TransformedTriangle.C = TransformPoint(&Trans, Triangle.C);
	
	// A z transformovaneho trojuhelnika ziskame pohledovou rovninu ve spravne (normalni) poloze
	pInfo->ViewPlane = GetPlaneOfTriangle(TransformedTriangle);
	memcpy(pInfo->Transformation, Trans, sizeof(Trans)); 

	// A vypocitame matici pro inverzni transformaci (normalni -> zakladni poloha). Dokonce tato je inverzni
	// k transformacni matici (viz vyse). (Alfa = -VerViewAngle, Beta = -HorViewAngle, X = -ViewPoint.X, 
	// Y = -ViewPoint.Y, Z = -ViewPoint.Z)
	SinAlfa = sin( - Params.VerViewAngle );
	SinBeta = sin( - Params.HorViewAngle );
	CosAlfa = cos( - Params.VerViewAngle );
	CosBeta = cos( - Params.HorViewAngle );
	X = -Params.ViewPoint.X;
	Y = -Params.ViewPoint.Y;
	Z = -Params.ViewPoint.Z;
	Trans[0] = CosBeta;
	Trans[1] = - SinAlfa * SinBeta;
	Trans[2] = CosAlfa * SinBeta;
	Trans[3] = 0;
	Trans[4] = 0;
	Trans[5] = CosAlfa;
	Trans[6] = SinAlfa;
	Trans[7] = 0;
	Trans[8] = - SinBeta;
	Trans[9] = - SinAlfa * CosBeta;
	Trans[10] = CosAlfa * CosBeta;
	Trans[11] = 0;
	Trans[12] = CosBeta*X - SinBeta*Z;
	Trans[13] = CosAlfa*Y - SinAlfa*( SinBeta*X + CosBeta*Z );
	Trans[14] = SinAlfa*Y + CosAlfa*( SinBeta*X + CosBeta*Z );
	Trans[15] = 1;
	
	memcpy(pInfo->InvTransformation, Trans, sizeof(Trans)); 

	return( true );

}


// Funkce vyrenderuje bod Point na zaklade informaci z Info. Vysledek - souradnice pixelu na obrazovce - 
// se uklada do parametru Pixel predavaneho odazem, do parametru TransPoint se ulozi bod Point preneseny
// do zakladni polohy (transformovany matici Info.InvTransformation). Ostatni parametry funkce nemeni.
// Pokud vse usepeje, vraci se true, jinak false.
bool RenderPoint(Point2D *pPixel, Point3D *pTransPoint, const Point3D &Point, RenderInfo &Info)
{
	double Coeff, SolX, SolY;

	// Preneseme renderovany bod do zakladni polohy, bod pohledu definujme [0,0,0] a pohledovou
	// rovinu rovnobeznou s osami X a Y posunutou na Z o Info.ViewPlaneDistance (tj. take v zakladni poloze)
	*pTransPoint = TransformPoint(&(Info.InvTransformation), Point);

	// Vypocteme parametr pro rovnici primky urcujici prunik roviny pohledu (v zakladni poloze) 
	// s primkou z [0,0,0] do TransPoint, o obou lecos vime, staci nam tedy jen jedno deleni.
	if(pTransPoint->Z <= 0)
		return( false ); // Pokud by byl TransPoint.Z zaporny, byl by prunik za pozorovatelem = chyba
	Coeff = Info.ViewPlaneDistance / pTransPoint->Z;
	
	// Z parametru urcime bod pruniku a jeste ho posuneme vzhledem k obrazovce (napr. bod pruniku 
	// [0,0,Info.ViewPlaneDistance] tak umistime doprostred obrazovky, coz je ocekavane chovani)
	SolX = pTransPoint->X*Coeff + Info.ResXDiv2;
	SolY = pTransPoint->Y*Coeff + Info.ResYDiv2;
	
	// Neni bod prilis mimo obrazovku ?
	if ((SolX >= MAXPOINT2D/2) || (SolX <= MINPOINT2D/2) 
		|| (SolY >= MAXPOINT2D/2) || (SolY <= MINPOINT2D/2))
		return( false );

	pPixel->X = (short int)SolX;
	pPixel->Y = (short int)SolY;

	return( true );

}




// Procedura znici pPicture - uvolni jemu alokovanou pamet.
void DestroyRenderedPicture(RenderedPicture *pPicture)
{
	if(pPicture->Pixels)
		free(pPicture->Pixels);
	if(pPicture->PixelInfo)
		free(pPicture->PixelInfo);
	pPicture->Resolution.X = pPicture->Resolution.Y = 0;
	pPicture->Pixels = NULL;
	pPicture->PixelInfo = NULL;
}

// Funkce vytvori novy nebo prenastavi existujici RenderPicture, jeho vlastnosti budou nastaveny podle Params.
// Pokud vse uspeje, vraci se true, jinak false. Vysledek se predava parametrem pPicture.
bool CreateRenderedPictureEx(RenderedPicture *pPicture, const ViewParams &Params, bool AllocPixelInfo)
{	
	// Pokud novy obraz odpovida existujicimu staremu, ponechame puvodni bezezmen - urychleni
	if(pPicture->Pixels && pPicture->PixelInfo && (pPicture->Resolution.X==Params.ResX) && (pPicture->Resolution.Y==Params.ResY) )
		return( true );	
	
	DestroyRenderedPicture( pPicture );

	if( (Params.ResX<=0) || (Params.ResY<=0) )
		return( false );

	pPicture->Pixels = (pColor32)malloc(sizeof(Color32) * Params.ResX * Params.ResY);
	if( AllocPixelInfo )
		pPicture->PixelInfo = (pPixelInfoElem)malloc(sizeof(PixelInfoElem) * Params.ResX * Params.ResY);
	
	if( !pPicture->Pixels || !pPicture->PixelInfo )
	{
		DestroyRenderedPicture( pPicture );
		return( false );
	}

	pPicture->Resolution.X = Params.ResX;
	pPicture->Resolution.Y = Params.ResY;
	return( true );
}

// Vytvori obrazek pPicture s rozlisenim Resolution, pPicture->PixelInfo se nealokuje!!!
bool CreateRenderedPicture(RenderedPicture *pPicture, Point2D Resolution)
{	
	if( (Resolution.X<=0) || (Resolution.Y<=0) )
		return( false );

	// Pokud novy obraz odpovida existujicimu staremu, ponechame puvodni bezezmen - urychleni
	if(pPicture->Pixels && (pPicture->Resolution.X==Resolution.X) && (pPicture->Resolution.Y==Resolution.Y) )
	{
		if( pPicture->PixelInfo )
			free( pPicture->PixelInfo );
		pPicture->PixelInfo = NULL;
		return( true );	
	}
	
	DestroyRenderedPicture( pPicture );

	pPicture->Pixels = (pColor32)malloc(sizeof(Color32) * Resolution.X * Resolution.Y);
	if( !pPicture->Pixels )
		return( false );
	
	pPicture->Resolution = Resolution;	
	return( true );
}








/* Format RSF souboru
 
   Co:             Velikost:
   
   RSFHeader.......sizeof(RSFHeader)
   Triangles.......RSFHeader.NumOfTriangles * sizeof(RSFTriangle)
   Points..........RSFHeader.NumOfPoints * sizeof(RSFPoint)
   MasterTexture...sizeof(Color32) * RSFHeader.MasterTextrueRes.X * RSFHeader.MasterTextrueRes.Y;
   SlaveTexture....sizeof(Color32) * RSFHeader.SlaveTextrueRes.X * RSFHeader.SlaveTextrueRes.Y;

*/


// Funkce nahraje ze souboru Filename (typicky nejaky *.RSF) novou scenu, tu ulozi do Scene. Pokud vse
// uspeje, vraci se true, jinak false. Hodnota Filename se nezmeni.
bool OpenRenderScene(const char *Filename, RenderScene *pScene)
{
	RenderScene &Scene = *pScene;

	FILE *File;
	RSFHeader Header;
	size_t PointsRead, TrianglesRead, LinesRead, MasterPixelsRead, SlavePixelsRead; // Pocty skutecne nactenych elementu
	
	// Otevreme soubor se scenou
	File = fopen(Filename, "rb");

	if( !File )
		return( false );

	// Nacteme hlavicku
	if( fread(&Header, sizeof(RSFHeader), 1, File) != 1 )
	{
		fclose(File);
		return( false );
	}

	// Kontrola formatu
	if(( strncmp(Header.InfoString, "RSF", 3) != 0 ) 
		|| ( Header.MasterTextureRes.X < 0 )
		|| ( Header.MasterTextureRes.Y < 0 )
		|| ( Header.SlaveTextureRes.X < 0 )
		|| ( Header.SlaveTextureRes.Y < 0 )
		|| ( Header.NumOfPoints > 1000000 )
		|| ( Header.NumOfTriangles > 1000000 )
		|| ( Header.NumOfLines > 10000000 ) )
	{		
		fclose(File);
		return( false );
	}

	CloseRenderScene(&Scene);	
	
	// Ulozeni udaju
	Scene.MasterTextureRes = Header.MasterTextureRes;
	Scene.SlaveTextureRes = Header.SlaveTextureRes;
	Scene.NumOfTriangles = Header.NumOfTriangles;
	Scene.NumOfPoints = Header.NumOfPoints;
	Scene.NumOfLines = Header.NumOfLines;
	Scene.MasterCameraPosition = Header.MasterCameraPosition;
	Scene.SlaveCameraPosition = Header.SlaveCameraPosition;
	
	// Alokujeme pamet pro objekty, pokud alokujeme pamet o nulove velikosti, pri spravnem prubehu
	// by mel malloc() vratit take nenulovy pointer
	Scene.Points = (pRSFPoint)malloc(sizeof(RSFPoint) * Header.NumOfPoints);
	Scene.Triangles = (pRSFTriangle)malloc(sizeof(RSFTriangle) * Header.NumOfTriangles);
	Scene.Lines = (pRSFLine)malloc(sizeof(RSFTriangle) * Header.NumOfLines );
	Scene.MasterTexture = (pColor32)malloc(sizeof(Color32) *  Header.MasterTextureRes.X * Header.MasterTextureRes.Y);
	Scene.SlaveTexture = (pColor32)malloc(sizeof(Color32) *  Header.SlaveTextureRes.X * Header.SlaveTextureRes.Y);

	if( !Scene.Points || !Scene.Triangles || !Scene.Lines || !Scene.MasterTexture || !Scene.SlaveTexture )
	{	
		fclose(File);
		CloseRenderScene(&Scene);
		return( false );
	}

	// Pokud pointer ukazuje na pole o nulove velikosti, uvolnime ho a nastavime na NULL
	if(Header.NumOfPoints==0)
		{ free(Scene.Points); Scene.Points=NULL; }
	if(Header.NumOfTriangles==0)
		{ free(Scene.Triangles); Scene.Triangles=NULL; }
	if(Header.NumOfLines==0)
		{ free(Scene.Lines); Scene.Lines=NULL; }
	if( (Header.MasterTextureRes.X==0) || (Header.MasterTextureRes.Y==0)  )
		{ free(Scene.MasterTexture); Scene.MasterTexture=NULL; }
	if( (Header.SlaveTextureRes.X==0) || (Header.SlaveTextureRes.Y==0)  )
		{ free(Scene.SlaveTexture); Scene.SlaveTexture=NULL; }

	// Nacteme data
	TrianglesRead = fread(Scene.Triangles, sizeof(RSFTriangle), Header.NumOfTriangles, File);
	PointsRead = fread(Scene.Points, sizeof(RSFPoint), Header.NumOfPoints, File);
	LinesRead = fread(Scene.Lines, sizeof(RSFLine), Header.NumOfLines, File);
	MasterPixelsRead = fread(Scene.MasterTexture, sizeof(Color32), Header.MasterTextureRes.X * Header.MasterTextureRes.Y, File);
	SlavePixelsRead = fread(Scene.SlaveTexture, sizeof(Color32), Header.SlaveTextureRes.X * Header.SlaveTextureRes.Y, File);

	fclose(File);

	// Kontrola dobreho prubehu
	if( (TrianglesRead != Header.NumOfTriangles)
		|| (PointsRead != Header.NumOfPoints)
		|| (TrianglesRead != Header.NumOfTriangles)
		|| (LinesRead != Header.NumOfLines)
		|| (MasterPixelsRead != (size_t)(Header.MasterTextureRes.X * Header.MasterTextureRes.Y))
		|| (SlavePixelsRead != (size_t)(Header.SlaveTextureRes.X * Header.SlaveTextureRes.Y)) )
	{	
		CloseRenderScene(&Scene);
		return( false );
	}

	return( true );

}


// Funkce ulozi render-scenu Scene do souboru Filename, nemeni hodnotu ani jednoho z parametru. Pokud
// vse uspeje (soubor byl vytvoren), vraci se true, jinak false.
bool SaveRenderScene(const char *Filename, const RenderScene &Scene)
{
	RSFHeader Header;
	FILE *File;
	size_t HeadersWritten, PointsWritten, TrianglesWritten, LinesWritten, 
	       MasterPixelsWritten, SlavePixelsWritten; // Pocty skutecne zapsanych elementu
	
	File = fopen(Filename, "wb");

	if( !File )
		return( false );

	// Ulozeni udaju do hlavicky
	strncpy(Header.InfoString, "RSF", 3);
	Header.NumOfPoints = Scene.NumOfPoints;
	Header.NumOfTriangles = Scene.NumOfTriangles;
	Header.NumOfLines = Scene.NumOfLines;
	Header.MasterTextureRes = Scene.MasterTextureRes;
	Header.SlaveTextureRes = Scene.SlaveTextureRes;
	Header.MasterCameraPosition = Scene.MasterCameraPosition;
	Header.SlaveCameraPosition = Scene.SlaveCameraPosition;

	// A zapis...
	HeadersWritten = fwrite(&Header, sizeof(RSFHeader), 1, File);
	TrianglesWritten = fwrite(Scene.Triangles, sizeof(RSFTriangle), Scene.NumOfTriangles, File);
	PointsWritten = fwrite(Scene.Points, sizeof(RSFPoint), Scene.NumOfPoints, File);
	LinesWritten = fwrite(Scene.Lines, sizeof(RSFLine), Scene.NumOfLines, File);
	MasterPixelsWritten = fwrite(Scene.MasterTexture, sizeof(Color32), Scene.MasterTextureRes.X * Scene.MasterTextureRes.Y, File);
	SlavePixelsWritten = fwrite(Scene.SlaveTexture, sizeof(Color32), Scene.SlaveTextureRes.X * Scene.SlaveTextureRes.Y, File);

	fclose(File);

	// Kontrola prubehu
	if( (HeadersWritten != 1)
	    || (TrianglesWritten != Scene.NumOfTriangles)
	    || (PointsWritten != Scene.NumOfPoints)
		|| (LinesWritten != Scene.NumOfLines)
	    || (MasterPixelsWritten != (size_t)(Scene.MasterTextureRes.X * Scene.MasterTextureRes.Y))
	    || (SlavePixelsWritten != (size_t)(Scene.SlaveTextureRes.X * Scene.SlaveTextureRes.Y)) )
		return( false );

	return( true );
}




// Procedura zavre Scene (dealokuje pamet), mela by se volat pokud jiz neni objekt potreba
void CloseRenderScene(RenderScene *pScene)
{
	if( pScene->MasterTexture )
		free(pScene->MasterTexture);
	if( pScene->SlaveTexture )
		free(pScene->SlaveTexture);
	if( pScene->Triangles )
		free(pScene->Triangles);
	if( pScene->Points )
		free(pScene->Points);

	memset( pScene, 0, sizeof(RenderScene) );
}


// Funkce vykresli do obrazku Picture obdelnik barvy Color, ktery ma levy horni roh UpperLeft a pravy spodni
// BottomRight. Pokud vse uspeje, vraci se true, jinak false.
bool DrawRectToPicture(RenderedPicture *pPicture, Point2D UpperLeft, Point2D BottomRight, Color32 Color)
{
	signed short MaxX, MaxY, LineLength;
	long Index;
	pColor32 Ptr, MaxPtr;
		
	if(!pPicture->Pixels)
		return( false );
	
	MaxX = pPicture->Resolution.X - 1;
	MaxY = pPicture->Resolution.Y - 1;

	if(UpperLeft.Y < 0)			return( false );
	if(BottomRight.Y > MaxY)	return( false );
	if(UpperLeft.X > MaxX)		return( false );
	if(BottomRight.X < 0)		return( false );

	if(UpperLeft.Y > MaxY)		UpperLeft.Y = MaxY;
	if(BottomRight.Y < 0)		BottomRight.Y = 0;	
	if(UpperLeft.X < 0)			UpperLeft.X = 0;
	if(BottomRight.X > MaxX)	BottomRight.X = MaxX;

	if( (UpperLeft.X > BottomRight.X) || (UpperLeft.Y < BottomRight.Y) ) 
		return(false);

	LineLength = BottomRight.X - UpperLeft.X + 1;
	Color &= 0x00ffffff;

	for(Index=BottomRight.Y*pPicture->Resolution.X+UpperLeft.X;
	    Index <= (UpperLeft.Y+1)*pPicture->Resolution.X - 1;
	    Index += pPicture->Resolution.X)	
	{
		Ptr = pPicture->Pixels + Index;
		MaxPtr = pPicture->Pixels + Index + LineLength - 1;
		for(; Ptr<=MaxPtr; Ptr++)
			*Ptr = Color;
	}
		
	return( true );
}


void DrawPictureToPicture(RenderedPicture *pPicture, Point2D Position, const RenderedPicture &Source)
{
	pColor32 WritePtr, ReadPtr, MaxWritePtr;

	WritePtr = pPicture->Pixels + Position.Y*pPicture->Resolution.X + Position.X;
	MaxWritePtr = WritePtr + (Source.Resolution.Y-1)*pPicture->Resolution.X;
	ReadPtr = Source.Pixels;

	for(; WritePtr<=MaxWritePtr; WritePtr += pPicture->Resolution.X)
	{
		memcpy( WritePtr, ReadPtr, Source.Resolution.X * sizeof(Color32) );
		ReadPtr += Source.Resolution.X;
	}			
}


void DrawPictureToPicture(RenderedPicture *pPicture, Point2D Position, const RenderedPicture &Source, float Zoom)
{
	int x, y, max_x, max_y, src_y_ex;
	float src_res_x_ex, src_x, src_y, i_zoom;
	Color32 *Ptr, *StartPtr;

	i_zoom = 1 / Zoom;
	src_res_x_ex = Source.Resolution.X / Zoom;
	max_x = (Source.Resolution.X-1) * Zoom ;
	max_y = (Source.Resolution.Y-1) * Zoom ;
	StartPtr = pPicture->Pixels + Position.Y*pPicture->Resolution.X + Position.X;

	src_y_ex = 0;
	src_y = 0.0;

	for( y=0; y<=max_y; y++ )
	{
		src_x = 0.0;
		Ptr = StartPtr;

		for( x=0; x<=max_x; x++ )		
		{			
			*Ptr = Source.Pixels[ src_y_ex + (int)src_x ];
			Ptr++;
			src_x += i_zoom;
		}

		StartPtr += pPicture->Resolution.X;
		//src_y_ex += src_res_x_ex;

		src_y += i_zoom;
		src_y_ex = (int)src_y * Source.Resolution.X;

	}
}





void DrawLineToPicture(RenderedPicture *pPicture, const Line2D &Line, Color32 Color)
{
	LineCalculationState LState;
	LineCalculationProcedure LProc;
	Point2D Pixel;

	LProc = PrepareLineCalculation( &LState, Line, pPicture->Resolution );
	while( LineCalculationContinues(LState) )
	{
		Pixel = LProc( &LState ); // zjistime dalsi pixel usecky						
		pPicture->Pixels[ Pixel.Y*pPicture->Resolution.X + Pixel.X ] = Color;
	}
}


bool ApplyZoomToPicture( RenderedPicture *pPicture, float Zoom )
{
	RenderedPicture NewPict;

	if( !pPicture->Pixels || !pPicture->Resolution.X || !pPicture->Resolution.Y )
		return( false );
	
	if( Zoom == 1.00 )
		return( true );
	if( Zoom <= 0 )
		return( false );

	NewPict.Resolution.X = pPicture->Resolution.X * Zoom ;
	NewPict.Resolution.Y = pPicture->Resolution.Y * Zoom ;

	NewPict.Pixels = (Color32 *)malloc( NewPict.Resolution.X * NewPict.Resolution.Y * sizeof(Color32) );
	if( !NewPict.Pixels )
	{
		DestroyRenderedPicture( &NewPict );
		return( false );
	}

	int src_x, src_y;
	float i_zoom = 1 / Zoom;
	Color32 *Ptr = NewPict.Pixels;

	for( int y=0; y<NewPict.Resolution.Y; y++ )
		for( int x=0; x<NewPict.Resolution.X; x++ )		
		{
			src_x = x * i_zoom;
			src_y = y * i_zoom;

			*Ptr = pPicture->Pixels[ src_y*pPicture->Resolution.X + src_x ];
			Ptr++;
		}

	DestroyRenderedPicture( pPicture );
	*pPicture = NewPict;

	return( true );
}


// Funkce GetHistogram(...) ulozi do pHistogram histogram obrazku Picture.
// pHistogram je pointer na pole o 256 prvcich typu unsigned int
// Mode urcuje zpusob vytvareni histogramu, muze to byt jedna
// z nasledujicich hodnot:
//   HISTOGRAM_MODE_R....histogram podle cervene barevne slozky
//   HISTOGRAM_MODE_G....histogram podle zelene barevne slozky
//   HISTOGRAM_MODE_B....histogram podle modre barevne slozky
//   HISTOGRAM_MODE_BRIGHTNESS....histogram podle celkoveho jasu pixelu
//
// Navic muzete pridat nasledujici atribut operatorem OR (|)
//   HISTOGRAM_MODE_ADD....histogram se nebude incializovat, tj. pricte se k puvodnimu

void GetHistogram(const RenderedPicture &Picture, unsigned int *pHistogram, unsigned int Mode)
{
	unsigned char *BytePtr, *MaxBytePtr;
	Color32 *ColorPtr, *MaxColorPtr;

	// Inicializace histogramu, pokud nebyl pouzit atribut HISTOGRAM_MODE_ADD
	if( ! (Mode & HISTOGRAM_MODE_ADD) )
		memset(pHistogram, 0, COLOR32_PRECISION*sizeof(unsigned int));

	
	if( Mode <= 2 )
	{		
		// Mode je bud HISTOGRAM_MODE_R, HISTOGRAM_MODE_G nebo HISTOGRAM_MODE_B,
		// posuneme tedy BytePtr na byte odpovidajici barevne slozce R,G nebo B
		BytePtr = (unsigned char *)Picture.Pixels + Mode;
		MaxBytePtr = BytePtr + (Picture.Resolution.X*Picture.Resolution.Y)*sizeof(Color32) - 1;

		for( ; BytePtr <= MaxBytePtr; BytePtr += 4 ) 
			pHistogram[ *BytePtr ] ++;
	}
	else
	{
		// Mode je HISTOGRAM_MODE_BRIGHTNESS, udelame histogram tedy podle celkoveho jasu pixelu
		ColorPtr = Picture.Pixels;
		MaxColorPtr = ColorPtr + Picture.Resolution.X*Picture.Resolution.Y - 1;

		for( ; ColorPtr <= MaxColorPtr; ColorPtr++ ) 
			pHistogram[ GetBrightness(*ColorPtr) ] ++;		
	}

}

// Funkce provede ekvalizaci histogramu Histogram, vysledek - vyhledavaci tabulku -
// ulozit do LUT.
void EqualizeHistogram(const unsigned int *Histogram, unsigned int NumOfPixels, unsigned char *LUT)
{
	int D, outI, inI, firstI, outHeight, inHeight;
	
	outI = 0;
	outHeight = 0;
	D = NumOfPixels / COLOR32_PRECISION; // Optimalni pocet pixelu se stejny jasem
	for( inI=0; inI<COLOR32_PRECISION; inI++ )
	{
		firstI = outI;
		inHeight = (int)Histogram[inI];
		while( inHeight > D-outHeight )
		{
			inHeight -= D-outHeight;
			outI++;
			outHeight = 0;
		}
		outHeight += inHeight;
		LUT[ inI ] = (firstI + outI) / 2;
	}		
}


// Funkce aplikuje vyhledavaci tabulku LUT na obrazek pPicture, hodnota Mode
// urcuje zpusob aplikace, muze mit jednu z nasledujicich hodnot:
//   HISTOGRAM_MODE_R....aplikace na cervenou barevnou slozku
//   HISTOGRAM_MODE_G....aplikace na zelenou barevnou slozku
//   HISTOGRAM_MODE_B....aplikace na modrou barevnou slozku
//   HISTOGRAM_MODE_BRIGHTNESS....aplikace na celkovy jas pixelu
void ApplyLUTToPicture(RenderedPicture *pPicture, const unsigned char *LUT, unsigned int Mode)
{	
	Color32 *ColorPtr, *MaxColorPtr;
	unsigned char *BytePtr, *MaxBytePtr;
	unsigned int Brightness, Coeff;
		
	// Rekonstrukce obrazku pomoci LUT
	if( Mode <= 2 )
	{
		// Posuneme BytePtr na byte odpovidajici urcite barevne slozce, ta je urcena
		// parametrem Mode
		BytePtr = (unsigned char *)pPicture->Pixels + Mode;
		MaxBytePtr = BytePtr + (pPicture->Resolution.X*pPicture->Resolution.Y)*sizeof(unsigned long) - 1;

		for( ; BytePtr <= MaxBytePtr; BytePtr += 4 ) 
			*BytePtr = LUT[ *BytePtr ];					
	}
	else
	{
		// Ekvalizace jasu
		ColorPtr = pPicture->Pixels;
		MaxColorPtr = ColorPtr + pPicture->Resolution.X*pPicture->Resolution.Y - 1;

		BytePtr = (unsigned char *)ColorPtr; // Ukazuje na cervenou slozku prvniho pixelu

		for( ; ColorPtr <= MaxColorPtr; ColorPtr ++ ) 
		{
			Brightness = GetBrightness(*ColorPtr);
			if( Brightness != 0 )
				Coeff = ((unsigned int)LUT[ Brightness ] << 16) / Brightness ;
			else
				Coeff = 65536;
			
			BytePtr[0] = ((unsigned int)BytePtr[0]*Coeff) >> 16;
			BytePtr[1] = ((unsigned int)BytePtr[1]*Coeff) >> 16;
			BytePtr[2] = ((unsigned int)BytePtr[2]*Coeff) >> 16;	
			BytePtr += 4;
		}			
	}
}

// Funkce ekvalizuje obrazek pPicture, parameter Mode urcuje zpusob, muze to byt 
// jedna z nasledujicih hodnot:
//   HISTOGRAM_MODE_BRIGHTNESS ....... Ekvalizace podle jasu pixelu
//   HISTOGRAM_MODE_COLORS ........... Ekvalizace podle barevnych slozek RGB
void EqualizePicture(RenderedPicture *pPicture, unsigned int Mode)
{
	unsigned int Histogram[ COLOR32_PRECISION ];
	unsigned char LUT[ COLOR32_PRECISION ];
	unsigned int NumOfPixels = pPicture->Resolution.X * pPicture->Resolution.Y;

	if( Mode == HISTOGRAM_MODE_BRIGHTNESS )
	{
		GetHistogram( *pPicture, Histogram, HISTOGRAM_MODE_BRIGHTNESS );
		EqualizeHistogram( Histogram, NumOfPixels, LUT );
		ApplyLUTToPicture( pPicture, LUT, HISTOGRAM_MODE_BRIGHTNESS );
	}
	else
	{
		GetHistogram( *pPicture, Histogram, HISTOGRAM_MODE_R );
		EqualizeHistogram( Histogram, NumOfPixels, LUT );
		ApplyLUTToPicture( pPicture, LUT, HISTOGRAM_MODE_R );

		GetHistogram( *pPicture, Histogram, HISTOGRAM_MODE_G );
		EqualizeHistogram( Histogram, NumOfPixels, LUT );
		ApplyLUTToPicture( pPicture, LUT, HISTOGRAM_MODE_G );

		GetHistogram( *pPicture, Histogram, HISTOGRAM_MODE_B );
		EqualizeHistogram( Histogram, NumOfPixels, LUT );
		ApplyLUTToPicture( pPicture, LUT, HISTOGRAM_MODE_B );
	}
}


// Funkce spolecne ekvalizuje obrazky pPictureA a pPictureB, parameter Mode urcuje
// zpusob, muze to byt jedna z nasledujicih hodnot:
//   HISTOGRAM_MODE_BRIGHTNESS ....... Ekvalizace podle jasu pixelu
//   HISTOGRAM_MODE_COLORS ........... Ekvalizace podle barevnych slozek RGB
void EqualizeTwoPictures(RenderedPicture *pPictureA, RenderedPicture *pPictureB, unsigned int Mode)
{
	unsigned int Histogram[ COLOR32_PRECISION ];
	unsigned char LUT[ COLOR32_PRECISION ];
	unsigned int NumOfPixels = pPictureA->Resolution.X * pPictureA->Resolution.Y
	                           + pPictureB->Resolution.X * pPictureB->Resolution.Y;

	if( Mode == HISTOGRAM_MODE_BRIGHTNESS )
	{
		GetHistogram( *pPictureA, Histogram, HISTOGRAM_MODE_BRIGHTNESS );
		GetHistogram( *pPictureB, Histogram, HISTOGRAM_MODE_BRIGHTNESS | HISTOGRAM_MODE_ADD );
		EqualizeHistogram( Histogram, NumOfPixels, LUT );
		ApplyLUTToPicture( pPictureA, LUT, HISTOGRAM_MODE_BRIGHTNESS );
		ApplyLUTToPicture( pPictureB, LUT, HISTOGRAM_MODE_BRIGHTNESS );
	}
	else
	{
		GetHistogram( *pPictureA, Histogram, HISTOGRAM_MODE_R );
		GetHistogram( *pPictureB, Histogram, HISTOGRAM_MODE_R | HISTOGRAM_MODE_ADD );
		EqualizeHistogram( Histogram, NumOfPixels, LUT );
		ApplyLUTToPicture( pPictureA, LUT, HISTOGRAM_MODE_R );
		ApplyLUTToPicture( pPictureB, LUT, HISTOGRAM_MODE_R );

		GetHistogram( *pPictureA, Histogram, HISTOGRAM_MODE_G );
		GetHistogram( *pPictureB, Histogram, HISTOGRAM_MODE_G | HISTOGRAM_MODE_ADD );
		EqualizeHistogram( Histogram, NumOfPixels, LUT );
		ApplyLUTToPicture( pPictureA, LUT, HISTOGRAM_MODE_G );
		ApplyLUTToPicture( pPictureB, LUT, HISTOGRAM_MODE_G );

		GetHistogram( *pPictureA, Histogram, HISTOGRAM_MODE_B );
		GetHistogram( *pPictureB, Histogram, HISTOGRAM_MODE_B | HISTOGRAM_MODE_ADD );
		EqualizeHistogram( Histogram, NumOfPixels, LUT );
		ApplyLUTToPicture( pPictureA, LUT, HISTOGRAM_MODE_B );
		ApplyLUTToPicture( pPictureB, LUT, HISTOGRAM_MODE_B );
	}
}



void ApplyBarlettFilter(RenderedPicture *pPicture, unsigned int Size)
{	
	BarlettEyeProcedure *BarlettEye;	
	Point2D pt;
	int x,y;

	// Jake oko pouzijeme ?
	switch( Size )
	{
		case 3: BarlettEye = (BarlettEyeProcedure *)BarlettEye3x3; break;
		case 5:	BarlettEye = (BarlettEyeProcedure *)BarlettEye5x5; break;
		case 7:	BarlettEye = (BarlettEyeProcedure *)BarlettEye7x7; break;
		default: BarlettEye = (BarlettEyeProcedure *)BarlettEye1x1; break;
	}
	
	for(y=0; y<pPicture->Resolution.Y; y++)
		for(x=0; x<pPicture->Resolution.X; x++)
		{
			pt.X = x;
			pt.Y = y;
			pPicture->Pixels[y*pPicture->Resolution.X + x] = BarlettEye(pPicture->Pixels, pt, pPicture->Resolution);
		}

}





// Funkce nastavi v Picture.PixelInfo obdelnik elementu typu PIXELINFO_POINT s indexem bodu PointIndex, 
// ten ma levy horni roh UpperLeft a pravy spodni BottomRight. Pokud vse uspeje, vraci se true, jinak false.
bool SetPointRectToPixelInfo(RenderedPicture &Picture, Point2D UpperLeft, Point2D BottomRight, unsigned int PointIndex)
{
	signed short MaxX, MaxY, LineLength;
	long Index;
	pPixelInfoElem Ptr, MaxPtr;
	PixelInfoElem Elem;
		
	if(!Picture.PixelInfo)
		return( false );
	
	MaxX = Picture.Resolution.X - 1;
	MaxY = Picture.Resolution.Y - 1;

	if(UpperLeft.Y < 0)			return( false );
	if(BottomRight.Y > MaxY)	return( false );
	if(UpperLeft.X > MaxX)		return( false );
	if(BottomRight.X < 0)		return( false );

	if(UpperLeft.Y > MaxY)		UpperLeft.Y = MaxY;
	if(BottomRight.Y < 0)		BottomRight.Y = 0;	
	if(UpperLeft.X < 0)			UpperLeft.X = 0;
	if(BottomRight.X > MaxX)	BottomRight.X = MaxX;

	if( (UpperLeft.X > BottomRight.X) || (UpperLeft.Y < BottomRight.Y) ) 
		return(false);

	LineLength = BottomRight.X - UpperLeft.X + 1;
	Elem.Index = PointIndex;
	Elem.Type = PIXELINFO_POINT;
	Elem.Reserved1 = 0;
	Elem.Reserved2 = 0;

	for(Index=BottomRight.Y*Picture.Resolution.X+UpperLeft.X;
	    Index <= (UpperLeft.Y+1)*Picture.Resolution.X - 1;
	    Index += Picture.Resolution.X)	
	{
		Ptr = Picture.PixelInfo + Index;
		MaxPtr = Picture.PixelInfo + Index + LineLength - 1;
		for(; Ptr<=MaxPtr; Ptr++)
			*Ptr = Elem;
	}
		
	return( true );
}




// Funkce obarvi jiz vyrenderovanou scenu Scene, tj. obarvi body a otexturuje trojuhelniky, ktere se vyskytnou
// ve vyslednem obraze. Picture.PixelInfo obsahuje informace o kazdem pixelu vysledneho obrazu ziskane
// renderingem. Vysledek se zapise do Picture.Pixels. Pokud vse uspeje, vraci se true, jinak false.
bool DyeRenderedScene(const RenderScene &Scene, RenderedPicture &Picture, Color32 BgColor)
{
	pPixelInfoElem PixelInfoPtr;
	pColor32 ColorPtr;
	signed short IndexX, IndexY; 	

	// Promenne pro otexturovani trojuhelniku
	int AlfaNumerator, BetaNumerator, Denominator;
	float DenominatorInv; 
	Vector2D VectorAB, VectorAC, VectorAToPoint;
	Vector2D TexVectorAB, TexVectorAC;
	Point2D TexturePoint;
	unsigned int TextureIndex, MaxMasterTextureIndex, MaxSlaveTextureIndex, MaxTextureIndex;
	unsigned int LastTriangleIndex = 0xFFFFFFFF; // Trojuhelnik s takovymto indexem se nevyskytne
	pTriangle2D RenderedTrianPtr;
	pTriangle2D TextureTrianPtr;
	pColor32 TexturePixels;
	Point2D TextureResolution;

	if( !Picture.Pixels || !Picture.PixelInfo )
		return( false );

	// Pomocnne - urychlujici promenne
	PixelInfoPtr = Picture.PixelInfo;
	ColorPtr = Picture.Pixels;
	MaxMasterTextureIndex = Scene.MasterTextureRes.X * Scene.MasterTextureRes.Y - 1;
	MaxSlaveTextureIndex = Scene.SlaveTextureRes.X * Scene.SlaveTextureRes.Y - 1;

	// Pro vsechny pixely...
	for(IndexY=0; IndexY<Picture.Resolution.Y; IndexY++)
	{
		for(IndexX=0; IndexX<Picture.Resolution.X; IndexX++)
		{
			// Rozdeleni podle typu objektu, ktery lezi na danem pixelu
			switch(PixelInfoPtr->Type)			
			{				
				case PIXELINFO_TRIANGLE:			
					// Na tomhle pixelu je trojuhelnik, tak zjistime prislusnou barvu					
					if( LastTriangleIndex != PixelInfoPtr->Index )
					{
						// Pokud tento trojuhelnik nema texturu, vykreslime ho jako pozadi
						// Tento test jsme umisitili sem, aby se vypocet zbytecne nezdrzoval
						// v beznem pripade
						if( Scene.Triangles[ PixelInfoPtr->Index ].TextureSource == TEXTURE_NONE )
						{
							*ColorPtr = BgColor;
							break;
						}

						// Tento trojuhelnik jsme minule netexturovali, nebo jsme na nove radce,
						// takze predpocitame promenne spolecne pro tento radek.
						RenderedTrianPtr = &( Scene.Triangles[PixelInfoPtr->Index].RenderedPosition );
						TextureTrianPtr = &( Scene.Triangles[PixelInfoPtr->Index].TexturePosition ); 

						//    Metoda texturovani: 
						// Bod trojuhelnika (uvnitr RSFTriangle::RenderedPosition), pro 
						// ktery chceme zjistit barvu, si vyjadrime jako linearni kombinaci vektoru hran 
						// A->B a A->C, a takto ziskane koeficienty aplikujeme na stejne 
						// vektory (A'->B' a A'->C') trojuhelniku s texturou (RSFTriangle::TextureSource)						

						// Vektory odpovidajici RSFTriangle::RenderedPosition
						VectorAB.X = RenderedTrianPtr->B.X - RenderedTrianPtr->A.X;
						VectorAB.Y = RenderedTrianPtr->B.Y - RenderedTrianPtr->A.Y;
						VectorAC.X = RenderedTrianPtr->C.X - RenderedTrianPtr->A.X;
						VectorAC.Y = RenderedTrianPtr->C.Y - RenderedTrianPtr->A.Y;
						VectorAToPoint.Y = IndexY - RenderedTrianPtr->A.Y;
						VectorAToPoint.X = IndexX - RenderedTrianPtr->A.X;
											
						// Jmenovatel zlomku
						Denominator = (int)VectorAB.X*(int)VectorAC.Y - (int)VectorAB.Y*(int)VectorAC.X;
					
						// Vektory odpovidajici RSFTriangle::TextureSource
						TexVectorAB.X = TextureTrianPtr->B.X - TextureTrianPtr->A.X;
						TexVectorAB.Y = TextureTrianPtr->B.Y - TextureTrianPtr->A.Y;
						TexVectorAC.X = TextureTrianPtr->C.X - TextureTrianPtr->A.X;
						TexVectorAC.Y = TextureTrianPtr->C.Y - TextureTrianPtr->A.Y;

						 // Nejsou body trojuhelniku v primce ?
						if( Denominator == 0 )
						{								
							LastTriangleIndex = 0xFFFFFFFF;
							break;
						}					
					
						// Citatele zlomku
						AlfaNumerator = (int)VectorAToPoint.X*(int)VectorAC.Y - (int)VectorAToPoint.Y*(int)VectorAC.X; 
						BetaNumerator = (int)VectorAToPoint.Y*(int)VectorAB.X - (int)VectorAToPoint.X*(int)VectorAB.Y;

						// Nasobeni floatem je rychlejsi nez deleni intem
						DenominatorInv = 1.0 / (float)Denominator;

						if( Scene.Triangles[ PixelInfoPtr->Index ].TextureSource == TEXTURE_MASTERCAM )
						{
							// Zdrojem textury je master kamera
							TexturePixels = Scene.MasterTexture;
							MaxTextureIndex = MaxMasterTextureIndex;
							TextureResolution = Scene.MasterTextureRes;
						}
						else
						{
							// Zdrojem textury je slave kamera
							TexturePixels = Scene.SlaveTexture;
							MaxTextureIndex = MaxSlaveTextureIndex;
							TextureResolution = Scene.SlaveTextureRes;
						}
					
						// Pokud budeme priste opet texturovat tento trojuhelnik a budem na teto radce,
						// cely vnitrek if se vynecha = urychleni.
						LastTriangleIndex = PixelInfoPtr->Index;									
					}
									
					// Vypocet souradnic pixelu textury, ze ktereho zjistime barvu	
					TexturePoint.X = TextureTrianPtr->A.X + (AlfaNumerator*(int)TexVectorAB.X + BetaNumerator*(int)TexVectorAC.X)
			                                                   * DenominatorInv;
					TexturePoint.Y = TextureTrianPtr->A.Y + (AlfaNumerator*(int)TexVectorAB.Y + BetaNumerator*(int)TexVectorAC.Y)
		                                                     * DenominatorInv;																

					// Vypocet indexu do pole s pixely textury (jiz zname jeho souradnice)
					//TextureIndex = (TextureResolution.Y - TexturePoint.Y - 1) * TextureResolution.X 
					TextureIndex = TexturePoint.Y*TextureResolution.X + TexturePoint.X;

					// A obarveni pixelu vysledneho obrazku
					if( TextureIndex <= MaxTextureIndex )
						*ColorPtr = TexturePixels[ TextureIndex ];
					break;					

				case PIXELINFO_POINT:
					// Na tomhle pixelu je bod, tak ho jednoduse obarvime
					*ColorPtr = Scene.Points[ PixelInfoPtr->Index ].Color;					
					break;

				case PIXELINFO_TRIANGLEBORDER:
					*ColorPtr = 0x00ffffff;
					break;

				case PIXELINFO_LINE:
					*ColorPtr = Scene.Lines[ PixelInfoPtr->Index ].Color;
					break;

				default:
					// Zadny objekt na tomto pixelu - vybarvime jako pozadi
					*ColorPtr = BgColor;
					break;

			} // switch(...)			
		
			// Posun na dalsi pixel vysledneho obrazku
			ColorPtr++;
			PixelInfoPtr++;

			// Posuneme i urychlujici promenne pro posledni otexturovany trojuhelnik  ( Scene.Triangles[LastTriangleIndex] )
			AlfaNumerator += (int)VectorAC.Y;
			BetaNumerator -= (int)VectorAB.Y;
			
		} // for(IndexX...)

		// Pri prechodu na novy radek se vetsina promennych pro otexturovani trojuhelniku zmeni
		LastTriangleIndex = 0xFFFFFFFF;

	} // for(IndexY...)

	return( true ); // Uspech...
}




// Funkce vyrenderuje scenu Scene na zaklade pohledovych informaci z Params, vysledek se uklada do parametru 
// Picture. ten musi byt jiz pred volanim teto funkce pripraven (pro dane 
// rozliseni) napr. funkci CreateRenderedPicture(). Pokud vse uspeje, vraci se true, jinak false.
bool RenderWholeScene(const RenderScene &Scene, const ViewParams &Params, RenderedPicture *pPicture)
{
	RenderedPicture &Picture = *pPicture;

	// Promenne
	unsigned int	Index;
	RenderInfo		Info;
	TrianLinesInfo	TrianInfo;
	int				LineY, LineX, LineStartX, LineEndX;
	Triangle3D		HelpTrian3D;
	StraightLine	Line;
	pPixelInfoElem	PixelInfoPtr;		
	unsigned int	ComparedTriangleIndex;
	bool			ComparedTriangleIsDeeper;	
	double			ScalarMult1, ScalarMult2, HelpVal;
	signed short	PointSizeDiv2;
	Point2D			UpperLeft, BottomRight, Pixel, Point1, Point2;
	Point3D			HelpPoint;


	// Zakladni kontrola parametru
	if(!Picture.Pixels || (Picture.Resolution.X<=0) || (Picture.Resolution.Y<=0))
		return( false );

	if( (Params.ResX!=Picture.Resolution.X) && (Params.ResY!=Picture.Resolution.Y) )
		return( false );

	// Inicializace (vynulovani) informaci o pixelech 
	memset(Picture.PixelInfo, 0, Picture.Resolution.X*Picture.Resolution.Y*sizeof(Picture.PixelInfo[0]));


	// "Prelozime" parametry pohledu do vnitrniho (rychlejsiho) formatu
	if( !CreateRenderInfo(&Info, Params) )
		return( false );

	// Podle tohoto paprsku (primky) vypocitavame hloubku objektu, tyto jeho slozky se v cyklech nemeni
	Line.Point.X = Line.Point.Y = Line.Point.Z = 0;
	Line.Vector.Z = Params.ViewPlaneDistance; 

	if( !Params.DontDrawTriangles && !Params.DrawTrianBorders)
	{
		// Rendering vsech trojuhelniku (textury se v teto funkci nemapuji !)
		for(Index=0; Index<Scene.NumOfTriangles; Index++)
		{
			// Polohu vyrenderovaneho trojuhelniku ulozime do Scene.Triangles[].RenderedPosition,
			// jeho zakladni polohu ukladame do HelpTrian3D (to je poloha ziskana inverzni transformaci-viz fce RenderPoint)
			if(    !RenderPoint(&Scene.Triangles[Index].RenderedPosition.A,
			                    &HelpTrian3D.A,
			                    Scene.Points[Scene.Triangles[Index].PointAIndex].Position, 
			                    Info)
			    || !RenderPoint(&Scene.Triangles[Index].RenderedPosition.B,
			                    &HelpTrian3D.B,
			                    Scene.Points[Scene.Triangles[Index].PointBIndex].Position,
			                    Info)
			    || !RenderPoint(&Scene.Triangles[Index].RenderedPosition.C,
				                &HelpTrian3D.C,
			                    Scene.Points[Scene.Triangles[Index].PointCIndex].Position,
			                    Info)  )
				continue;

			// Pripravime indexovani radek trojuhelnika, pokud je cely mimo obraz, zkoncime a pokracujeme
			// s novym trojuhelnikem
			if( !PrepareTrianLinesIndexing(Scene.Triangles[Index].RenderedPosition, Picture.Resolution, TrianInfo) )
				continue;
			
			// Zjistime rovinu trojuhelnika v zakladni poloze (to je to same jako prenest normalni rovinu 
			// trojuhelnika do zakladni polohy). Delame to pro urychleni vypoctu hloubky pixelu.
			Scene.Triangles[Index].BasicStatePlane = GetPlaneOfTriangle(HelpTrian3D);		

			// Aktualni trojuhelnik jeste nebyl porovnan (na hloubku) s nejakym jinym 
			ComparedTriangleIndex = 0xFFFFFFFF; 	
		
			// Po radkach zaneseme trojuhelnik do Scene.PixelInfo (tak aby neprekryl blizsi trojuhelniky)
			for(LineY=TrianInfo.StartLine; LineY<=TrianInfo.EndLine; LineY++)
			{
				// Zjistime zacatek a konec prislusne radky (cela je uvnitr trojuhelnika)
				if( !GetTrianLineIndexes(TrianInfo, LineY, LineStartX, LineEndX) )
					continue;

				// Ukazatel do pole s informacemi o pixelech, pri pruchodu radkou bude 
				// v kazdem kroku jednoduse inkrementovan
				PixelInfoPtr = Picture.PixelInfo + ( LineY*Picture.Resolution.X + LineStartX );

				// Tato promenna se uvnitr cyklu nemenni - viz dale
				Line.Vector.Y = LineY - Info.ResYDiv2;				

				// Pro kazdy pixel z trojuhelnika na radce LineY...
				for(LineX=LineStartX; LineX<=LineEndX; LineX++)
				{

					// Na tomto pixelu jeste neni nic ?
					if( PixelInfoPtr->Type == PIXELINFO_NOOBJECT )
					{
						PixelInfoPtr->Type = PIXELINFO_TRIANGLE;
						PixelInfoPtr->Index = Index;
						PixelInfoPtr++;						
						continue;
					}
				
					// Na tomto pixelu je jiz neco jineho nec trojuhelnik ?
					if( PixelInfoPtr->Type != PIXELINFO_TRIANGLE )
					{
						// Nic se delat nebude
						PixelInfoPtr++; 
						continue;
					}

					// Je na tomto pixelu porovnany trojuhelnik ? (tj. takovy o kterem vime, 
					// jestli je hloubeji nebo blize nez aktualni trojuhelnik)
					if(PixelInfoPtr->Index == ComparedTriangleIndex)
					{
						if( ComparedTriangleIsDeeper )
							PixelInfoPtr->Index = Index;
						PixelInfoPtr++; 						
						continue;
					}


				
					// Na pixelu je neporovnany trojuhelnik, tj. nevime, jestli je blize nebo hloubeji nez aktualni.
					// Tak ho s aktualnim srovname. Hloubka je v podstate delka usecky z bodu pohledu v zakladni
					// poloze [0,0,0] prochazejici pohledovou rovinou (tj. rovina rovnobezna s osami x a y, posunuta
					// na ose z o Params.ViewPlaneDistance) v miste odpovidajicim prave resenemu pixelu a tato usecka
					// konci v miste pruniku s rovinou trojuhelnika v zakladni poloze (tj. poloha ziskana inverzni 
					// transformaci). Kvuli urychleni nepouzivame funkci GetPenetrationPoint(),
					// protoze o paprscich vime dost veci navic.
	
					Line.Vector.X = LineX - Info.ResXDiv2;
					//Line.Vector.Y = LineY - Info.ResYDiv2; - toto bylo vyhozeno pred telo cyklu									
				
					ScalarMult1 = ScalarMultiply( Scene.Triangles[Index].BasicStatePlane.Normal, 
					                              Line.Vector );	
					ScalarMult2 = ScalarMultiply( Scene.Triangles[PixelInfoPtr->Index].BasicStatePlane.Normal, 
					                              Line.Vector );	
				
					// ScalarMult1 a ScalarMult2 nemohou byt nulove, pokud by byly, trojuhelniky by na tomto
					// pixelu nemohli vubec lezet (jejich rovina by byla rovnobezna s paprskem pohledu)
				
					// Staci porovnat parametry (pro parametrickou rovnici paprsku=primky) urcujici pruniky, 
					// protoze vektor paprsku je pro oba trojuhelniky stejny			
				
					HelpVal = (Scene.Triangles[Index].BasicStatePlane.D / ScalarMult1)
					           - (Scene.Triangles[PixelInfoPtr->Index].BasicStatePlane.D / ScalarMult2);

					// Pokud rozdil parametru (tedy hloubek) je prilis "maly", nemuzeme s jistotou rici, 
					// ktery z trojuhelniku je skutecne blize (paprsek se totiz urcuje podle pixelu, ktery
					// prave zjistujeme), proto neulozime do ComparedTriangleIndex,
					// ze tyto dva trojuhelniky jiz byly porovnany - rozhodnuti odlozime na pozdejc, 
					// kdy rozdil hloubek bude vetsi

					// Pokud je konstanta v nasledujicim porovnani "mala", vypocet je rychlejsi (rozhodnuti,
					// ktery trojuhelnik je hloubeji se provede casteji, casteji ale dochazi i k chybnemu
					// vykresleni), pokud je "velka", vypocet je pomalejsi, zobrazeni ale presnejsi.			

					if( fabs(HelpVal) < 0.0001 )
					{					
						if( HelpVal > 0 )
							PixelInfoPtr->Index = Index;                    
					}
					else				
					{
						// Rozdil hloubek je dostatecny - ulozime si tedy na pozdeji, ktery z trojuhelniku je blize
						ComparedTriangleIndex = PixelInfoPtr->Index;

						if( HelpVal > 0 )
						{
							PixelInfoPtr->Index = Index;
							ComparedTriangleIsDeeper = true;
						}
						else
							ComparedTriangleIsDeeper = false;
					}	
			
					// Posuneme pointer na dalsi element - rychlejsi nez ho pocitat absolutne v kazdem cyklu
					PixelInfoPtr++;

				} // konec for(LineX=...)
			} // for(LineY=...)
		} // Konec cyklu for pro rendering trojuhleniku
	} // konec if( !Params.DontDrawTriangles )



	// Ma se kreslit ohraniceni trojuhelniku
	if( Params.DrawTrianBorders )
	{

		// Rendering vsech trojuhelniku 
		for(Index=0; Index<Scene.NumOfTriangles; Index++)
		{
			// Polohu vyrenderovaneho trojuhelniku ulozime do Scene.Triangles[].RenderedPosition,
			// jeho zakladni polohu ukladame do HelpTrian3D (to je poloha ziskana inverzni transformaci-viz fce RenderPoint)
			if(    !RenderPoint(&Scene.Triangles[Index].RenderedPosition.A,
			                    &HelpTrian3D.A,
			                    Scene.Points[Scene.Triangles[Index].PointAIndex].Position, 
			                    Info)
			    || !RenderPoint(&Scene.Triangles[Index].RenderedPosition.B,
			                    &HelpTrian3D.B,
			                    Scene.Points[Scene.Triangles[Index].PointBIndex].Position,
			                    Info)
			    || !RenderPoint(&Scene.Triangles[Index].RenderedPosition.C,
				                &HelpTrian3D.C,
			                    Scene.Points[Scene.Triangles[Index].PointCIndex].Position,
			                    Info)  )
				continue;

			 // A do Picture.PixelInfo "vykreslime" dane usecky
			LineCalculationState LState;
			LineCalculationProcedure LProc;
			Line2D HelpLine;
			PixelInfoElem Elem;

			Elem.Index = Index;
			Elem.Type = PIXELINFO_TRIANGLEBORDER;

			HelpLine.A = Scene.Triangles[Index].RenderedPosition.A;
			HelpLine.B = Scene.Triangles[Index].RenderedPosition.B;			
			LProc = PrepareLineCalculation(&LState, HelpLine, Picture.Resolution);
			while( LineCalculationContinues(LState) )
			{
				Pixel = LProc( &LState ); // Pri kazdem volani vraci dalsi bod usecky				
				Picture.PixelInfo[ Pixel.Y*Picture.Resolution.X + Pixel.X ] = Elem;
			}

			HelpLine.A = Scene.Triangles[Index].RenderedPosition.B;
			HelpLine.B = Scene.Triangles[Index].RenderedPosition.C;			
			LProc = PrepareLineCalculation(&LState, HelpLine, Picture.Resolution);
			while( LineCalculationContinues(LState) )
			{
				Pixel = LProc( &LState ); // Pri kazdem volani vraci dalsi bod usecky
				Picture.PixelInfo[ Pixel.Y*Picture.Resolution.X + Pixel.X ] = Elem;
			}

			HelpLine.A = Scene.Triangles[Index].RenderedPosition.A;
			HelpLine.B = Scene.Triangles[Index].RenderedPosition.C;			
			LProc = PrepareLineCalculation(&LState, HelpLine, Picture.Resolution);
			while( LineCalculationContinues(LState) )
			{
				Pixel = LProc( &LState ); // Pri kazdem volani vraci dalsi bod usecky
				Picture.PixelInfo[ Pixel.Y*Picture.Resolution.X + Pixel.X ] = Elem;
			}
		}
	}

	// Vyrenderujeme usecky, hloubku vubec neresime
	for(Index=0; Index<Scene.NumOfLines; Index++)
	{
		if( !RenderPoint(&Point1,
		                 &HelpTrian3D.A, // nepotrebujeme
		                 Scene.Points[Scene.Lines[Index].PointsIndexes[0]].Position, 
		                 Info)
		    || !RenderPoint(&Point2,
		                 &HelpTrian3D.B,
		                 Scene.Points[Scene.Lines[Index].PointsIndexes[1]].Position,
		                 Info)	 )
				continue;

		 // A do Picture.PixelInfo "vykreslime" dane usecky
		LineCalculationState LState;
		LineCalculationProcedure LProc;
		Line2D HelpLine;
		PixelInfoElem Elem;

		Elem.Index = Index;
		Elem.Type = PIXELINFO_LINE;

		HelpLine.A = Point1;
		HelpLine.B = Point2;
		LProc = PrepareLineCalculation(&LState, HelpLine, Picture.Resolution);
		while( LineCalculationContinues(LState) )
		{
			Pixel = LProc( &LState ); // Pri kazdem volani vraci dalsi bod usecky				
			Picture.PixelInfo[ Pixel.Y*Picture.Resolution.X + Pixel.X ] = Elem;
		}

	}
	
	
	


	// Rendering bodu
	if( Params.DrawPoints )
	{

		for(Index=0; Index<Scene.NumOfPoints; Index++)
		{
			if( Scene.Points[Index].BillboardSize == 0 )
				continue;

			// Do Pixel ulozime vyslednou renderovanou polohu a do HelpPoint ulozime bod v zakladni poloze.
			if( !RenderPoint(&Pixel, &HelpPoint, Scene.Points[Index].Position, Info) )
				continue;
	
			// Zjistime hloubku tohoto bodu (vzdalenost [0,0,0]-HelpPoint je stejna jako 
			// vzdalenost Params.ViewPoint-Scene.Points[Index].Position, nebot matice zprostredkovavajici
			// inverzni transformaci (tj. prevod do zakladni polohy) definuje LINEARNI zobrazeni)
			Scene.Points[Index].Depth = sqrt(ScalarMultiply(HelpPoint)); // = GetDistance(Params.ViewPoint, Scene.Points[Index].Position);

			// Zjistime, zda se bod ma vykreslovat (jestli ma dostatecne malou hloubku a jestli je v obraze)
			if( (Pixel.X < 0) || (Pixel.X >= Params.ResX) || (Pixel.Y < 0) || (Pixel.Y >= Params.ResY) )
				continue;
		
			// Bod vykreslime jako obdelnik, ktery si nyni pripravime
			PointSizeDiv2 = (Scene.Points[Index].BillboardSize+1)/2;
			UpperLeft.X = Pixel.X - PointSizeDiv2;
			UpperLeft.Y = Pixel.Y + PointSizeDiv2;
			BottomRight.X = UpperLeft.X + Scene.Points[Index].BillboardSize;
			BottomRight.Y = UpperLeft.Y - Scene.Points[Index].BillboardSize;

			PixelInfoElem &PixelInfo = Picture.PixelInfo[ Pixel.Y*Params.ResX + Pixel.X ];

			// Co jiz je na tomto pixelu ?
			switch( PixelInfo.Type )
			{
				case PIXELINFO_POINT:
					// Na tomto pixelu je bod, porovname jejich hloubky
					if(Scene.Points[PixelInfo.Index].Depth >= Scene.Points[Index].Depth)
						SetPointRectToPixelInfo(Picture, UpperLeft, BottomRight, Index);
					break;

				case PIXELINFO_TRIANGLE:
					// Na tomto pixelu je trojuhelnik, zjistime jeho hloubku
					Line.Vector.X = (double)Pixel.X - Info.ResXDiv2;
					Line.Vector.Y = (double)Pixel.Y - Info.ResYDiv2;
			
					// Opet nepouzivame funkci GetPenetrationPoint(), protoze o primce a rovine vime lecos navic = urychleni.
					ScalarMult1 = ScalarMultiply( Scene.Triangles[ PixelInfo.Index ].BasicStatePlane.Normal,
				                              Line.Vector );	
					if( ScalarMult1 == 0 )
						break;
			
					// Porovname hloubku trojuhelniku a bodu, 0.005 pouzijeme ve prospech bodu, protoze zpusob vypoctu hloubky
					// trojuhelnika se ridi pixelem a jeho rovinou v zakladni poloze, vypocet hloubky bodu zavisi pouze na bodu samotnem,
					// a my chceme aby body na trojuhelniku (nebo jeho krajni) byly vykresleny a "neblikaly" pri zmene pohledu.
					if( (-Scene.Triangles[ PixelInfo.Index ].BasicStatePlane.D * sqrt(ScalarMultiply(Line.Vector)) / ScalarMult1 )
						>= (Scene.Points[Index].Depth - 0.005) )
						SetPointRectToPixelInfo(Picture, UpperLeft, BottomRight, Index);
			
					break;

				default:
				//case PIXELINFO_NOOBJECT:
					// Zadny objekt tu neni, bod jednoduse vykreslime (pokud je nektera jeho cast
					// v pozadi jineho objektu - prekresli se)
					SetPointRectToPixelInfo(Picture, UpperLeft, BottomRight, Index);
					break;
			}
		} 

	} // if( Params.DrawPoints )		
	

	// A vybarvime vyrenderovanou scenu
	DyeRenderedScene(Scene, Picture, Params.BgColor & 0x00ffffff);

	return( true );

}


// Funkce ulozi obrazek Picture do souboru Filename ve formatu BMP ve 24-bitove barevne kvalite,
// pokud se to povede, vraci se true, jinak false
bool WriteBitmap(const char *Filename, const RenderedPicture &Picture)
{
	MYBITMAPFILEHEADER FileHeader;
	MYBITMAPINFOHEADER InfoHeader;
	FILE *File;
	long LineSize, Y;
	unsigned char *WriteBuffer, *WriteBufferPtr, *ReadPtr, *MaxReadPtr;

	File = fopen(Filename, "wb");
	if( !File )
		return( false );

	// Ulozime informace do hlavicek
	FileHeader.bfType = 19778; // 'BM'
	FileHeader.bfSize = 10;
	FileHeader.bfReserved1 = 0;
	FileHeader.bfReserved2 = 54;
	FileHeader.bfOffBits = 3435921408;

	InfoHeader.biBitCount = 24;
	InfoHeader.biClrImportant = 0;
	InfoHeader.biClrUsed = 0;
	InfoHeader.biCompression = 0;
	InfoHeader.biHeight = Picture.Resolution.Y;
	InfoHeader.biPlanes = 1;
	InfoHeader.biSize = SIZEOF_MYBITMAPINFOHEADER;
	InfoHeader.biSizeImage = Picture.Resolution.Y * Picture.Resolution.X * 3;
	InfoHeader.biWidth = Picture.Resolution.X;
	InfoHeader.biXPelsPerMeter = 10000;
	InfoHeader.biYPelsPerMeter = 10000;

	// Ulozime hlavicky
	if( fwrite(&FileHeader, SIZEOF_MYBITMAPFILEHEADER, 1, File) != 1)
	{
		fclose(File);
		return( false );
	}
	if( fwrite(&InfoHeader, SIZEOF_MYBITMAPINFOHEADER, 1, File) != 1) 
	{
		fclose(File);
		return( false );
	}

	// Radky jsou v BMP souboru zarovnany na ctyri byty
	if( Picture.Resolution.X*3 % 4 )
		LineSize = Picture.Resolution.X*3 + 4 - (Picture.Resolution.X*3 % 4);
	else
		LineSize = Picture.Resolution.X*3;

	// Alokacu zapisovaciho bufferu
	WriteBuffer = (unsigned char *)malloc( LineSize );
	if( !WriteBuffer )
	{
		fclose(File);
		return( false );
	}

	// Konce radku v BMP souboru zarovname nulami na 4-bytovou hranici
	WriteBuffer[ LineSize - 1 ] = 0;
	WriteBuffer[ LineSize - 2 ] = 0;
	WriteBuffer[ LineSize - 3 ] = 0;
	
	// Pro kazdou radku...
	for( Y=0; Y<Picture.Resolution.Y; Y++ )
	{
		// Radku prelozime do BMP formatu
		WriteBufferPtr = WriteBuffer;
		ReadPtr = (unsigned char *)( Picture.Pixels + Y*Picture.Resolution.X );
		MaxReadPtr = (unsigned char *)( Picture.Pixels + (Y+1)*Picture.Resolution.X - 1 );
		for( ; ReadPtr<=MaxReadPtr; ReadPtr += 4 )
		{
			WriteBufferPtr[0] = ReadPtr[0];
			WriteBufferPtr[1] = ReadPtr[1];
			WriteBufferPtr[2] = ReadPtr[2];
			WriteBufferPtr += 3;
		}

		// a zapiseme ji do souboru
		if( fwrite(WriteBuffer, LineSize, 1, File) != 1 )
		{
			// Nastala chyba		
			free(WriteBuffer);			
			fclose(File);			
			return( false );
		}	
	}

	free(WriteBuffer);
	fclose(File);
	return( true );
}







